# Subcycle Floquet Code

The `subcycle-floquet-code` package contains the source code that has been used to simulated the trARPES spectra in the main paper [main paper](link to be inserted).

The programs are written in modern Fortran, while interface and plotting scripts are written in python.

## Directory structure ##

The repository includes the following directories:

### code ###

This is where the source code and the python driver scipts that generate input for the Fortran programs.

### utils ###

Contains some useful python modules for defining the Hamiltonian and plotting the spectra.

### driver_scipts ###

Contains the job scripts used to produce the results presented in the paper. 

## Installation of the code ##

### Dependencies ###

The Fortran code depends on the following software packages

* [CMake](https://cmake.org)
* [HDF5](https://www.hdfgroup.org), including a Fortran interface
* A version of BLAS/LAPACK
* MPI and a MPI-enabled Fortran compiler.
* For the interface and plotting scripts, we use `numpy`, `matplotlib`. For reading `hdf5` output, `h5py` is required.

### Compilation ###

For compiling the Fortran code, navigate to the `code` directory. There you will find the directory `ConfigExamples` that contains
examples on how to configure the installation. Copy one the files to the code directory:

	# from .../code/
	cp ConfigExamples/ConfigCache_intel.inc Config.inc

Open and modify the `Config.inc` file according to your system. In particular, you need to specify the library path where BLAS/LAPACK is installed
as `lapack_path`, the name of your library containing BLAS/LAPACK as `lapackblas_library`, and the installation path of HDF5: `hdf5_prefix`. Optionally add
Fortran compiler flags as needed.

As the next step, create a directory for compiling and configure the installation:

	mkdir build
	cd build
	FC=mpif90 cmake -C ../Config.inc ../

Here we assumed the Fortran compiler is `mpif90`. If the configuration was successful, compile the code with

	make

If the compilation was successful, the `code` directory now contains the directory `exe` containing the executables.

## Running the simulations ##

Once the code has been compiled, navigate to the `driver_scripts` directory. Here you will find the following python scripts:

| Script                             | Purpose                                                                |
|------------------------------------|------------------------------------------------------------------------|
| `bite_kp_snapshots_prep.py`        | Generates input files for the noninteracting k.p simulation.           |
| `bite_kp_snapshots_obe.py`         | Runs the simulation for the noninteracting k.p model.                  |
| `threeband_gkba_2bloc_prep.py`     | Generates input files for the interacting three-band simulation.       |
| `threeband_2bloc_bath_spectrum.py` | Runs the simulation for the interacting three-band model.              |
| `plot_spectra_free.py`             | Plots the spectra obtained from the noninteracting k.p simulation.     |
| `plot_spectra_3bnd.py`             | Plots the spectra obtained from the interacting three-band simulation. |

To run the simulation of the noninteracting k.p model, run
	python3 bite_kp_snapshots_prep.py

You can also open the script and modify the input, trying different parameters such as the field strength of duration of the probe pulse. The simlation is run using the script `bite_kp_snapshots_obe.py`, which is organized as a job script for the [SLURM](https://slurm.schedmd.com/documentation.html) workload manager as run on the [Merlin6 cluster](https://www.psi.ch/en/awi/the-merlin-hpc-cluster) at the Paul Scherrer Institute. You will need to modify the SLURM header according to your HPC system. Assuming SLURM as the workload manager, the simulation is launched by

	sbatch bite_kp_snapshots_obe.py

Alternatively, you can also run the driver script directly on your machine. Remove the SLURM header (everything above the python import statements) and define the variables `taskid`, `nthreads`, and `mpicmd` in accordance with your system. 
For instance, you could loop over taskid, set `nthreads=1`, and `mpicmd = mpirun -np 2`. Then run the calculation by

	python3 bite_kp_snapshots_obe.py


The procedure for running the similations of the interacting three-band model is analogous. Run 

	python3 threeband_gkba_2bloc_prep.py

to generate the input files for the simulation. Then launch the calculations by

	sbatch threeband_gkba_2bloc_prep.py

Note that the simulation of the interacting model requires more memory and computing time. 

After finishing the calculation for either model, you will find the output files in the `out/` directory. You can plot the trARPES spectra directly by using the plotting script `plot_spectra_free.py` and `plot_spectra_3bnd.py`, respectively. Modify the input variables to be consistent with the simulation parameters and run

	python3 plot_spectra_free.py

to directly plot spectra.

All results in the paper are obtained from the scripts in this repository upon slight modification to scan over different input variables.



