# ----------------------------------------------------------------------
import sys
import os
import numpy as np
import f90nml
# ----------------------------------------------------------------------
class Hamk_2Bloc_Bath():
    #========================================
    def __init__(self,PathExe,PathInp='./inp/',PathOut='./out/',PathLog='./log/'):
        self.exe = PathExe + '/exe/hamk_2bloc_bath_mpi.x'
        self.PathInp = PathInp
        self.PathOut = PathOut
        self.PathLog = PathLog
        self.timeparams = {
            'Nt': 1,
            'Tmax': 1.0
            }

        self.runparams = {
            'AdiabaticSwitch': False,
            'CalcSpectrum': False
        }

        self.kbeparams = {
            'ImplicitOrder': 6,
            'GregoryOrder': 6,
            'NeqIter': 1
        }
    #========================================
    def SetSystem(self,Beta,Mu,file_ham,file_pmom,file_field='',file_bath='',file_matel='',\
        file_tmat=''):
        self.system = {
            'MuChem': Mu,
            'Beta': Beta,
            'file_ham': file_ham,
            'file_pmom': file_pmom,
            'file_field': file_field,
            'file_bath': file_bath,
            'file_matel': file_matel,
            'file_tmat': file_tmat
            }
    #========================================
    def SetTimestepping(self,Tmax,Nt,TStartSwitch,TEndSwitch,TauSwitch,Tdeph=None,\
        dephasing_on=False):
        self.timeparams = {
            'Nt': Nt,
            'Tmax': Tmax,
            'TStartSwitch': TStartSwitch,
            'TEndSwitch': TEndSwitch,
            'TauSwitch': TauSwitch,
            'dephasing_on': dephasing_on
            }

        if Tdeph != None:
            self.timeparams['Tdeph'] = Tdeph
    #========================================
    def SetRunParams(self,AdiabaticSwitch=None,CalcSpectrum=None):

        if AdiabaticSwitch != None:
            self.runparams['AdiabaticSwitch'] = AdiabaticSwitch
        if CalcSpectrum != None:
            self.runparams['CalcSpectrum'] = CalcSpectrum
    #========================================
    def SetProbe(self,Nomega,omega_min,omega_max,probe_dur,probe_t0,\
        lape=False,kperp=0.0,pulse_shape=0,diag_matel=False):
        self.probeparams = {
            'lape': lape,
            'kperp': kperp,
            'Nomega': Nomega,
            'omega_min': omega_min,
            'omega_max': omega_max,
            'num_probe_t0': len(probe_t0),
            'probe_dur': probe_dur,
            'pulse_shape': pulse_shape,
            'diag_matel': diag_matel
            }

        self.probetimes = {'probe_t0': list(probe_t0)}

    #========================================
    def WriteInput(self,file_inp):
        inp = {
            'SYSPARAMS': self.system,
            'TIMEPARAMS': self.timeparams,
            'RUNPARAMS': self.runparams,
            'KBEPARAMS': self.kbeparams
            }

        if self.runparams['CalcSpectrum']:
            inp['PROBEPARAMS'] = self.probeparams
            inp['PROBETIMES'] = self.probetimes

        with open(file_inp, 'w') as nml_file:
            f90nml.write(inp, nml_file)
    #========================================
    def Run(self,mpicmd,prefix,debug_mode=False):
        Nt = self.timeparams['Nt']
        tmax = self.timeparams['Tmax']
        file_pref = prefix + '_Nt{}_tmax{}'.format(Nt,tmax)

        file_inp = self.PathInp + file_pref + '.inp'
        file_out = self.PathOut + file_pref
        file_log = self.PathLog + file_pref + '.log'

        self.WriteInput(file_inp)

        
        if debug_mode:
            os.system(mpicmd + ' ' + self.exe + ' ' + file_inp + ' ' + file_out)
        else:
            os.system(mpicmd + ' ' + self.exe + ' ' + file_inp + ' ' + file_out + ' > ' + file_log)
    #========================================
# ----------------------------------------------------------------------
