# ----------------------------------------------------------------------
import sys
import os
import numpy as np
import f90nml
# ----------------------------------------------------------------------
class Ham_TDARPES():
    #========================================
    def __init__(self,PathExe,PathInp='./inp/',PathOut='./out/',PathLog='./log/'):
        self.exe = PathExe + '/exe/hamk_free_tdarpes.x'
        self.PathInp = PathInp
        self.PathOut = PathOut
        self.PathLog = PathLog
        self.timeparams = {
            'Nt': 1,
            'Tmax': 1.0
            }
        self.probeparams = {
            'lape': False,
            'Nomega': 2,
            'omega_min': 0.0,
            'omega_max': 1.0,
            'num_probe_t0': 1
        }

        self.probetimes = {
            'probe_t0': [0.0]
        }
    #========================================
    def SetSystem(self,Beta,Mu,screen_surf,screen_pe,file_ham,file_pmom,file_field='',\
        T1=1.0e8,T2=1.0e8,file_matel=''):
        self.system = {
            'MuChem': Mu,
            'Beta': Beta,
            'screen_surf': list(screen_surf),
            'screen_pe': list(screen_pe),
            'file_ham': file_ham,
            'file_pmom': file_pmom,
            'file_field': file_field,
            'file_matel': file_matel,
            'T1': T1,
            'T2': T2
            }
    #========================================
    def SetTimestepping(self,Tmax,Nt,evol=0,store_obs=False,store_spect=False,\
        output_every=10):
        self.timeparams = {
            'Nt': Nt,
            'Tmax': Tmax,
            'evol': evol,
            'store_spect': store_spect,
            'store_obs': store_obs,
            'output_every': output_every
            }
    #========================================
    def SetProbe(self,Nomega,omega_min,omega_max,probe_dur,probe_t0,\
        lape=False,kperp=0.0,pulse_shape=0,diag_matel=True):
        self.probeparams = {
            'lape': lape,
            'kperp': kperp,
            'Nomega': Nomega,
            'omega_min': omega_min,
            'omega_max': omega_max,
            'num_probe_t0': len(probe_t0),
            'probe_dur': probe_dur,
            'pulse_shape': pulse_shape,
            'diag_matel': diag_matel
            }

        self.probetimes = {'probe_t0': list(probe_t0)}

    #========================================
    def WriteInput(self,file_inp):
        inp = {
            'SYSPARAMS': self.system,
            'TIMEPARAMS': self.timeparams,
            'PROBEPARAMS': self.probeparams,
            'PROBETIMES': self.probetimes
            }
        with open(file_inp, 'w') as nml_file:
            f90nml.write(inp, nml_file)
    #========================================
    def Run(self,mpicmd,prefix,debug_mode=False):
        Nt = self.timeparams['Nt']
        tmax = self.timeparams['Tmax']
        file_pref = prefix + '_Nt{}_tmax{}'.format(Nt,tmax)

        file_inp = self.PathInp + file_pref + '.inp'
        file_out = self.PathOut + file_pref
        file_log = self.PathLog + file_pref + '.log'

        self.WriteInput(file_inp)

        if debug_mode:
            os.system(mpicmd + ' ' + self.exe + ' ' + file_inp + ' ' + file_out)
        else:
            os.system(mpicmd + ' ' + self.exe + ' ' + file_inp + ' ' + file_out + ' > ' + file_log)
    #========================================
# ----------------------------------------------------------------------
