module Mtdarpes_gkba
!======================================================================================
   use Mdebug
   use Mdef,only: dp,iu,one,zero
   use Mgrid1d,only: grid1d_t
   use Mgregory,only: GregoryCumIntegrate
   !-----------------------
   !    KBKIT modules
   !-----------------------
   use kbkit_greenfun
   use kbkit_gkba
   !-----------------------
   implicit none
!--------------------------------------------------------------------------------------
   integer,parameter :: pulse_sin2=0,pulse_gauss=1,pulse_sech2=2,pulse_lorentz=3
   real(dp),parameter :: pi_ = 3.1415926535897932384626433832795_dp
!--------------------------------------------------------------------------------------
   private
   public &
      SaveSpectrumToHDF5,&
      ProbeSpectrum,&
      ProbeSpectrum_MatrixElements,&      
      ProbeSpectrum_PumpPulse,&
      ProbeSpectrum_PumpPulse_MatrixElements
!--------------------------------------------------------------------------------------
   interface SaveSpectrumToHDF5
      module procedure SaveSpectrumToHDF5_kgrid,SaveSpectrumToHDF5_kpath
   end interface SaveSpectrumToHDF5

   interface ProbeSpectrum
      module procedure ProbeSpectrum_Ham,ProbeSpectrum_gkba
   end interface ProbeSpectrum

   interface ProbeSpectrum_MatrixElements
      module procedure ProbeSpectrum_MatrixElements_Ham,&
         ProbeSpectrum_MatrixElements_GKBA
   end interface ProbeSpectrum_MatrixElements

   interface ProbeSpectrum_PumpPulse
      module procedure ProbeSpectrum_PumpPulse_Ham,&
         ProbeSpectrum_PumpPulse_gkba
   end interface ProbeSpectrum_PumpPulse

   interface ProbeSpectrum_PumpPulse_MatrixElements
      module procedure ProbeSpectrum_PumpPulse_MatrixElements_Ham,&
       ProbeSpectrum_PumpPulse_MatrixElements_gkba
   end interface ProbeSpectrum_PumpPulse_MatrixElements
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
   subroutine GetVPhasefac(nmax,dt,Ak,phi)
      integer,intent(in)     :: nmax
      real(dp),intent(in)    :: dt
      real(dp),intent(in)    :: Ak(0:)
      real(dp),intent(inout) :: phi(0:,0:)
      real(dp),allocatable :: wg(:,:)
      integer :: n,j
      real(dp),allocatable :: phi_a(:)

      allocate(phi_a(0:nmax))
      phi_a = GregoryCumIntegrate(nmax,dt,Ak)
      do n=0,nmax
         do j=0,nmax
            phi(n,j) = phi_a(n) - phi_a(j)
         end do
      end do

      deallocate(phi_a)

   end subroutine GetVPhasefac
!--------------------------------------------------------------------------------------
   subroutine SaveSpectrumToHDF5_kgrid(flname,Nprobe,probe_ts,Nomega,omega_min,omega_max,Nk1,Nk2,spectrum)
      use Mhdf5_utils
      character(len=*),intent(in) :: Flname
      integer,intent(in)  :: Nprobe,Nomega,Nk1,Nk2
      real(dp),intent(in) :: omega_min,omega_max
      real(dp),intent(in) :: probe_ts(:)
      real(dp),intent(in) :: spectrum(0:,:,:)
      integer(HID_t) :: file_id

      call hdf_open_file(file_id, trim(Flname), STATUS='NEW')
      call hdf_write_attribute(file_id,'','nprobe', Nprobe)
      call hdf_write_attribute(file_id,'','nomega', Nomega)
      call hdf_write_attribute(file_id,'','omega_min', omega_min)
      call hdf_write_attribute(file_id,'','omega_max', omega_max)
      call hdf_write_attribute(file_id,'','nk1', Nk1) 
      call hdf_write_attribute(file_id,'','nk2', Nk2)

      call hdf_write_dataset(file_id,'probe_ts',probe_ts)
      call hdf_write_dataset(file_id,'spectrum',spectrum)

      call hdf_close_file(file_id)
   end subroutine SaveSpectrumToHDF5_kgrid
!--------------------------------------------------------------------------------------
   subroutine SaveSpectrumToHDF5_kpath(flname,Nprobe,probe_ts,Nomega,omega_min,omega_max,Nk,spectrum)
      use Mhdf5_utils
      character(len=*),intent(in) :: Flname
      integer,intent(in)  :: Nprobe,Nomega,Nk
      real(dp),intent(in) :: omega_min,omega_max
      real(dp),intent(in) :: probe_ts(:)
      real(dp),intent(in) :: spectrum(0:,:,:)
      integer(HID_t) :: file_id

      call hdf_open_file(file_id, trim(Flname), STATUS='NEW')
      call hdf_write_attribute(file_id,'','nprobe', Nprobe)
      call hdf_write_attribute(file_id,'','nomega', Nomega)
      call hdf_write_attribute(file_id,'','omega_min', omega_min)
      call hdf_write_attribute(file_id,'','omega_max', omega_max)
      call hdf_write_attribute(file_id,'','nk', Nk) 

      call hdf_write_dataset(file_id,'probe_ts',probe_ts)
      call hdf_write_dataset(file_id,'spectrum',spectrum)

      call hdf_close_file(file_id)
   end subroutine SaveSpectrumToHDF5_kpath
!--------------------------------------------------------------------------------------
   pure elemental real(dp) function PulseSin2(Tdur,t)
#ifdef OMPSIMD
!$OMP DECLARE SIMD(PulseSin2) UNIFORM(Tdur)
#endif    
      real(dp),intent(in)::Tdur,t


   PulseSin2=sin(pi_*t/Tdur)**2 / Tdur

   end function PulseSin2
!--------------------------------------------------------------------------------------
   pure elemental real(dp) function box_win(t1,t2,t)
#ifdef OMPSIMD
!$OMP DECLARE SIMD(box_win) UNIFORM(t1,t2)
#endif    
      real(dp),intent(in) :: t1,t2
      real(dp),intent(in) :: t
      real(dp) :: tx,tq,phi

      tx = t - t1
      tq = t2 - t1
      phi = pi_ * tx/tq 

      box_win = 0.0_dp
      if((tx > 0.0_dp) .and. (tx < tq)) then
         box_win = 0.5_dp - 0.75_dp * cos(phi) + 0.25_dp * cos(phi)**3
      elseif(tx >= tq) then
         box_win = 1.0_dp
      end if

   end function box_win
!--------------------------------------------------------------------------------------
   pure elemental real(dp) function gauss_mod(Tdur,t)
#ifdef OMPSIMD
!$OMP DECLARE SIMD(gauss_mod) UNIFORM(Tdur)
#endif    
      real(dp),intent(in) :: Tdur
      real(dp),intent(in) :: t
      real(dp) :: r1,r2,tm

      tm = 0.5_dp * Tdur

      r1 = box_win(0.0_dp, Tdur/6.0_dp, t)
      r2 = 1.0_dp - box_win(5.0_dp/6.0_dp*Tdur, Tdur, t)

      gauss_mod = r1 * r2 * exp(-0.5_dp*(6.0_dp*(t-tm)/Tdur)**2)

   end function gauss_mod
!--------------------------------------------------------------------------------------
   pure elemental real(dp) function sech2_mod(Tdur,t)
#ifdef OMPSIMD
!$OMP DECLARE SIMD(sech2_mod) UNIFORM(Tdur)
#endif    
      real(dp),intent(in) :: Tdur
      real(dp),intent(in) :: t
      real(dp) :: r1,r2,tm

      tm = 0.5_dp * Tdur

      r1 = box_win(0.0_dp, Tdur/8.0_dp, t)
      r2 = 1.0_dp - box_win(7.0_dp/8.0_dp*Tdur, Tdur, t)

      sech2_mod = r1 * r2 / cosh(8.0_dp*(t-tm)/Tdur)**2

   end function sech2_mod
!--------------------------------------------------------------------------------------
   pure elemental real(dp) function lorentz_mod(Tdur,t)
#ifdef OMPSIMD
!$OMP DECLARE SIMD(lorentz_mod) UNIFORM(Tdur)
#endif    
      real(dp),intent(in) :: Tdur
      real(dp),intent(in) :: t
      real(dp) :: r1,r2,tm,x

      tm = 0.5_dp * Tdur

      r1 = box_win(0.0_dp, 0.4_dp * Tdur, t)
      r2 = 1.0_dp - box_win(0.6_dp * Tdur, Tdur, t)

      x = 10.0_dp * (t - tm)/Tdur
      lorentz_mod = r1 * r2 / (1.0_dp + x**2)

   end function lorentz_mod
!--------------------------------------------------------------------------------------
   pure elemental real(dp) function probe_pulse(Tdur,pulse_shape,t)
#ifdef OMPSIMD
!$OMP DECLARE SIMD(probe_pulse) UNIFORM(Tdur,pulse_shape)
#endif    
      real(dp),intent(in) :: Tdur
      integer,intent(in)  :: pulse_shape
      real(dp),intent(in) :: t

      select case(pulse_shape)
      case(pulse_sin2)
         probe_pulse = PulseSin2(Tdur,t)
      case(pulse_gauss)
         probe_pulse = gauss_mod(Tdur,t)
      case(pulse_sech2)
         probe_pulse = sech2_mod(Tdur,t)
      case(pulse_lorentz)
         probe_pulse = lorentz_mod(Tdur,t)
      end select   
   end function probe_pulse    
!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_Ham(tgrd,wgrd,Tdur,t0,H,Rho,spect,pulse_shape)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(CntrFun_1time_t),intent(in) :: Rho
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen
      real(dp) :: tmax

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates

      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            H%ret,Rho%ret,n_min,nlen,pulse_shape_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

   end subroutine ProbeSpectrum_Ham
!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_gkba(tgrd,wgrd,Tdur,t0,H,gkba,spect,pulse_shape)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(gkba_t),intent(in)          :: gkba
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen
      real(dp) :: tmax

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates
      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            H%ret,gkba%Rho,n_min,nlen,pulse_shape_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

   end subroutine ProbeSpectrum_gkba
!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_MatrixElements_Ham(tgrd,wgrd,Tdur,t0,H,Rho,Matel,spect,pulse_shape,&
      diag_matel)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(CntrFun_1time_t),intent(in) :: Rho
      complex(dp),intent(in)           :: MatEl(:,0:)
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      logical,intent(in),optional      :: diag_matel
      logical :: diag_matel_
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen
      real(dp) :: tmax

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      diag_matel_ = .false.
      if(present(diag_matel)) diag_matel_ = diag_matel

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates

      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_Matel_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            matel(:,iw),H%ret,Rho%ret,n_min,nlen,pulse_shape_,diag_matel_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

   end subroutine ProbeSpectrum_MatrixElements_Ham
!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_MatrixElements_gkba(tgrd,wgrd,Tdur,t0,H,gkba,Matel,spect,pulse_shape,&
      diag_matel)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(gkba_t),intent(in)          :: gkba
      complex(dp),intent(in)           :: MatEl(:,0:)
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      logical,intent(in),optional      :: diag_matel
      logical :: diag_matel_
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen
      real(dp) :: tmax

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      diag_matel_ = .false.
      if(present(diag_matel)) diag_matel_ = diag_matel

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates

      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_Matel_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            matel(:,iw),H%ret,gkba%Rho,n_min,nlen,pulse_shape_,diag_matel_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

   end subroutine ProbeSpectrum_MatrixElements_gkba
!--------------------------------------------------------------------------------------


!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_PumpPulse_Ham(tgrd,wgrd,Tdur,t0,H,Rho,kvec,Avec,spect,pulse_shape)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(CntrFun_1time_t),intent(in) :: Rho
      real(dp),intent(in)              :: kvec(:)
      real(dp),intent(in)              :: Avec(:,0:)
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen,n,ndim,idir
      real(dp) :: tmax
      real(dp),allocatable :: Avec_k(:)

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates

      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      allocate(Avec_k(0:tgrd%Npts))
      Avec_k = 0.0_dp
      ndim = size(kvec)
      do idir=1,ndim
         do n=0,tgrd%Npts
            Avec_k(n) = Avec_k(n) - kvec(idir)*Avec(idir,n) + 0.5_dp*Avec(idir,n)**2
         end do
      end do

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_Phase_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            H%ret,Rho%ret,Avec_k,n_min,nlen,pulse_shape_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

      deallocate(Avec_k)

   end subroutine ProbeSpectrum_PumpPulse_Ham
!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_PumpPulse_gkba(tgrd,wgrd,Tdur,t0,H,gkba,kvec,Avec,spect,pulse_shape)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(gkba_t),intent(in)          :: gkba
      real(dp),intent(in)              :: kvec(:)
      real(dp),intent(in)              :: Avec(:,0:)
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen,n,ndim,idir
      real(dp) :: tmax
      real(dp),allocatable :: Avec_k(:)

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates

      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      allocate(Avec_k(0:tgrd%Npts))
      Avec_k = 0.0_dp
      ndim = size(kvec)
      do idir=1,ndim
         do n=0,tgrd%Npts
            Avec_k(n) = Avec_k(n) - kvec(idir)*Avec(idir,n) + 0.5_dp*Avec(idir,n)**2
         end do
      end do

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_Phase_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            H%ret,gkba%Rho,Avec_k,n_min,nlen,pulse_shape_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

      deallocate(Avec_k)

   end subroutine ProbeSpectrum_PumpPulse_gkba
!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_PumpPulse_MatrixElements_Ham(tgrd,wgrd,Tdur,t0,H,Rho,kvec,Avec,&
      Matel,spect,pulse_shape,diag_matel)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(CntrFun_1time_t),intent(in) :: Rho
      real(dp),intent(in)              :: kvec(:)
      real(dp),intent(in)              :: Avec(:,0:)
      complex(dp),intent(in)           :: MatEl(:,0:)
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      logical,intent(in),optional      :: diag_matel
      logical :: diag_matel_
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen,n,ndim,idir
      real(dp) :: tmax
      real(dp),allocatable :: Avec_k(:)

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      diag_matel_ = .false.
      if(present(diag_matel)) diag_matel_ = diag_matel

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates

      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      allocate(Avec_k(0:tgrd%Npts))
      Avec_k = 0.0_dp
      ndim = size(kvec)
      do idir=1,ndim
         do n=0,tgrd%Npts
            Avec_k(n) = Avec_k(n) - kvec(idir)*Avec(idir,n) + 0.5_dp*Avec(idir,n)**2
         end do
      end do

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_Matel_Phase_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            matel(:,iw),H%ret,Rho%ret,Avec_k,n_min,nlen,pulse_shape_,diag_matel_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

      deallocate(Avec_k)

   end subroutine ProbeSpectrum_PumpPulse_MatrixElements_Ham
!--------------------------------------------------------------------------------------
   subroutine ProbeSpectrum_PumpPulse_MatrixElements_gkba(tgrd,wgrd,Tdur,t0,H,gkba,&
      kvec,Avec,Matel,spect,pulse_shape,diag_matel)
      type(grid1d_t),intent(in)        :: tgrd
      type(grid1d_t),intent(in)        :: wgrd
      real(dp),intent(in)              :: Tdur,t0
      type(CntrFun_1time_t),intent(in) :: H
      type(gkba_t),intent(in)          :: gkba
      real(dp),intent(in)              :: kvec(:)
      real(dp),intent(in)              :: Avec(:,0:)
      complex(dp),intent(in)           :: MatEl(:,0:)
      real(dp),intent(inout)           :: spect(0:)
      integer,intent(in),optional      :: pulse_shape
      logical,intent(in),optional      :: diag_matel
      logical :: diag_matel_
      integer :: pulse_shape_
      integer :: nst,iw,n_min,n_max,nlen,n,ndim,idir
      real(dp) :: tmax
      real(dp),allocatable :: Avec_k(:)

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      diag_matel_ = .false.
      if(present(diag_matel)) diag_matel_ = diag_matel

      tmax = t0 + Tdur
      n_min = max(floor(t0 / tgrd%dX), 0)
      n_max = min(ceiling(tmax / tgrd%dX), tgrd%Npts)
      nlen = n_max - n_min
      nst = H%nstates

      if(nlen < 0) then
         spect = 0.0_dp
         return
      end if

      allocate(Avec_k(0:tgrd%Npts))
      Avec_k = 0.0_dp
      ndim = size(kvec)
      do idir=1,ndim
         do n=0,tgrd%Npts
            Avec_k(n) = Avec_k(n) - kvec(idir)*Avec(idir,n) + 0.5_dp*Avec(idir,n)**2
         end do
      end do

      !$OMP PARALLEL 
      !$OMP DO
      do iw=0,wgrd%Npts
         spect(iw) = PESIntensity_Matel_Phase_RK5(wgrd%Xpts(iw),tgrd%dX,Tdur,t0-n_min*tgrd%dX,&
            matel(:,iw),H%ret,gkba%Rho,Avec_k,n_min,nlen,pulse_shape_,diag_matel_)
      end do
      !$OMP END DO
      !$OMP END PARALLEL

      deallocate(Avec_k)

   end subroutine ProbeSpectrum_PumpPulse_MatrixElements_gkba
!--------------------------------------------------------------------------------------



!--------------------------------------------------------------------------------------
   function PESIntensity_RK5(w,dt,Tdur,t0,H,rho,n_min,nlen,pulse_shape) result(spect)
      use Mrungekutta,only: ODE_step_RK5
      use Mlinalg,only: trace
      real(dp),intent(in)    :: w
      real(dp),intent(in)    :: dt
      real(dp),intent(in)    :: Tdur,t0
      complex(dp),intent(in) :: H(:,:,0:)
      complex(dp),intent(in) :: Rho(:,:,0:)
      integer,intent(in)     :: n_min,nlen
      integer,intent(in)     :: pulse_shape
      real(dp) :: spect
      integer :: nst,j
      complex(dp) :: integ
      complex(dp),allocatable :: yt_kern(:,:),TrYt(:)

      spect = 0.0_dp

      nst = size(H,dim=1)
      allocate(yt_kern(nst,nst)); yt_kern = zero
      allocate(TrYt(0:nlen)); TrYt = zero
      do j=0,nlen-1
         yt_kern = ODE_step_RK5(nst,j,dt,deriv_func,yt_kern)
         TrYt(j+1) = iu * trace(yt_kern)
      end do

      spect = GregoryFreqIntegral(w,dt,Tdur,t0,nlen,TrYt,pulse_shape)
   !................................................
   contains
   !................................................
      function deriv_func(nst,t,y) result(dydt)
         integer,parameter    :: pintp=5
         integer,intent(in)     :: nst
         real(dp),intent(in)    :: t
         complex(dp),intent(in) :: y(:,:)
         complex(dp)            :: dydt(nst,nst)
         integer :: kmin,k
         real(dp) :: s
         real(dp) :: wp(pintp)
         complex(dp),dimension(nst,nst) :: Rhot,Ht

         s = probe_pulse(Tdur,pulse_shape,t)

         call GetPolyWeights(nlen,dt,t,kmin,wp)
         Rhot = zero; Ht = zero
         do k=0,pintp-1
            Rhot = Rhot + wp(k+1) * Rho(:,:,n_min+kmin+k)
            Ht = Ht + wp(k+1) * H(:,:,n_min+kmin+k)
         end do

         dydt = s * exp(-iu * w * t) * Rhot
         dydt = dydt - iu * matmul(Ht,y)

      end function deriv_func
   !................................................

   end function PESIntensity_RK5
!--------------------------------------------------------------------------------------
   function PESIntensity_Matel_RK5(w,dt,Tdur,t0,matel,H,rho,n_min,nlen,pulse_shape,diag_matel) result(spect)
      use Mrungekutta,only: ODE_step_RK5
      use Mlinalg,only: trace
      real(dp),intent(in)    :: w
      real(dp),intent(in)    :: dt
      real(dp),intent(in)    :: Tdur,t0
      complex(dp),intent(in) :: Matel(:)
      complex(dp),intent(in) :: H(:,:,0:)
      complex(dp),intent(in) :: Rho(:,:,0:)
      integer,intent(in)     :: n_min,nlen
      integer,intent(in)     :: pulse_shape
      logical,intent(in)     :: diag_matel
      real(dp) :: spect
      integer :: nst,j,a1,a2
      complex(dp) :: integ
      complex(dp),allocatable :: yt_kern(:,:),TrYt(:)

      spect = 0.0_dp

      nst = size(H,dim=1)
      allocate(yt_kern(nst,nst)); yt_kern = zero
      allocate(TrYt(0:nlen)); TrYt = zero
      do j=0,nlen-1
         yt_kern = ODE_step_RK5(nst,j,dt,deriv_func,yt_kern)
         if(diag_matel) then
            do a1=1,nst
               TrYt(j+1) = TrYt(j+1) + iu * abs(matel(a1))**2 * yt_kern(a1,a1)
            end do
         else
            do a2=1,nst
               do a1=1,nst
                  TrYt(j+1) = TrYt(j+1) + iu * matel(a1) * yt_kern(a1,a2) * conjg(matel(a2))
               end do
            end do
         end if
      end do

      spect = GregoryFreqIntegral(w,dt,Tdur,t0,nlen,TrYt,pulse_shape)
   !................................................
   contains
   !................................................
      function deriv_func(nst,t,y) result(dydt)
         integer,parameter    :: pintp=5
         integer,intent(in)     :: nst
         real(dp),intent(in)    :: t
         complex(dp),intent(in) :: y(:,:)
         complex(dp)            :: dydt(nst,nst)
         integer :: kmin,k
         real(dp) :: s
         real(dp) :: wp(pintp)
         complex(dp),dimension(nst,nst) :: Rhot,Ht

         s = probe_pulse(Tdur,pulse_shape,t)

         call GetPolyWeights(nlen,dt,t,kmin,wp)
         Rhot = zero; Ht = zero
         do k=0,pintp-1
            Rhot = Rhot + wp(k+1) * Rho(:,:,n_min+kmin+k)
            Ht = Ht + wp(k+1) * H(:,:,n_min+kmin+k)
         end do

         dydt = s * exp(-iu * w * t) * Rhot
         dydt = dydt - iu * matmul(Ht,y)

      end function deriv_func
   !................................................

   end function PESIntensity_Matel_RK5
!--------------------------------------------------------------------------------------
   function PESIntensity_Phase_RK5(w,dt,Tdur,t0,H,rho,Ak,n_min,nlen,pulse_shape) result(spect)
      use Mrungekutta,only: ODE_step_RK5
      use Mlinalg,only: trace
      real(dp),intent(in)    :: w
      real(dp),intent(in)    :: dt
      real(dp),intent(in)    :: Tdur,t0
      complex(dp),intent(in) :: H(:,:,0:)
      complex(dp),intent(in) :: Rho(:,:,0:)
      real(dp),intent(in)    :: Ak(0:)
      integer,intent(in)     :: n_min,nlen
      integer,intent(in)     :: pulse_shape
      real(dp) :: spect
      integer :: nst,j
      complex(dp) :: integ
      complex(dp),allocatable :: yt_kern(:,:),TrYt(:)

      spect = 0.0_dp

      nst = size(H,dim=1)
      allocate(yt_kern(nst,nst)); yt_kern = zero
      allocate(TrYt(0:nlen)); TrYt = zero
      do j=0,nlen-1
         yt_kern = ODE_step_RK5(nst,j,dt,deriv_func,yt_kern)
         TrYt(j+1) = iu * trace(yt_kern)
      end do

      spect = GregoryFreqIntegral(w,dt,Tdur,t0,nlen,TrYt,pulse_shape)
   !................................................
   contains
   !................................................
      function deriv_func(nst,t,y) result(dydt)
         integer,parameter    :: pintp=5
         integer,intent(in)     :: nst
         real(dp),intent(in)    :: t
         complex(dp),intent(in) :: y(:,:)
         complex(dp)            :: dydt(nst,nst)
         integer :: kmin,k,i
         real(dp) :: s
         real(dp) :: wp(pintp)
         real(dp) :: Akt
         complex(dp),dimension(nst,nst) :: Rhot,Ht

         s = probe_pulse(Tdur,pulse_shape,t)

         call GetPolyWeights(nlen,dt,t,kmin,wp)
         Rhot = zero; Ht = zero; Akt = 0.0_dp
         do k=0,pintp-1
            Rhot = Rhot + wp(k+1) * Rho(:,:,n_min+kmin+k)
            Ht = Ht + wp(k+1) * H(:,:,n_min+kmin+k)
            Akt = Akt + wp(k+1) * Ak(n_min+kmin+k)
         end do

         do i=1,nst
            Ht(i,i) = Ht(i,i) - Akt
         end do

         dydt = s * exp(-iu * w * t) * Rhot
         dydt = dydt - iu * matmul(Ht,y)

      end function deriv_func
   !................................................

   end function PESIntensity_Phase_RK5
!--------------------------------------------------------------------------------------
   function PESIntensity_Matel_Phase_RK5(w,dt,Tdur,t0,matel,H,rho,Ak,n_min,nlen,&
      pulse_shape,diag_matel) result(spect)
      use Mrungekutta,only: ODE_step_RK5
      use Mlinalg,only: trace
      real(dp),intent(in)    :: w
      real(dp),intent(in)    :: dt
      real(dp),intent(in)    :: Tdur,t0
      complex(dp),intent(in) :: Matel(:)
      complex(dp),intent(in) :: H(:,:,0:)
      complex(dp),intent(in) :: Rho(:,:,0:)
      real(dp),intent(in)    :: Ak(0:)
      integer,intent(in)     :: n_min,nlen
      integer,intent(in)     :: pulse_shape
      logical,intent(in)     :: diag_matel
      real(dp) :: spect
      integer :: nst,j,a1,a2
      complex(dp) :: integ
      complex(dp),allocatable :: yt_kern(:,:),TrYt(:)

      spect = 0.0_dp

      nst = size(H,dim=1)
      allocate(yt_kern(nst,nst)); yt_kern = zero
      allocate(TrYt(0:nlen)); TrYt = zero
      do j=0,nlen-1
         yt_kern = ODE_step_RK5(nst,j,dt,deriv_func,yt_kern)
         if(diag_matel) then
            do a1=1,nst
               TrYt(j+1) = TrYt(j+1) + iu * abs(matel(a1))**2 * yt_kern(a1,a1)
            end do
         else
            do a2=1,nst
               do a1=1,nst
                  TrYt(j+1) = TrYt(j+1) + iu * matel(a1) * yt_kern(a1,a2) * conjg(matel(a2))
               end do
            end do
         end if
      end do

      spect = GregoryFreqIntegral(w,dt,Tdur,t0,nlen,TrYt,pulse_shape)
   !................................................
   contains
   !................................................
      function deriv_func(nst,t,y) result(dydt)
         integer,parameter    :: pintp=5
         integer,intent(in)     :: nst
         real(dp),intent(in)    :: t
         complex(dp),intent(in) :: y(:,:)
         complex(dp)            :: dydt(nst,nst)
         integer :: kmin,k,i
         real(dp) :: s
         real(dp) :: wp(pintp)
         real(dp) :: Akt
         complex(dp),dimension(nst,nst) :: Rhot,Ht

         s = probe_pulse(Tdur,pulse_shape,t)

         call GetPolyWeights(nlen,dt,t,kmin,wp)
         Rhot = zero; Ht = zero; Akt = 0.0_dp
         do k=0,pintp-1
            Rhot = Rhot + wp(k+1) * Rho(:,:,n_min+kmin+k)
            Ht = Ht + wp(k+1) * H(:,:,n_min+kmin+k)
            Akt = Akt + wp(k+1) * Ak(n_min+kmin+k)
         end do

         do i=1,nst
            Ht(i,i) = Ht(i,i) - Akt
         end do

         dydt = s * exp(-iu * w * t) * Rhot
         dydt = dydt - iu * matmul(Ht,y)

      end function deriv_func
   !................................................

   end function PESIntensity_Matel_Phase_RK5
!--------------------------------------------------------------------------------------
   function GregoryFreqIntegral(w,dt,Tdur,t0,nlen,TrYt,pulse_shape) result(spect)
      integer,parameter :: np=5
      real(dp),dimension(6),parameter :: wg=[&
         0.31559193121693124_dp,&
         1.3921792328042324_dp,&
         0.62397486772486765_dp,&
         1.2440806878306878_dp,&
         0.90990410052910042_dp,&
         1.0142691798941799_dp]
      real(dp),intent(in)    :: w
      real(dp),intent(in)    :: dt
      real(dp),intent(in)    :: Tdur,t0
      integer,intent(in)     :: nlen
      complex(dp),intent(in) :: TrYt(0:)
      integer,intent(in)     :: pulse_shape
      real(dp) :: spect
      integer :: j
      real(dp) :: s

      spect = 0.0_dp
      do j=0,np
         s = probe_pulse(Tdur,pulse_shape,j*dt-t0)
         spect = spect + s * wg(j+1) * (sin(w*j*dt) * dble(TrYt(j)) + cos(w*j*dt) * aimag(TrYt(j)) )
      end do
      do j=nlen-np,nlen
         s = probe_pulse(Tdur,pulse_shape,j*dt-t0)
         spect = spect + s * wg(nlen-j+1) * (sin(w*j*dt) * dble(TrYt(j)) + cos(w*j*dt) * aimag(TrYt(j)) )
      end do
      do j=np+1,nlen-np-1
         s = probe_pulse(Tdur,pulse_shape,j*dt-t0)
         spect = spect + s * (sin(w*j*dt) * dble(TrYt(j)) + cos(w*j*dt) * aimag(TrYt(j)) )
      end do

      spect = dt * spect
   end function GregoryFreqIntegral
!--------------------------------------------------------------------------------------
   subroutine GetPolyWeights(Nt,dt,t,kmin,wp)
      integer,parameter    :: pintp=5
      real(dp),parameter   :: tol=1.0e-10_dp
      integer,intent(in)   :: Nt
      real(dp),intent(in)  :: dt
      real(dp),intent(in)  :: t
      integer,intent(out)  :: kmin
      real(dp),intent(out) :: wp(pintp)
      integer  :: imin,imax,kmax,j
      real(dp) :: al
      real(dp) :: xp(pintp)

      al = t/dt
      imin = floor(al); imax = ceiling(al)

      kmin = imin-2; kmax = imax+1
      if(kmin < 0) then
         kmin = 0; kmax = 4
      end if
      if(kmax >= Nt) then
         kmin = Nt-4; kmax = Nt
      end if

      do j=0,pintp-1
         xp(j+1) = (j + kmin) * dt
      end do

      do j=1,pintp
         wp(j) = LagrPolyTerm(pintp-1,xp,j,t)
      end do

   end subroutine GetPolyWeights
!--------------------------------------------------------------------------------------
   pure real(dp) function LagrPolyTerm(Np,xp,j,x)
      !! Returns j-th term of Lagrange polynomial
      integer,intent(in)  :: Np    !! number of points
      real(dp),intent(in) :: xp(:) !! values on the x-axis, dimension(Np+1)
      integer,intent(in)  :: j     !! which term
      real(dp),intent(in) :: x     !! where to evalute polynonial
      integer :: m
      real(dp) :: Lj

      LagrPolyTerm = 1.0_dp
      do m=1,Np+1
         if(m==j) cycle
         LagrPolyTerm = LagrPolyTerm*(x-xp(m))/(xp(j)-xp(m))
      end do

   end function LagrPolyTerm
!--------------------------------------------------------------------------------------

!======================================================================================  
end module Mtdarpes_gkba
