!--------------------------------------------------------------------------------------
  subroutine Init_CntrFun_1time(CntrFun,Beta,Nstates,Nt)
    class(CntrFun_1time_t)::CntrFun
    integer,intent(in)::Nstates,Nt
    real(dp),intent(in)::Beta

    allocate(CntrFun%Ret(Nstates,Nstates,0:Nt))
    allocate(CntrFun%Mats(Nstates,Nstates))

    CntrFun%Ret=zero
    CntrFun%Mats=zero

    CntrFun%Nstates=Nstates
    CntrFun%Nt=Nt
    CntrFun%Beta=Beta

    CntrFun%Initialized=.true.

  end subroutine Init_CntrFun_1time
!--------------------------------------------------------------------------------------
  subroutine Free_CntrFun_1time(CntrFun)
    class(CntrFun_1time_t)::CntrFun

    deallocate(CntrFun%Ret)
    deallocate(CntrFun%Mats)

    CntrFun%Initialized=.false.

  end subroutine Free_CntrFun_1time
  !--------------------------------------------------------------------------------------
  pure subroutine CntrFun_1time_SetZero(CntrFun)
    class(CntrFun_1time_t),intent(inout)::CntrFun

    CntrFun%Ret=zero
    CntrFun%Mats=zero

  end subroutine CntrFun_1time_SetZero
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_1time_SetConst(CntrFun,val)
    class(CntrFun_1time_t),intent(inout)::CntrFun
    complex(dp),intent(in)::val(:,:)
    integer::i,j

    forall(i=1:CntrFun%Nstates,j=1:CntrFun%Nstates) CntrFun%Ret(i,j,0:)=val(i,j)
    CntrFun%Mats=val

  end subroutine CntrFun_1time_SetConst
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_1time_Interpolate(CntrFun,tgrd,t,val)
    integer,parameter :: pintp=5
    real(dp),parameter :: tol=1.0e-12_dp
    class(CntrFun_1time_t),intent(in) :: CntrFun
    type(grid1d_t),intent(in)           :: tgrd
    real(dp),intent(in)               :: t
    complex(dp),intent(out)           :: val(CntrFun%Nstates,CntrFun%Nstates)
    integer  :: imin,imax,kmin,kmax,k
    real(dp) :: al
    real(dp) :: xp(pintp),wp(pintp)

    if(t < tgrd%Xmin) then
       val = zero
       return
    end if

    if(t > tgrd%Xmax) then
       val = zero
       return
    end if

    al = t/tgrd%dX
    imin = floor(al); imax = ceiling(al)
    if(abs(imin - al) < tol) then
      val = CntrFun%Ret(:,:,imin)
      return
    end if

    kmin = imin-2; kmax = imax+1
    if(kmin < 0) then
       kmin = 0; kmax = 4
    end if
    if(kmax > tgrd%Npts) then
       kmin = tgrd%Npts-4; kmax = tgrd%Npts
    end if

    xp = tgrd%Xpts(kmin:kmax)
    forall(k=1:pintp) wp(k) = LagrPolyTerm(pintp-1,xp,k,t)

    val = zero
    do k=0,pintp-1
       val = val + wp(k+1)*CntrFun%Ret(:,:,kmin+k)
    end do

  end subroutine CntrFun_1time_Interpolate
!--------------------------------------------------------------------------------------
  subroutine Init_1time_array(func,N,Beta,nstates,Nt)
    type(CntrFun_1time_t),allocatable,intent(inout) :: func(:)
    integer,intent(in)  :: N
    real(dp),intent(in) :: Beta
    integer,intent(in)  :: nstates
    integer,intent(in)  :: Nt
    integer :: i

    allocate(func(N))
    do i=1,N
       call func(i)%Init(Beta,nstates,Nt)
    end do

  end subroutine Init_1time_array
!--------------------------------------------------------------------------------------
  subroutine Clean_1time_array(func)
    type(CntrFun_1time_t),allocatable,intent(inout) :: func(:)
    integer :: i

    do i=1,size(func)
       call func(i)%Clean()
    end do
    deallocate(func)

  end subroutine Clean_1time_array
!--------------------------------------------------------------------------------------
   pure real(dp) function LagrPolyTerm(Np,xp,j,x)
      !! Returns j-th term of Lagrange polynomial
      integer,intent(in)  :: Np    !! number of points
      real(dp),intent(in) :: xp(:) !! values on the x-axis, dimension(Np+1)
      integer,intent(in)  :: j     !! which term
      real(dp),intent(in) :: x     !! where to evalute polynonial
      integer :: m
      real(dp) :: Lj

      LagrPolyTerm = 1.0_dp
      do m=1,Np+1
         if(m==j) cycle
         LagrPolyTerm = LagrPolyTerm*(x-xp(m))/(xp(j)-xp(m))
      end do

   end function LagrPolyTerm
!--------------------------------------------------------------------------------------


