!--------------------------------------------------------------------------------------
  subroutine Init_CntrFun_TimeSlice(CntrFun,FermBos,Beta,Nstates,Nt,Ntau)
    class(CntrFun_timeslice_t),intent(inout)::CntrFun
    integer,intent(in)::FermBos,Nstates,Nt,Ntau
    real(dp),intent(in)::Beta

    allocate(CntrFun%Lesser(Nstates,Nstates,0:Nt))
    allocate(CntrFun%Greater(Nstates,Nstates,0:Nt))
    allocate(CntrFun%Left(Nstates,Nstates,0:Ntau))
    allocate(CntrFun%Mats(Nstates,Nstates,0:Ntau))

    CntrFun%Lesser=zero
    CntrFun%Greater=zero
    CntrFun%Left=zero
    CntrFun%Mats=zero

    CntrFun%Nstates=Nstates
    CntrFun%Nt=Nt
    CntrFun%Ntau=Ntau
    CntrFun%FermBos=FermBos
    CntrFun%Beta=Beta

    CntrFun%Initialized=.true.

  end subroutine Init_CntrFun_TimeSlice
!--------------------------------------------------------------------------------------
  subroutine Init_CntrFun_TimeSlice_Keld(CntrFun,FermBos,Beta,Nstates,Nt)
    class(CntrFun_timeslice_t)::CntrFun
    integer,intent(in)::FermBos,Nstates,Nt
    real(dp),intent(in)::Beta

    allocate(CntrFun%Lesser(Nstates,Nstates,0:Nt))
    allocate(CntrFun%Greater(Nstates,Nstates,0:Nt))

    CntrFun%Lesser=zero
    CntrFun%Greater=zero

    CntrFun%Nstates=Nstates
    CntrFun%Nt=Nt
    CntrFun%Ntau=0
    CntrFun%FermBos=FermBos
    CntrFun%Beta=Beta

    CntrFun%Initialized=.true.

  end subroutine Init_CntrFun_TimeSlice_Keld
!--------------------------------------------------------------------------------------
  subroutine Free_CntrFun_TimeSlice(CntrFun)
    class(CntrFun_timeslice_t)::CntrFun

    deallocate(CntrFun%Lesser)
    deallocate(CntrFun%Greater)
    if(allocated(CntrFun%Left)) deallocate(CntrFun%Left)
    if(allocated(CntrFun%Mats)) deallocate(CntrFun%Mats)

    CntrFun%Initialized=.false.

  end subroutine Free_CntrFun_TimeSlice
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_TimeSlice_SetZero(CntrFun,SetMatsZero)
    class(CntrFun_timeslice_t),intent(inout) :: CntrFun
    logical,intent(in),optional :: SetMatsZero
    logical :: matzero

    matzero = .false.
    if(present(SetMatsZero)) matzero = SetMatsZero

    CntrFun%Lesser(:,:,0:)=zero
    CntrFun%Greater(:,:,0:)=zero
    if(CntrFun%Ntau > 0) then
       CntrFun%Left(:,:,0:)=zero
       if(matzero) CntrFun%Mats(:,:,0:)=zero
    end if

  end subroutine CntrFun_TimeSlice_SetZero
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_timeslice_Scalar_Retarded(CntrFun,jt,CntrFun_Ret)
    !.................................................................
    !  t_i >= t_j : (t_i not explicit)
    !
    !  \Sigma^{R}(t_i,t_j) = \Sigma^>(t_i,t_j) - \Sigma^<(t_i,t_j)
    !                    = \Sigma^>(t_i,t_j) + (\Sigma^<(t_j,t_i))*
    !.................................................................
    integer,intent(in)::jt
    class(CntrFun_timeslice_t),intent(in)::CntrFun
    complex(dp),intent(out)::CntrFun_Ret

    CntrFun_Ret=CntrFun%Greater(1,1,jt) + conjg(CntrFun%Lesser(1,1,jt))

  end subroutine CntrFun_timeslice_Scalar_Retarded
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_timeslice_Matrix_Retarded(CntrFun,jt,CntrFun_Ret)
    !.................................................................
    !  t_i >= t_j : (t_i not explicit)
    !
    !  \Sigma^R(t_i,t_j) = \Sigma^>(t_i,t_j) - \Sigma^<(t_i,t_j)
    !                    = \Sigma^>(t_i,t_j) + (\Sigma^<(t_j,t_i))*
    !.................................................................
    integer,intent(in)::jt
    class(CntrFun_timeslice_t),intent(in)::CntrFun
    complex(dp),dimension(CntrFun%Nstates,CntrFun%Nstates),intent(out)::CntrFun_Ret

    CntrFun_Ret=CntrFun%Greater(:,:,jt) + conjg(transpose(CntrFun%Lesser(:,:,jt)))

  end subroutine CntrFun_timeslice_Matrix_Retarded
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_timeslice_Scalar_Advanced(CntrFun,jt,CntrFun_Ad)
    !.................................................................
    !  t_i >= t_j : (t_i not explicit)
    !
    !  \Sigma^A(t_i,t_j) = \Sigma^>(t_i,t_j) - \Sigma^<(t_i,t_j)
    !                    = \Sigma^>(t_i,t_j) + (\Sigma^<(t_j,t_i))*
    !.................................................................
    integer,intent(in)::jt
    class(CntrFun_timeslice_t),intent(in)::CntrFun
    complex(dp),intent(out)::CntrFun_Ad

    CntrFun_Ad=conjg(CntrFun%Greater(1,1,jt)) + CntrFun%Lesser(1,1,jt)

  end subroutine CntrFun_timeslice_Scalar_Advanced
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_timeslice_Matrix_Advanced(CntrFun,jt,CntrFun_Ad)
    !.................................................................
    !  t_i >= t_j : (t_i not explicit)
    !
    !  \Sigma^R(t_i,t_j) = \Sigma^>(t_i,t_j) - \Sigma^<(t_i,t_j)
    !                    = \Sigma^>(t_i,t_j) + (\Sigma^<(t_j,t_i))*
    !.................................................................
    class(CntrFun_timeslice_t),intent(in)::CntrFun
    integer,intent(in)::jt
    complex(dp),dimension(CntrFun%Nstates,CntrFun%Nstates),intent(out)::CntrFun_Ad

    CntrFun_Ad=CntrFun%Lesser(:,:,jt ) + conjg(transpose(CntrFun%Greater(:,:,jt)))

  end subroutine CntrFun_timeslice_Matrix_Advanced
!--------------------------------------------------------------------------------------
  pure subroutine CntrFun_timeslice_Matrix_Right(CntrFun,itau,CntrFun_Right)
    class(CntrFun_timeslice_t),intent(in)::CntrFun
    integer,intent(in)::itau
    complex(dp),dimension(CntrFun%Nstates,CntrFun%Nstates),intent(out)::CntrFun_Right
    integer::ktau

    ktau=CntrFun%Ntau-itau
    CntrFun_Right = -sign(1,CntrFun%FermBos)*conjg(transpose(CntrFun%Left(:,:,ktau)))

  end subroutine CntrFun_timeslice_Matrix_Right
!--------------------------------------------------------------------------------------
  pure function CntrFun_timeslice_Rho_eq(CntrFun) result(Rho)
    class(CntrFun_timeslice_t),intent(in)::CntrFun
    complex(dp),dimension(CntrFun%Nstates,CntrFun%Nstates)::Rho

    Rho = -CntrFun%Mats(:,:,CntrFun%Ntau)

  end function CntrFun_timeslice_Rho_eq
!--------------------------------------------------------------------------------------
  pure function CntrFun_timeslice_Rho_neq(CntrFun,it) result(Rho)
    class(CntrFun_timeslice_t),intent(in)::CntrFun
    integer,intent(in)::it
    complex(dp),dimension(CntrFun%Nstates,CntrFun%Nstates)::Rho

    Rho = -iu*CntrFun%Lesser(:,:,it)

  end function CntrFun_timeslice_Rho_neq
!--------------------------------------------------------------------------------------
  subroutine Init_timeslice_array_L(func,N,FermBos,Beta,Nstates,Nt,Ntau)
    type(CntrFun_timeslice_t),allocatable,intent(inout) :: func(:)
    integer,intent(in)  :: N
    integer,intent(in)  :: FermBos
    real(dp),intent(in) :: Beta
    integer,intent(in)  :: nstates
    integer,intent(in)  :: Nt
    integer,intent(in)  :: Ntau
    integer :: i

    allocate(func(N))
    do i=1,N
       call func(i)%Init(FermBos,Beta,Nstates,Nt,Ntau)
    end do

  end subroutine Init_timeslice_array_L
!--------------------------------------------------------------------------------------
  subroutine Init_timeslice_array_Keld(func,N,FermBos,Beta,Nstates,Nt)
    type(CntrFun_timeslice_t),allocatable,intent(inout) :: func(:)
    integer,intent(in)  :: N
    integer,intent(in)  :: FermBos
    real(dp),intent(in) :: Beta
    integer,intent(in)  :: nstates
    integer,intent(in)  :: Nt
    integer :: i

    allocate(func(N))
    do i=1,N
       call func(i)%Init_Keldysh(FermBos,Beta,Nstates,Nt)
    end do

  end subroutine Init_timeslice_array_Keld
!--------------------------------------------------------------------------------------
  subroutine Clean_timeslice_array(func)
    type(CntrFun_timeslice_t),allocatable,intent(inout) :: func(:)
    integer :: i

    do i=1,size(func)
       call func(i)%Clean()
    end do
    deallocate(func)

  end subroutine Clean_timeslice_array
!--------------------------------------------------------------------------------------

