module kbkit_greenfun
!======================================================================================
   use Mdebug
   use Mdef,only:&
      dp,&
      iu,&
      nfermi,&
      nnfermi,&
      nbose,&
      one,&
      zero
   use Mlinalg,only:&
      EigHE,&
      Eye
   use Mgrid1d,only:&
      grid1d_t
   implicit none
!--------------------------------------------------------------------------------------

!--------------------------------------------------------------------------------------
   private
   public :: &
      CntrFun_1time_t,&
      CntrFun_2time_t,&
      CntrFun_timeslice_t,&
      Cntr2tm_to_CntrSlice,&
      CntrSlice_to_Cntr2tm,&
      Init_1time_array,&
      Init_2time_array,&
      Init_timeslice_array,&
      Clean_1time_array,&
      Clean_2time_array,&
      Clean_timeslice_array
!--------------------------------------------------------------------------------------
   type CntrFun_2time_t
      !! two-time contour function C(t,t'), typically for storing the Green's function.
      logical  :: Initialized=.false.
      integer  :: FermBos        !! 1 means bosons, -1 fermions,
      integer  :: Nstates        !! Nstates  -   matrix rank
      integer  :: Nt1,Nt2        !! number of time points; we keep different size for
      !! the purposeof MPI parallalization (in this
      !! case Nt2 can be of smaller size). We allocate
      !! C>(Nt1,Nt2) , C<(Nt2,Nt1), C^{left}(Nt1,Ntau) , C^M(Ntau)
      integer  :: Ntau           !! number of points on Matsubara branch
      real(dp) :: Beta           !! inverse temperature
      complex(dp),allocatable,dimension(:,:,:,:)  :: Lesser,Greater
      !! Lesser/Greater components
      complex(dp),allocatable,dimension(:,:,:,:)  :: Left
      !! left-mixing component
      complex(dp),allocatable,dimension(:,:,:)    :: Mats
      !! Matsubara component
   contains
      procedure :: Init => Init_CntrFun_2time_KB
      procedure :: Init_Keldysh => Init_CntrFun_2time_Keld
      procedure :: Clean => Free_CntrFun_2time
      procedure :: SetZero => CntrFun_2time_SetZero
      procedure :: GetRetarded => CntrFun_2time_Matrix_Retarded
      procedure :: GetAdvanced => CntrFun_2time_Matrix_Advanced
      procedure :: GetRight => CntrFun_2time_Matrix_Right
      procedure :: Rho_eq => CntrFun_2time_Rho_eq
      procedure :: Rho_Neq => CntrFun_2time_Rho_Neq
      procedure :: Symmetrize => CntrFun_2time_symmetrize
   end type Cntrfun_2time_t

   type CntrFun_timeslice_t
      !! Time slice of two-time contour function C(T,t) ,
      !! (t=0,...,T) , typically for storing the self-energy
      logical  :: Initialized=.false.
      integer  :: FermBos !! 1 means bosons, -1 fermions
      integer  :: Nstates !! matrix rank
      integer  :: Nt      !! similar to Nt2 as in CntrFun2time_t
      integer  :: Ntau    !! number of points on Matsubara branch
      real(dp) :: Beta    !! inverse temperature
      complex(dp),allocatable,dimension(:,:,:) :: Lesser,Greater
      !! Lesser/Greater components
      complex(dp),allocatable,dimension(:,:,:) :: Left
      !! left-mixing component
      complex(dp),allocatable,dimension(:,:,:) :: Mats
      !! Matsubara component
   contains
      procedure :: Init => Init_CntrFun_timeslice
      procedure :: Init_Keldysh => Init_CntrFun_timeslice_keld
      procedure :: Clean => Free_CntrFun_timeslice
      procedure :: SetZero => CntrFun_timeslice_SetZero
      procedure :: GetRetarded => CntrFun_timeslice_Matrix_Retarded
      procedure :: GetAdvanced => CntrFun_timeslice_Matrix_Advanced
      procedure :: GetRight=> CntrFun_timeslice_Matrix_Right
      procedure :: Rho_eq => CntrFun_timeslice_Rho_eq
      procedure :: Rho_Neq => CntrFun_timeslice_Rho_Neq
   end type CntrFun_timeslice_t

   type CntrFun_1time_t
      !! One-time contour function f(t) , where t is
      !! either real -> f(t) or imaginary f(-i*tau) = f(t0)
      logical  :: Initialized=.false.
      integer  :: Nstates !! matrix rank
      integer  :: Nt      !! number of time points
      real(dp) :: Beta    !! inverse temperature
      complex(dp),allocatable,dimension(:,:,:) :: Ret
      !! real-time component
      complex(dp),allocatable,dimension(:,:) :: Mats
      !! imaginary-time component
   contains
      procedure :: Init => Init_CntrFun_1time
      procedure :: Clean => Free_CntrFun_1time
      procedure :: SetZero => CntrFun_1time_SetZero
      procedure :: SetConst =>  CntrFun_1time_SetConst
      procedure :: Interpolate => CntrFun_1time_Interpolate
   end type CntrFun_1time_t
!--------------------------------------------------------------------------------------
   interface Init_timeslice_array
      module procedure Init_timeslice_array_L,Init_timeslice_array_Keld
   end interface Init_timeslice_array

   interface Init_2time_array
      module procedure Init_2time_array_L,Init_2time_array_Keld
   end interface Init_2time_array
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
include "greenfun_2time_inc.F90"
include "greenfun_timeslice_inc.F90"
include "greenfun_1time_inc.F90"
!--------------------------------------------------------------------------------------
   pure subroutine Cntr2tm_to_CntrSlice(it,CntrFun_2tm,CntrFun_slice,CopyMats)
      integer,intent(in)::it
      type(CntrFun_2time_t),intent(in)::CntrFun_2tm
      type(CntrFun_timeslice_t),intent(inout)::CntrFun_slice
      logical,intent(in),optional::CopyMats

      associate(Ntau => CntrFun_2tm%Ntau, n => CntrFun_2tm%Nstates)
         CntrFun_slice%Lesser(1:n,1:n,0:it)=CntrFun_2tm%Lesser(1:n,1:n,0:it,it)
         CntrFun_slice%Greater(1:n,1:n,0:it)=CntrFun_2tm%Greater(1:n,1:n,it,0:it)
         if(allocated(CntrFun_2tm%Left)) CntrFun_slice%Left(1:n,1:n,0:Ntau)=CntrFun_2tm%Left(1:n,1:n,it,0:Ntau)
         if(present(CopyMats)) then
            if(CopyMats.and.allocated(CntrFun_slice%Mats)) then
               CntrFun_slice%Mats(1:n,1:n,0:Ntau)=CntrFun_2tm%Mats(1:n,1:n,0:Ntau)
            end if
         end if
      end associate

   end subroutine Cntr2tm_to_CntrSlice
!--------------------------------------------------------------------------------------
   pure subroutine CntrSlice_to_Cntr2tm(it,CntrFun_slice,CntrFun_2tm,CopyMats,kbe_storage)
      integer,intent(in)::it
      type(CntrFun_2time_t),intent(inout)::CntrFun_2tm
      type(CntrFun_timeslice_t),intent(inout)::CntrFun_slice
      logical,intent(in),optional::CopyMats,kbe_storage
      logical :: kbe_style
      integer :: jt

      kbe_style = .true.
      if(present(kbe_storage)) kbe_style = kbe_storage

      if(kbe_style) then
         associate(Ntau => CntrFun_2tm%Ntau, n => CntrFun_2tm%Nstates)
            CntrFun_2tm%Lesser(1:n,1:n,0:it,it) = CntrFun_slice%Lesser(1:n,1:n,0:it)
            CntrFun_2tm%Greater(1:n,1:n,it,0:it) = CntrFun_slice%Greater(1:n,1:n,0:it)
            if(allocated(CntrFun_2tm%Left)) CntrFun_2tm%Left(1:n,1:n,it,0:Ntau) = CntrFun_slice%Left(1:n,1:n,0:Ntau)

            do jt=0,it
               CntrFun_2tm%Lesser(1:n,1:n,it,jt) = -hermconjg(CntrFun_2tm%Lesser(1:n,1:n,jt,it))
               CntrFun_2tm%Greater(1:n,1:n,jt,it) = -hermconjg(CntrFun_2tm%Greater(1:n,1:n,it,jt))
            end do

            if(present(CopyMats)) then
               if(CopyMats.and.allocated(CntrFun_2tm%Mats)) then
                  CntrFun_2tm%Mats(1:n,1:n,0:Ntau) = CntrFun_slice%Mats(1:n,1:n,0:Ntau)
               end if
            end if
         end associate
      else
         ! .. GKBA storage scheme ..
         associate(n => CntrFun_2tm%Nstates)
            CntrFun_2tm%Lesser(1:n,1:n,it,0:it) = CntrFun_slice%Lesser(1:n,1:n,0:it)
            CntrFun_2tm%Greater(1:n,1:n,it,0:it) = CntrFun_slice%Greater(1:n,1:n,0:it)
            do jt=0,it-1
               CntrFun_2tm%Lesser(1:n,1:n,jt,it) = -hermconjg(CntrFun_2tm%Lesser(1:n,1:n,it,jt))
               CntrFun_2tm%Greater(1:n,1:n,jt,it) = -hermconjg(CntrFun_2tm%Greater(1:n,1:n,it,jt))
            end do
         end associate
      end if

   end subroutine CntrSlice_to_Cntr2tm
!--------------------------------------------------------------------------------------
   pure function HermConjg(A) result(Acc)
      complex(dp),intent(in) :: A(:,:)
      complex(dp) :: Acc(size(A,dim=2),size(A,dim=1))

      Acc = conjg(transpose(A))

   end function HermConjg
!--------------------------------------------------------------------------------------

!======================================================================================
end module Kbkit_Greenfun
