module kbkit_params
!======================================================================================
   use Mdef,only: dp
   implicit none
!--------------------------------------------------------------------------------------
   private
   public :: kbe_params_t
!--------------------------------------------------------------------------------------
   type :: kbe_params_t
      ! .. Matsubara solver params ..
      integer  :: MaxFixpIter = 8
      integer  :: Ntau_eq = 1024
      integer  :: Mats_MaxIter = 100
      real(dp) :: Mats_ErrTol=1.0e-10_dp
      real(dp) :: Mats_FixpTol=1.0e-8_dp
      ! .. KBE solver params ..
      integer  :: GregoryOrder = 6
      integer  :: ExtrapolOrder = 3
      integer  :: InterpolOrder = 4
      integer  :: Predictor = 5
      integer  :: Corrector = 4
      integer  :: ExplicitOrder = 5
      integer  :: ImplicitOrder = 6
      integer  :: nstart = 5
      integer  :: NeqIter = 2
      integer  :: MaxBootstrapIter=20
      real(dp) :: Bootstrap_ErrTol=1.0e-10_dp
   contains
      procedure,public :: ReadFromNML
      procedure,public :: Set
   end type kbe_params_t
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
   subroutine ReadFromNML(me,fname)
      class(kbe_params_t)         :: me
      character(len=*),intent(in) :: fname
      integer :: unit_inp
      ! .............................
      ! .. Matsubara solver params ..
      integer  :: MaxFixpIter = 8
      integer  :: Ntau_eq = 1024
      integer  :: Mats_MaxIter = 100
      real(dp) :: Mats_FixpTol=1.0e-8_dp
      real(dp) :: Mats_ErrTol=1.0e-10_dp
      ! .. KBE solver params ..
      integer  :: GregoryOrder = 6
      integer  :: ExtrapolOrder = 3
      integer  :: InterpolOrder = 4
      integer  :: Predictor = 5
      integer  :: Corrector = 4
      integer  :: ExplicitOrder = 5
      integer  :: ImplicitOrder = 6
      integer  :: NeqIter = 2
      integer  :: MaxBootstrapIter=20
      real(dp) :: Bootstrap_ErrTol=1.0e-10_dp
      namelist/KBEPARAMS/MaxFixpIter,Ntau_eq,Mats_MaxIter,Mats_FixpTol,GregoryOrder,&
         ExtrapolOrder,InterpolOrder,Predictor,Corrector,ExplicitOrder,&
         ImplicitOrder,NeqIter,MaxBootstrapIter,Bootstrap_ErrTol,Mats_ErrTol

      open(newunit=unit_inp,file=trim(fname),status='old',action='read')
      read(unit_inp,nml=KBEPARAMS)
      close(unit_inp)
      
      me%MaxFixpIter = MaxFixpIter
      me%Ntau_eq = Ntau_eq
      me%Mats_MaxIter = Mats_MaxIter
      me%Mats_FixpTol = Mats_FixpTol
      me%Mats_ErrTol = Mats_ErrTol
      me%GregoryOrder = GregoryOrder
      me%ExtrapolOrder = ExtrapolOrder
      me%InterpolOrder = InterpolOrder
      me%Predictor = Predictor
      me%Corrector = Corrector
      me%ExplicitOrder = ExplicitOrder
      me%ImplicitOrder = ImplicitOrder
      me%nstart = me%ImplicitOrder-1
      me%NeqIter = NeqIter
      me%MaxBootstrapIter = MaxBootstrapIter
      me%Bootstrap_ErrTol = Bootstrap_ErrTol
      
   end subroutine ReadFromNML
!--------------------------------------------------------------------------------------
   subroutine Set(me,pars)
      class(kbe_params_t)           :: me
      type(kbe_params_t),intent(in) :: pars

      me%MaxFixpIter = pars%MaxFixpIter
      me%Ntau_eq = pars%Ntau_eq
      me%Mats_MaxIter = pars%Mats_MaxIter
      me%Mats_FixpTol = pars%Mats_FixpTol
      me%Mats_ErrTol = pars%Mats_ErrTol
      me%GregoryOrder = pars%GregoryOrder
      me%ExtrapolOrder = pars%ExtrapolOrder
      me%InterpolOrder = pars%InterpolOrder
      me%Predictor = pars%Predictor
      me%Corrector = pars%Corrector
      me%ExplicitOrder = pars%ExplicitOrder
      me%ImplicitOrder = pars%ImplicitOrder
      me%nstart = me%ImplicitOrder - 1
      me%NeqIter = pars%NeqIter
      me%MaxBootstrapIter = pars%MaxBootstrapIter
      me%Bootstrap_ErrTol = pars%Bootstrap_ErrTol

   end subroutine Set
!--------------------------------------------------------------------------------------

!======================================================================================
end module kbkit_params
