module Mham_free_tdarpes
!======================================================================================
   use,intrinsic :: iso_fortran_env,only: output_unit,error_unit
   use omp_lib
   use Mdebug
   use Mdef,only: dp,iu,one,zero
   use Mgrid1d,only: grid1d_t
   use Mlinalg,only: util_rotate,util_rotate_cc
   use Meqval,only: DTrAB,EqFromHam
   use Mevol,only: GenU_CF4, UnitaryStepFBW
   use Mpropagate,only: Propagate_OBE,Propagate_Houston,&
      Propagate_Houston_ext, Propagate_OBE_1bnd, Propagate_Houston_1bnd
   use Mtdarpes_gkba,only: &
      ProbeSpectrum, ProbeSpectrum_MatrixElements ,&
      ProbeSpectrum_PumpPulse, ProbeSpectrum_PumpPulse_MatrixElements
   !---------------------
   !    KBKIT modules
   !---------------------
   use kbkit_greenfun
   !---------------------
   implicit none
   include 'formats.h'
!--------------------------------------------------------------------------------------
   integer,parameter :: evol_free=0, evol_obe=1, evol_houst=2, evol_houst_ext=3
   ! .. KBE parameters ..
   logical :: large_size
   ! .. external vector potential ..
   procedure(vecpot_efield_func),pointer :: field => null()
!--------------------------------------------------------------------------------------
   private
   public :: ham_tdarpes_t
!--------------------------------------------------------------------------------------
   type ham_tdarpes_t
      logical  :: retarded_mode=.false.
      integer  :: nbnd
      integer  :: evol=evol_free
      real(dp) :: screen_par,screen_perp,screen_pe_par,screen_pe_perp
      real(dp) :: Beta,MuChem
      real(dp) :: T1,T2
      real(dp),allocatable,dimension(:,:) :: Tmat
      real(dp),allocatable,dimension(:,:) :: Avec_pe
      ! .. grids ..
      type(grid1d_t) :: tgrd
      ! .. Hamiltonian class ..
      real(dp),allocatable,dimension(:)        :: epsk
      complex(dp),pointer,dimension(:)         :: mel => null()
      complex(dp),allocatable,dimension(:,:)   :: vect
      complex(dp),allocatable,dimension(:,:,:) :: Pmom
      ! .. contour objects ..
      type(CntrFun_1time_t),private :: Ht,Rho
   contains
      procedure,public  :: Init
      procedure,public  :: SetLaserpulse
      procedure,public  :: SetMatrixElements      
      procedure,public  :: Propagate
      procedure,public  :: GetObs
      procedure,public  :: CalcTARPES
      procedure,private :: GetHam
      procedure,public  :: Occupation
      procedure,public  :: Dipole
   end type ham_tdarpes_t
!--------------------------------------------------------------------------------------
   abstract interface

   subroutine vecpot_efield_func(t,A,E)
      import :: dp
      implicit none
      real(dp),intent(in) :: t
      real(dp),intent(out) :: A(3),E(3)
   end subroutine vecpot_efield_func

   end interface
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
   subroutine Init(me,Beta,MuChem,H0,Pmom,screen_dyn,screen_pe,Nt,Tmax,T1,T2,Tmat,evol,retarded_mode)
      use Mlinalg,only: EigHE,Eye
      class(ham_tdarpes_t)          :: me
      real(dp),intent(in)           :: Beta
      real(dp),intent(in)           :: MuChem
      complex(dp),intent(in)        :: H0(:,:)
      complex(dp),intent(in)        :: Pmom(:,:,:)
      real(dp),intent(in)           :: screen_dyn(2)
      real(dp),intent(in)           :: screen_pe(2)
      integer,intent(in)            :: Nt
      real(dp),intent(in)           :: Tmax
      real(dp),intent(in),optional  :: T1,T2
      real(dp),intent(in),optional  :: Tmat(:,:)
      integer,intent(in),optional   :: evol
      logical,intent(in),optional   :: retarded_mode

      if(present(T1)) me%T1 = T1
      if(present(T2)) me%T2 = T2
      if(present(evol)) me%evol = evol
      if(present(retarded_mode)) me%retarded_mode = retarded_mode

      if(present(Tmat) .and. me%evol == evol_houst_ext) then
         allocate(me%Tmat(me%nbnd,me%nbnd))
         me%Tmat = Tmat
      end if

      me%Beta = Beta
      me%MuChem = MuChem
      me%nbnd = size(H0,1)
      allocate(me%Pmom(me%nbnd,me%nbnd,3))
      me%Pmom = Pmom
      me%screen_par = screen_dyn(1)
      me%screen_perp = screen_dyn(2)
      me%screen_pe_par = screen_pe(1)
      me%screen_pe_perp = screen_pe(2)

      call me%tgrd%Init(Nt,0.0_dp,Tmax)

      large_size = me%nbnd >= 8

      call me%Ht%Init(me%Beta,me%nbnd,Nt)
      call me%Rho%Init(me%Beta,me%nbnd,Nt)
      
      me%Ht%Mats = H0

      allocate(me%epsk(me%nbnd),me%vect(me%nbnd,me%nbnd))
      call EigHE(me%Ht%Mats, me%epsk, me%vect)

      if(me%retarded_mode) then
         me%Rho%Mats = Eye(me%nbnd)
      else
         call EqFromHam(me%nbnd,me%Beta,me%MuChem,me%Ht%Mats,me%Rho%Mats)
      end if
      me%Rho%Ret(:,:,0) = me%Rho%Mats

   end subroutine Init
!--------------------------------------------------------------------------------------
   subroutine SetLaserpulse(me,Laserfield)
      class(ham_tdarpes_t)  :: me
      procedure(vecpot_efield_func) :: Laserfield
      integer :: it
      real(dp) :: AF(3),EF(3)

      field => Laserfield

      allocate(me%Avec_pe(3,0:me%tgrd%Npts)); me%Avec_pe = 0.0_dp
      if(associated(field)) then
         do it=0,me%tgrd%Npts
            call field(me%tgrd%Xpts(it),AF,EF)
            me%Avec_pe(1:2,it) = me%screen_pe_par * AF(1:2)
            me%Avec_pe(3,it) = me%screen_pe_perp * AF(3)
         end do
      end if

   end subroutine SetLaserpulse
!--------------------------------------------------------------------------------------
   subroutine SetMatrixElements(me,mel)
      class(ham_tdarpes_t)  :: me
      complex(dp),target,intent(in) :: mel(:)

      me%mel => mel

   end subroutine SetMatrixElements
!--------------------------------------------------------------------------------------
   function GetHam(me,t) result(Ht)
      class(ham_tdarpes_t) :: me
      real(dp),intent(in)   :: t
      complex(dp),dimension(me%nbnd,me%nbnd) :: Ht
      integer :: i,idir
      real(dp) :: AF(3),EF(3),AFscr(3)

      if(.not.associated(field)) then
         Ht = me%Ht%Mats
         return
      end if

      call field(t,AF,EF)

      AFscr(1:2) = me%screen_par * AF(1:2)
      AFscr(3) = me%screen_perp * AF(3)

      Ht = me%Ht%Mats
      do idir=1,3
         Ht = Ht - AFscr(idir) * me%Pmom(:,:,idir)
      end do
      do i=1,me%nbnd
         Ht(i,i) = Ht(i,i) + 0.5_dp * sum(AFscr**2)
      end do

   end function GetHam
!--------------------------------------------------------------------------------------
   subroutine Propagate(me)
      use Mdef,only: nfermi
      real(dp),parameter :: c1=0.5_dp-sqrt(3.0_dp)/6.0_dp
      real(dp),parameter :: c2=0.5_dp+sqrt(3.0_dp)/6.0_dp
      class(ham_tdarpes_t) :: me
      integer :: tstp
      real(dp) :: t1,t2,EF(2),feq(me%nbnd)
      complex(dp),dimension(me%nbnd,me%nbnd) :: H1,H2,Udt

      do tstp=0,me%tgrd%Npts
         me%Ht%Ret(:,:,tstp) = me%GetHam(me%tgrd%Xpts(tstp))
      end do
     
      if(me%retarded_mode) then
         do tstp=0,me%tgrd%Npts
            me%Rho%Ret(:,:,tstp) = me%Rho%Mats
         end do
         return
      end if

      select case(me%evol)
      case(evol_free)
         do tstp=0,me%tgrd%Npts-1
            t1 = (tstp + c1) * me%tgrd%dX
            t2 = (tstp + c2) * me%tgrd%dX
            H1 = me%GetHam(t1)
            H2 = me%GetHam(t2)
            call GenU_CF4(me%tgrd%dX,H1,H2,Udt)
            call UnitaryStepFBW(me%nbnd,Udt,me%Rho%ret(:,:,tstp),me%Rho%ret(:,:,tstp+1))
         end do
      case(evol_obe)
         select case(me%nbnd)
         case(1)
            feq(1) = dble(me%Rho%Mats(1,1))
            call Propagate_OBE_1bnd(me%tgrd,feq(1),me%T1,me%HT,me%Rho)
         case(2)
            feq(1) = dble(me%Rho%Mats(1,1))
            feq(2) = dble(me%Rho%Mats(2,2))
            call Propagate_OBE(me%tgrd,feq,me%T1,me%T2,me%Ht,me%Rho,band_basis=.true.)
         case default
            write(error_unit,*) "OBEs only implemented for one- and two-band systems!"
            stop
         end select
      case(evol_houst)
         select case(me%nbnd)
         case(1)
            call Propagate_Houston_1bnd(me%tgrd,me%Beta,me%MuChem,me%T1,me%Ht,me%Rho)
         case(2)
            call Propagate_Houston(me%tgrd,me%Beta,me%MuChem,me%T1,me%T2,me%Ht,me%Rho)
         case default
            write(error_unit,*) "Houston OBEs only implemented for one- and two-band systems!"
            stop
         end select
      case(evol_houst_ext)
         call Propagate_Houston_ext(me%tgrd,me%Beta,me%MuChem,me%Tmat,me%Ht,me%Rho)
      case default
         write(error_unit,*) "Invalid flag for time evolution!"
         stop
      end select

   end subroutine Propagate
!--------------------------------------------------------------------------------------
   function Occupation(me,tstp) result(nocc)
      class(ham_tdarpes_t) :: me
      integer,intent(in)   :: tstp
      real(dp) :: nocc(me%nbnd)
      integer :: i
      complex(dp),dimension(me%nbnd,me%nbnd) :: rhob

      rhob = util_rotate(me%nbnd,me%vect,me%Rho%Ret(:,:,tstp),large_size=.false.)
      do i=1,me%nbnd
         nocc(i) = dble(Rhob(i,i))
      end do

   end function Occupation
!--------------------------------------------------------------------------------------
   function Dipole(me,tstp) result(dip)
      class(ham_tdarpes_t) :: me
      integer,intent(in)   :: tstp
      real(dp) :: dip(3)
      integer :: idir

      do idir=1,3
         dip(idir) = DTrAB(me%Rho%Ret(:,:,tstp),me%Pmom(:,:,idir))
      end do

   end function Dipole
!--------------------------------------------------------------------------------------
   subroutine GetObs(me,dip,occ)
      class(ham_tdarpes_t) :: me
      real(dp),allocatable,intent(out) :: dip(:,:),occ(:,:)
      integer :: tstp

      allocate(dip(3,0:me%tgrd%Npts),occ(me%nbnd,0:me%tgrd%Npts))

      do tstp=0,me%tgrd%Npts
         dip(:,tstp) = me%Dipole(tstp)
         occ(:,tstp) = me%Occupation(tstp)
      end do

   end subroutine GetObs
!--------------------------------------------------------------------------------------

!-------------------------------------------------------------------------------------- 
   subroutine CalcTARPES(me,tprobe,t0,wgrd,spect,kvec,pulse_shape,diag_matel)
      class(ham_tdarpes_t)      :: me
      real(dp),intent(in)       :: tprobe,t0
      type(grid1d_t),intent(in) :: wgrd
      real(dp),intent(inout)    :: spect(0:)
      real(dp),intent(in),optional :: kvec(:)
      integer,intent(in),optional  :: pulse_shape
      logical,intent(in),optional :: diag_matel
      logical :: lape,diag_matel_
      integer :: pulse_shape_
      integer :: i
      complex(dp),allocatable :: matel(:,:)

      lape = present(kvec)

      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      diag_matel_ = .false.
      if(present(diag_matel)) diag_matel_ = diag_matel

      if(associated(me%mel)) then

         allocate(matel(me%nbnd,0:wgrd%Npts))
         do i=1,me%nbnd
            matel(i,0:) = me%mel(i)
         end do

         if(lape) then
            call ProbeSpectrum_PumpPulse_MatrixElements(me%tgrd,wgrd,tprobe,t0,me%Ht,me%Rho,&
               kvec,me%Avec_pe,matel,spect,pulse_shape=pulse_shape_,diag_matel=diag_matel_)
         else
            call ProbeSpectrum_MatrixElements(me%tgrd,wgrd,tprobe,t0,me%Ht,me%Rho,matel,spect,&
               pulse_shape=pulse_shape_,diag_matel=diag_matel_)
         end if

         deallocate(matel)

      else
         if(lape) then
            call ProbeSpectrum_PumpPulse(me%tgrd,wgrd,tprobe,t0,me%Ht,me%Rho,kvec,me%Avec_pe,spect,&
               pulse_shape=pulse_shape_)     
         else
            call ProbeSpectrum(me%tgrd,wgrd,tprobe,t0,me%Ht,me%Rho,spect,&
               pulse_shape=pulse_shape_)         
         end if
      end if

   end subroutine CalcTARPES
!-------------------------------------------------------------------------------------- 

!======================================================================================
end module Mham_free_tdarpes
