module Mhamk_2bloc_bath_mpi
!======================================================================================
   use,intrinsic :: iso_fortran_env,only: output_unit,error_unit
   use mpi
   use omp_lib
   use Mdebug
   use Mdef,only: dp,iu,one,zero
   use Mgrid1d,only: grid1d_t
   use Mlinalg,only: util_rotate,util_rotate_cc
   use Meqval,only: DTrAB,EqFromHam
   use Mselfen_2B_gkba,only: GenSigma_2B_loc
   use Membedding,only: semicircle_bath_t, gauss_bath_t
   use Madiabatic,only: AdiabaticSwitch,SelfEn_MultiplySwitch
   use Marray1d_dist,only: dist_array1d_t, GetDisplSize1D
   use Mtdarpes_gkba,only: &
      ProbeSpectrum, ProbeSpectrum_MatrixElements ,&
      ProbeSpectrum_PumpPulse, ProbeSpectrum_PumpPulse_MatrixElements
   !---------------------
   !    KBKIT modules
   !---------------------
   use kbkit_greenfun
   use kbkit_gkba
   use kbkit_params
   !---------------------
   implicit none
   include 'formats.h'
!--------------------------------------------------------------------------------------
   character(len=*),parameter :: fmt_info='(" Info: ",a)'
   integer,parameter :: dos_gauss=0, dos_semi=1
   ! .. KBE parameters ..
   logical :: large_size
   integer,parameter :: FERM=-1,BOS=1
   type(kbe_params_t) :: kbe
   ! .. external vector potential ..
   procedure(vecpot_efield_func),pointer :: field => null()
   ! .. parallelization ..
   integer,parameter  :: master=0,from_master=1,from_worker=2
   integer :: ntasks,taskid,nthreads,threadid,ierr
   logical :: on_root
   integer :: status(MPI_STATUS_SIZE)
   type(dist_array1d_t),private :: kdist
!--------------------------------------------------------------------------------------
   private
   public :: bath_param_t, hamk_2bloc_bath_t
!--------------------------------------------------------------------------------------
   type bath_param_t
      integer :: dos
      integer :: nbaths,nst
      real(dp),allocatable,dimension(:)   :: MuChem,eps0,DeltaW
      real(dp),allocatable,dimension(:,:) :: cp
   end type bath_param_t

   type hamk_2bloc_bath_t
      logical  :: Embedding=.false.
      logical  :: AdiabaticSwitch = .false., CalcSelfEnergy=.true.
      integer  :: nbnd,Nk,Nk_loc,nbaths
      real(dp) :: Beta,MuChem
      real(dp) :: t_startswitch,t_endswitch,tau_switch
      real(dp),allocatable  :: Uhubb(:)
      real(dp),allocatable,dimension(:,:) :: Avec_pe
      ! .. grids ..
      type(grid1d_t) :: tgrd
      ! .. Hamiltonian class ..
      complex(dp),allocatable,dimension(:,:)         :: mel
      complex(dp),allocatable,dimension(:,:,:)       :: Hk0
      complex(dp),allocatable,dimension(:,:,:,:)     :: vk0
      complex(dp),allocatable,dimension(:,:,:)       :: vectk
      ! .. bath ..
      integer                                          :: bath_dos=0
      real(dp),allocatable,dimension(:,:)              :: bcp
      type(gauss_bath_t),allocatable,dimension(:)      :: baths_gauss
      type(semicircle_bath_t),allocatable,dimension(:) :: baths_semi
      ! .. GKBA object & solver ..
      type(gkba_t),allocatable,dimension(:) :: gkba
      ! .. contour objects ..
      type(CntrFun_1time_t),allocatable,dimension(:) :: Hk
      type(CntrFun_timeslice_t),private              :: Gloc,SGM
      complex(dp),allocatable,dimension(:,:,:)       :: Gloc_cont
   contains
      procedure,public  :: Init
      procedure,public  :: SetLaserpulse
      procedure,public  :: SetEmbedding
      procedure,public  :: SetAdiabaticSwitch
      procedure,public  :: SetDephasing
      procedure,public  :: SetMatrixElements 
      procedure,public  :: Bootstrapping
      procedure,public  :: Timestep
      procedure,public  :: BandOcc
      procedure,public  :: Dipole
      procedure,public  :: CalcTARPES
      procedure,private :: GetHam
      procedure,private :: Update_Gloc
      procedure,private :: Update_Sigma
   end type hamk_2bloc_bath_t
!--------------------------------------------------------------------------------------
   abstract interface

   subroutine vecpot_efield_func(t,A,E)
      import :: dp
      implicit none
      real(dp),intent(in) :: t
      real(dp),intent(out) :: A(3),E(3)
   end subroutine vecpot_efield_func

   end interface
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
   subroutine Init(me,Beta,MuChem,Hk0,vk0,U,Nt,Tmax,kbe_pars)
      use Mlinalg,only: EigHE,Eye
      class(hamk_2bloc_bath_t)      :: me
      real(dp),intent(in)           :: Beta
      real(dp),intent(in)           :: MuChem
      complex(dp),intent(in)        :: Hk0(:,:,:)
      complex(dp),intent(in)        :: vk0(:,:,:,:)
      real(dp),intent(in)           :: U(:)
      integer,intent(in)            :: Nt
      real(dp),intent(in)           :: Tmax
      type(kbe_params_t),intent(in) :: kbe_pars
      integer :: ik,ik_glob
      real(dp),allocatable :: epsk(:)
      character(len=32) :: spar
      integer :: nsize
      integer,allocatable :: displ(:),size_loc(:)


      call MPI_COMM_RANK(MPI_COMM_WORLD, taskid, ierr)
      call MPI_COMM_SIZE(MPI_COMM_WORLD, ntasks, ierr)
      on_root = taskid == master

      if(on_root) then
         write(output_unit,fmt50)
         write(output_unit,'(A)') '         Parallelization'
         write(output_unit,fmt50)
         write(spar,*) ntasks
         write(output_unit,fmt_info) "number of ranks: "//trim(adjustl(spar))

         !$OMP PARALLEL PRIVATE(nthreads,threadid)
         threadid = omp_get_thread_num()
         if(threadid == 0) then
            nthreads = omp_get_num_threads()
            write(spar,*) nthreads
            write(output_unit,fmt_info) "number of threads: "//trim(adjustl(spar))
            write(output_unit,*)
         end if
         !$OMP END PARALLEL
      end if

      if(on_root) then
         me%Nk = size(Hk0, dim=3)
         me%nbnd = size(Hk0, dim=1)
      end if

      call MPI_Bcast(me%Nk,1,MPI_INT,master,MPI_COMM_WORLD,ierr)
      call MPI_Bcast(me%nbnd,1,MPI_INT,master,MPI_COMM_WORLD,ierr)

      call kdist%Init(ntasks,taskid,me%Nk)
      me%Nk_loc = kdist%N_loc(taskid)

      allocate(me%Hk0(me%nbnd,me%nbnd,me%Nk_loc))
      allocate(displ(0:ntasks-1),size_loc(0:ntasks-1))
      call GetDisplSize1D(kdist%N_loc,me%nbnd*me%nbnd,displ,nsize,size_loc)

      call MPI_Scatterv(Hk0,size_loc,displ,MPI_DOUBLE_COMPLEX,me%Hk0,nsize,&
         MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,ierr)

      allocate(me%vk0(me%nbnd,me%nbnd,3,me%Nk_loc))
      call GetDisplSize1D(kdist%N_loc,me%nbnd*me%nbnd*3,displ,nsize,size_loc)

      call MPI_Scatterv(vk0,size_loc,displ,MPI_DOUBLE_COMPLEX,me%vk0,nsize,&
         MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,ierr)

      deallocate(displ,size_loc)      

      me%Beta = Beta
      me%MuChem = MuChem
      me%nbnd = size(me%Hk0,1)
      allocate(me%Uhubb(me%nbnd))
      me%Uhubb(1:me%nbnd) = U(1:me%nbnd)

      call me%tgrd%Init(Nt,0.0_dp,Tmax)

      call kbe%Set(kbe_pars)
      large_size = me%nbnd >= 8

      call InitIntegrator(kbe%GregoryOrder,me%tgrd)
      call Init_GKBA_Array(me%gkba,me%Nk_loc,me%nbnd,me%tgrd,kbe%GregoryOrder)
      call Init_1time_array(me%Hk,me%Nk_loc,me%Beta,me%nbnd,Nt)
      call me%SGM%Init_Keldysh(FERM,me%Beta,me%nbnd,me%tgrd%Npts)
      call me%Gloc%Init_Keldysh(FERM,me%Beta,me%nbnd,me%tgrd%Npts)
      allocate(me%Gloc_cont(me%nbnd,me%nbnd,0:me%tgrd%Npts))

      allocate(me%vectk(me%nbnd,me%nbnd,me%Nk_loc),epsk(me%nbnd))
      do ik=1,me%Nk_loc
         call EigHE(me%Hk0(:,:,ik), epsk, me%vectk(:,:,ik))
      end do
      deallocate(epsk)

      do ik=1,me%Nk_loc
         call me%gkba(ik)%SetEqFromHam(me%Beta,me%MuChem,me%Hk0(:,:,ik))
      end do

   end subroutine Init
!--------------------------------------------------------------------------------------
   subroutine SetLaserpulse(me,Laserfield)
      class(hamk_2bloc_bath_t)  :: me
      procedure(vecpot_efield_func) :: Laserfield
      integer :: it
      real(dp) :: EF(3),AF(3)

      field => Laserfield

      allocate(me%Avec_pe(3,0:me%tgrd%Npts)); me%Avec_pe = 0.0_dp
      if(associated(field)) then
         do it=0,me%tgrd%Npts
            call field(me%tgrd%Xpts(it),AF,EF)
            me%Avec_pe(1:3,it) = AF(1:3)
         end do
      end if

   end subroutine SetLaserpulse
!--------------------------------------------------------------------------------------
   subroutine SetAdiabaticSwitch(me,switch,t_startswitch,t_endswitch,tau_switch)
      class(hamk_2bloc_bath_t) :: me
      logical,intent(in)       :: switch
      real(dp),intent(in)      :: t_startswitch,t_endswitch,tau_switch

      me%AdiabaticSwitch = switch
      me%t_startswitch = t_startswitch
      me%t_endswitch = t_endswitch
      me%tau_switch = tau_switch

   end subroutine SetAdiabaticSwitch
!--------------------------------------------------------------------------------------
   subroutine SetEmbedding(me,bath_par)
      class(hamk_2bloc_bath_t)  :: me
      type(bath_param_t),intent(in) :: bath_par
      integer :: i,j,ibath

      me%Embedding = .true.

      allocate(me%bcp(me%nbnd,bath_par%nbaths))
      me%nbaths = bath_par%nbaths
      me%bcp = bath_par%cp
      me%bath_dos = bath_par%dos

      select case(me%bath_dos)
      case(dos_gauss)
         if(on_root) write(output_unit,fmt_info) "bath DOS: Gauss"
         allocate(me%baths_gauss(bath_par%nbaths))
         do ibath=1,bath_par%nbaths
            call me%baths_gauss(ibath)%Init(FERM,me%Beta,bath_par%MuChem(ibath),bath_par%eps0(ibath),&
               bath_par%DeltaW(ibath),me%tgrd%Npts,store_mode=2)

            call me%baths_gauss(ibath)%CalcGreen_diff(me%tgrd%dX)
         end do
      case(dos_semi)
         if(on_root) write(output_unit,fmt_info) "bath DOS: Bethe"
         allocate(me%baths_semi(bath_par%nbaths))
         do ibath=1,bath_par%nbaths
            call me%baths_semi(ibath)%Init(FERM,me%Beta,bath_par%MuChem(ibath),bath_par%eps0(ibath),&
               bath_par%DeltaW(ibath),me%tgrd%Npts,store_mode=2)

            call me%baths_semi(ibath)%CalcGreen_diff(me%tgrd%dX)
         end do         
      case default
         if(on_root) write(error_unit,fmt900) "unrecognized bath type"
         call MPI_Finalize(ierr); stop
      end select

   end subroutine SetEmbedding
!--------------------------------------------------------------------------------------
   subroutine SetDephasing(me,Tdeph_mat)
      class(hamk_2bloc_bath_t)  :: me
      real(dp),intent(in)       :: Tdeph_mat(:,:)
      integer :: ik

      do ik=1,me%Nk_loc
         call me%gkba(ik)%SetDephasing(Tdeph_mat=Tdeph_mat)
      end do

   end subroutine SetDephasing
!--------------------------------------------------------------------------------------
   subroutine SetMatrixElements(me,mel)
      class(hamk_2bloc_bath_t)  :: me
      complex(dp),intent(in)    :: mel(:,:)
      integer :: nsize
      integer,allocatable :: displ(:),size_loc(:)

      allocate(me%mel(me%nbnd,me%Nk_loc))
      call GetDisplSize1D(kdist%N_loc,me%nbnd,displ,nsize,size_loc)

      call MPI_Scatterv(mel,size_loc,displ,MPI_DOUBLE_COMPLEX,me%mel,nsize,&
         MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,ierr)

      deallocate(displ,size_loc)      

   end subroutine SetMatrixElements
!--------------------------------------------------------------------------------------
   function GetHam(me,ik,t) result(Ht)
      class(hamk_2bloc_bath_t) :: me
      integer,intent(in)       :: ik
      real(dp),intent(in)      :: t
      complex(dp),dimension(me%nbnd,me%nbnd) :: Ht
      integer :: i,idir
      real(dp) :: AF(3),EF(3)

      Ht = me%Hk0(:,:,ik)
      if(.not.associated(field)) then
         return
      end if

      call field(t,AF,EF)

      do idir=1,3
         Ht = Ht - AF(idir) * me%vk0(:,:,idir,ik)
      end do
      do i=1,me%nbnd
         Ht(i,i) = Ht(i,i) + 0.5_dp * sum(AF**2)
      end do

   end function GetHam
!--------------------------------------------------------------------------------------
   subroutine Update_Gloc(me,tstp)
      class(hamk_2bloc_bath_t) :: me
      integer,intent(in)       :: tstp
      integer :: ik,jt,ncount
      complex(dp),dimension(me%nbnd,me%nbnd) :: Ggtr,Glsr

      me%Gloc_cont = zero

      do ik=1,me%Nk_loc
         !$OMP PARALLEL PRIVATE(Ggtr)
         !$OMP DO SCHEDULE(STATIC,1)
         do jt=0,tstp
            Ggtr = me%GKBA(ik)%GetGreater(tstp,jt)
            me%Gloc_cont(:,:,jt) = me%Gloc_cont(:,:,jt) + Ggtr(:,:) / me%Nk
         end do
         !$OMP END DO
         !$OMP END PARALLEL
      end do

      ncount = me%nbnd * me%nbnd * (tstp + 1)
      call MPI_Allreduce(me%Gloc_cont(1,1,0),me%Gloc%Greater(1,1,0),ncount,MPI_DOUBLE_COMPLEX,&
         MPI_SUM,MPI_COMM_WORLD,ierr)

      me%Gloc_cont = zero
      do ik=1,me%Nk_loc
         !$OMP PARALLEL PRIVATE(Glsr)
         !$OMP DO SCHEDULE(STATIC,1)
         do jt=0,tstp
            Glsr = me%GKBA(ik)%GetLesser(tstp,jt)
            me%Gloc_cont(:,:,jt) = me%Gloc_cont(:,:,jt) + Glsr(:,:) / me%Nk
         end do
         !$OMP END DO
         !$OMP END PARALLEL
      end do

      ncount = me%nbnd * me%nbnd * (tstp + 1)
      call MPI_Allreduce(me%Gloc_cont(1,1,0),me%Gloc%Lesser(1,1,0),ncount,MPI_DOUBLE_COMPLEX,&
         MPI_SUM,MPI_COMM_WORLD,ierr)

   end subroutine Update_Gloc
!--------------------------------------------------------------------------------------
   subroutine Update_Sigma(me,tstp)
      class(hamk_2bloc_bath_t) :: me
      integer,intent(in)       :: tstp
      integer :: jt,i,j,ibath

      call  me%SGM%SetZero()

      !$OMP PARALLEL DEFAULT(SHARED)

      !$OMP DO SCHEDULE(STATIC,1)
      do jt=0,tstp
         call GenSigma_2B_loc(tstp,jt,me%Gloc,me%Uhubb,0.0_dp,1.0_dp,me%SGM)
      end do
      !$OMP END DO

      if(me%Embedding) then

         select case(me%bath_dos)
         case(dos_gauss)
            !$OMP DO SCHEDULE(STATIC,1)
            do jt=0,tstp
               do concurrent(i=1:me%nbnd, j=1:me%nbnd, ibath=1:me%nbaths)
                  me%SGM%Greater(i,j,jt) = me%SGM%Greater(i,j,jt) + me%bcp(i,ibath) * me%bcp(j,ibath) &
                     * me%baths_gauss(ibath)%Gemb_gtr(tstp-jt+me%tgrd%Npts)
                  me%SGM%Lesser(i,j,jt) = me%SGM%Lesser(i,j,jt) + me%bcp(i,ibath) * me%bcp(j,ibath) &
                     * me%baths_gauss(ibath)%Gemb_lsr(tstp-jt+me%tgrd%Npts)
               end do
            end do
            !$OMP END DO 
         case(dos_semi)
            !$OMP DO SCHEDULE(STATIC,1)
            do jt=0,tstp
               do concurrent(i=1:me%nbnd, j=1:me%nbnd, ibath=1:me%nbaths)
                  me%SGM%Greater(i,j,jt) = me%SGM%Greater(i,j,jt) + me%bcp(i,ibath) * me%bcp(j,ibath) &
                     * me%baths_semi(ibath)%Gemb_gtr(tstp-jt+me%tgrd%Npts)
                  me%SGM%Lesser(i,j,jt) = me%SGM%Lesser(i,j,jt) + me%bcp(i,ibath) * me%bcp(j,ibath) &
                     * me%baths_semi(ibath)%Gemb_lsr(tstp-jt+me%tgrd%Npts)
               end do
            end do
            !$OMP END DO    
         end select         
      end if

      !$OMP END PARALLEL

      if(me%AdiabaticSwitch) then
         call SelfEn_MultiplySwitch(tstp,me%tgrd%dX,me%t_startswitch,&
            me%t_endswitch,me%tau_switch,me%SGM)
      end if

   end subroutine Update_Sigma   
!--------------------------------------------------------------------------------------
   subroutine Bootstrapping(me,converged)
      class(hamk_2bloc_bath_t)     :: me
      logical,intent(out)          :: converged
      integer :: ik,iter,it
      real(dp) :: Err,Err_loc,Errk
      type(CntrFun_1time_t),allocatable :: Hk_func(:)
      type(CntrFun_2time_t),allocatable :: Gk_start(:)
      type(CntrFun_2time_t) :: SGM_start

      call Init_1time_array(Hk_func,me%Nk_loc,me%Beta,me%nbnd,kbe%nstart)
      call Init_2time_array(Gk_start,me%Nk_loc,FERM,me%Beta,me%nbnd,kbe%nstart)
      call SGM_start%Init_Keldysh(FERM,me%Beta,me%nbnd,kbe%nstart)

      do ik=1,me%Nk_loc
         do it=0,kbe%nstart
            Hk_func(ik)%Ret(:,:,it) = me%GetHam(ik,me%tgrd%Xpts(it))
         end do
         call me%GKBA(ik)%FillStart(Hk_func(ik)%ret(:,:,0))
      end do

      converged = .false. ; iter = 0; Err = HUGE(1.0d0)
     
      do while(iter < kbe%MaxBootstrapIter .and. Err > kbe%Bootstrap_ErrTol)
         iter = iter + 1

         !----- update polarization & self-energy -----
         do it=0,kbe%nstart
            call me%Update_Gloc(it)
            call me%Update_Sigma(it)
            call CntrSlice_to_Cntr2tm(it,me%SGM,SGM_start,kbe_storage=.false.)
         end do
         !----- bootstrap step -----
         Err_loc = 0.0_dp
         do ik=1,me%Nk_loc
            call GKBA_Start(me%gkba(ik),Hk_func(ik),SGM_start,Gk_start(ik),Errk,iter==1)
            Err_loc = Err_loc + Errk/me%Nk
         end do
         call MPI_Allreduce(Err_loc,Err,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr)

         if(on_root) then
            write(output_unit,fmt148,ADVANCE='NO') 'iteration',iter
            write(output_unit,fmt147) '  |  Error',Err
         end if

      end do 

      converged = Err < kbe%Bootstrap_ErrTol
      
      call Clean_2time_array(Gk_start)
      call SGM_start%Clean()
      call Clean_1time_array(Hk_func)

   end subroutine Bootstrapping
!--------------------------------------------------------------------------------------
   subroutine Timestep(me,tstp)
      class(hamk_2bloc_bath_t) :: me
      integer,intent(in)       :: tstp
      logical :: last_step
      integer :: iter
      integer :: ik
      complex(dp),dimension(me%nbnd,me%nbnd) :: Hk

      !====================================
      !           Predictor
      !====================================
      do ik=1,me%Nk_loc
         call GKBA_Predictor(me%GKBA(ik),tstp)
      end do
      !====================================
      !           Corrector
      !====================================
      CORRLOOP:do iter=1,kbe%NeqIter
         last_step = (iter == kbe%NeqIter)

         do ik=1,me%Nk_loc
            Hk = me%GetHam(ik,me%tgrd%Xpts(tstp+1))
            call me%GKBA(ik)%UpdateH(Hk)
         end do

         call me%Update_Gloc(tstp+1)
         call me%Update_Sigma(tstp+1)

         do ik=1,me%Nk_loc
            call GKBA_Corrector(me%GKBA(ik),tstp,me%SGM,last_step)
         end do

      end do CORRLOOP

   end subroutine Timestep
!--------------------------------------------------------------------------------------
   function BandOcc(me,tstp) result(occ)
      use Mlinalg,only: trace
      class(hamk_2bloc_bath_t) :: me
      integer,intent(in)       :: tstp
      real(dp) :: occ(me%nbnd)
      integer :: i,ik
      real(dp) :: occ_loc(me%nbnd)
      complex(dp) :: Rho_bnd(me%nbnd,me%nbnd)

      occ_loc = 0.0_dp
      do ik=1,me%Nk_loc
         Rho_bnd = util_rotate(me%nbnd,me%vectk(:,:,ik),me%GKBA(ik)%Rho(:,:,tstp),&
            large_size=large_size)
         do i=1,me%nbnd
            occ_loc(i) = occ_loc(i) + dble(Rho_bnd(i,i)) / me%Nk
         end do
      end do

      call MPI_Allreduce(occ_loc,occ,me%nbnd,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr)

   end function BandOcc
!--------------------------------------------------------------------------------------
   function Dipole(me,tstp) result(dip)
      class(hamk_2bloc_bath_t) :: me
      integer,intent(in)       :: tstp
      real(dp) :: dip(3)
      integer :: ik,idir
      real(dp) :: dip_loc(3)

      dip_loc = 0.0_dp
      do ik=1,me%Nk_loc
         do idir=1,3
            dip_loc(idir) = dip_loc(idir) + me%GKBA(ik)%ExpectVal(tstp,me%vk0(:,:,idir,ik))
         end do
      end do

      call MPI_Allreduce(dip_loc,dip,3,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr)
      dip = dip / me%Nk

   end function Dipole
!--------------------------------------------------------------------------------------
   subroutine CalcTARPES(me,tprobe,t0,wgrd,spect,kvecs,pulse_shape,diag_matel)
      use Mlinalg,only: Eye, util_rotate_cc
      class(hamk_2bloc_bath_t)  :: me
      real(dp),intent(in)       :: tprobe,t0
      type(grid1d_t),intent(in) :: wgrd
      real(dp),intent(inout)    :: spect(0:,:)
      real(dp),intent(in),optional :: kvecs(:,:)
      integer,intent(in),optional  :: pulse_shape
      logical,intent(in),optional  :: diag_matel
      logical :: lape,diag_matel_
      integer :: pulse_shape_
      integer :: ik,ik_glob,it,i
      real(dp),allocatable :: spect_loc(:,:)
      complex(dp),allocatable :: matel(:,:)
      integer :: nsize
      integer,allocatable :: displ(:),size_loc(:)
      type(CntrFun_1time_t) :: Hk_func

      lape = present(kvecs)
      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      diag_matel_ = .false.
      if(present(diag_matel)) diag_matel_ = diag_matel

      call Hk_func%Init(me%Beta,me%nbnd,me%tgrd%Npts)

      allocate(spect_loc(0:wgrd%Npts,me%Nk_loc))

      MATEL_SWITCH: if(allocated(me%mel)) then

         allocate(matel(me%nbnd,0:wgrd%Npts))

         if(lape) then
            do ik=1,me%Nk_loc
               ik_glob = kdist%Indx_loc2glob(taskid,ik)
               do i=1,me%nbnd
                  matel(i,0:) = me%mel(i,ik)
               end do
               do it=0,me%tgrd%Npts
                  Hk_func%Ret(:,:,it) = me%GetHam(ik,me%tgrd%Xpts(it))
               end do
               call ProbeSpectrum_PumpPulse_MatrixElements(me%tgrd,wgrd,tprobe,t0,Hk_func,me%GKBA(ik),&
                  kvecs(:,ik_glob),me%Avec_pe,matel,spect_loc(0:,ik),pulse_shape=pulse_shape_,&
                  diag_matel=diag_matel_)
            end do
         else  
            do ik=1,me%Nk_loc
               do i=1,me%nbnd
                  matel(i,0:) = me%mel(i,ik)
               end do
               do it=0,me%tgrd%Npts
                  Hk_func%Ret(:,:,it) = me%GetHam(ik,me%tgrd%Xpts(it))
               end do
               call ProbeSpectrum_MatrixElements(me%tgrd,wgrd,tprobe,t0,Hk_func,me%GKBA(ik),matel,&
                  spect_loc(0:,ik),pulse_shape=pulse_shape_,diag_matel=diag_matel_)
            end do
         end if

         deallocate(matel)

      else
         if(lape) then
            do ik=1,me%Nk_loc
               ik_glob = kdist%Indx_loc2glob(taskid,ik)
               do it=0,me%tgrd%Npts
                  Hk_func%Ret(:,:,it) = me%GetHam(ik,me%tgrd%Xpts(it))
               end do
               call ProbeSpectrum_PumpPulse(me%tgrd,wgrd,tprobe,t0,Hk_func,me%GKBA(ik),&
                  kvecs(:,ik_glob),me%Avec_pe,spect_loc(0:,ik),pulse_shape=pulse_shape_)
            end do
         else  
            do ik=1,me%Nk_loc
               do it=0,me%tgrd%Npts
                  Hk_func%Ret(:,:,it) = me%GetHam(ik,me%tgrd%Xpts(it))
               end do
               call ProbeSpectrum(me%tgrd,wgrd,tprobe,t0,Hk_func,me%GKBA(ik),spect_loc(0:,ik),&
                  pulse_shape=pulse_shape_)
            end do
         end if
      end if MATEL_SWITCH

      allocate(displ(0:ntasks-1),size_loc(0:ntasks-1))
      call GetDisplSize1D(kdist%N_loc,wgrd%Npts+1,displ,nsize,size_loc)

      call MPI_GATHERV(spect_loc,nsize,MPI_DOUBLE_PRECISION,spect,size_loc,displ,&
         MPI_DOUBLE_PRECISION,master,MPI_COMM_WORLD, ierr)

      deallocate(size_loc,displ,spect_loc)
      call Hk_func%Clean()

   end subroutine CalcTARPES
!-------------------------------------------------------------------------------------- 

!======================================================================================
end module Mhamk_2bloc_bath_mpi
