module Mhamk_free_tdarpes
!======================================================================================
   use,intrinsic :: iso_fortran_env,only: output_unit,error_unit
   use mpi
   use omp_lib
   use Mdebug
   use Mdef,only: dp,iu,one,zero
   use Mgrid1d,only: grid1d_t
   use Mlinalg,only: util_rotate,util_rotate_cc
   use Meqval,only: DTrAB,EqFromHam
   use Mevol,only: GenU_CF4, UnitaryStepFBW
   use Mpropagate,only: Propagate_OBE,Propagate_Houston,&
      Propagate_Houston_ext, Propagate_OBE_1bnd, Propagate_Houston_1bnd,&
      Propagate_Houston_multiband
   use Mtdarpes_gkba,only: &
      ProbeSpectrum, ProbeSpectrum_MatrixElements ,&
      ProbeSpectrum_PumpPulse, ProbeSpectrum_PumpPulse_MatrixElements
   use Marray1d_dist,only: dist_array1d_t, GetDisplSize1D
   !---------------------
   !    KBKIT modules
   !---------------------
   use kbkit_greenfun
   !---------------------
   implicit none
   include 'formats.h'
!--------------------------------------------------------------------------------------
   character(len=*),parameter :: fmt_info='(" Info: ",a)'
   integer,parameter :: evol_free=0, evol_obe=1, evol_houst=2, evol_houst_multi=3, &
      evol_houst_ext=4
   ! .. KBE parameters ..
   logical :: large_size
   ! .. external vector potential ..
   procedure(vecpot_efield_func),pointer :: field => null()
   ! .. parallelization ..
   integer,parameter  :: master=0,from_master=1,from_worker=2
   integer :: ntasks,taskid,nthreads,threadid,ierr
   logical :: on_root
   integer :: status(MPI_STATUS_SIZE)
   type(dist_array1d_t),private :: kdist
!--------------------------------------------------------------------------------------
   private
   public :: hamk_tdarpes_t
!--------------------------------------------------------------------------------------
   type hamk_tdarpes_t
      integer  :: nbnd,Nk,Nk_loc
      integer  :: evol=evol_free
      real(dp) :: screen_par,screen_perp,screen_pe_par,screen_pe_perp
      real(dp) :: Beta,MuChem
      real(dp) :: T1,T2
      real(dp),allocatable,dimension(:,:) :: Tmat
      real(dp),allocatable,dimension(:,:) :: Avec_pe
      ! .. grids ..
      type(grid1d_t) :: tgrd
      ! .. Hamiltonian class ..
      complex(dp),allocatable,dimension(:,:)         :: mel
      complex(dp),allocatable,dimension(:,:,:)       :: Hk0
      complex(dp),allocatable,dimension(:,:,:,:)     :: vk0
      complex(dp),allocatable,dimension(:,:,:)       :: vectk
      ! .. contour objects ..
      type(CntrFun_1time_t),allocatable :: Ht(:),Rho(:)
      complex(dp),allocatable :: Glsr(:,:,:,:)
   contains
      procedure,public  :: Init
      procedure,public  :: SetLaserpulse
      procedure,public  :: SetMatrixElements 
      procedure,public  :: Propagate
      procedure,public  :: GetObs
      procedure,public  :: CalcTARPES
      procedure,private :: GetHam
      procedure,public  :: Occupation
      procedure,public  :: Dipole
   end type hamk_tdarpes_t
!--------------------------------------------------------------------------------------
   abstract interface

   subroutine vecpot_efield_func(t,A,E)
      import :: dp
      implicit none
      real(dp),intent(in) :: t
      real(dp),intent(out) :: A(3),E(3) 
   end subroutine vecpot_efield_func

   end interface
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
   subroutine Init(me,Beta,MuChem,Hk0,vk0,screen_dyn,screen_pe,Nt,Tmax,T1,T2,Tmat,evol)
      use Mlinalg,only: EigHE,Eye
      class(hamk_tdarpes_t)         :: me
      real(dp),intent(in)           :: Beta
      real(dp),intent(in)           :: MuChem
      complex(dp),intent(in)        :: Hk0(:,:,:)
      complex(dp),intent(in)        :: vk0(:,:,:,:)
      real(dp),intent(in)           :: screen_dyn(2)
      real(dp),intent(in)           :: screen_pe(2)
      integer,intent(in)            :: Nt
      real(dp),intent(in)           :: Tmax
      real(dp),intent(in),optional  :: T1,T2
      real(dp),intent(in),optional  :: Tmat(:,:)
      integer,intent(in),optional   :: evol
      integer :: ik,ik_glob
      real(dp),allocatable :: epsk(:)
      character(len=32) :: spar
      integer :: nsize
      integer,allocatable :: displ(:),size_loc(:)


      call MPI_COMM_RANK(MPI_COMM_WORLD, taskid, ierr)
      call MPI_COMM_SIZE(MPI_COMM_WORLD, ntasks, ierr)
      on_root = taskid == master

      if(on_root) then
         write(output_unit,fmt50)
         write(output_unit,'(A)') '         Parallelization'
         write(output_unit,fmt50)
         write(spar,*) ntasks
         write(output_unit,fmt_info) "number of ranks: "//trim(adjustl(spar))

         !$OMP PARALLEL PRIVATE(nthreads,threadid)
         threadid = omp_get_thread_num()
         if(threadid == 0) then
            nthreads = omp_get_num_threads()
            write(spar,*) nthreads
            write(output_unit,fmt_info) "number of threads: "//trim(adjustl(spar))
            write(output_unit,*)
         end if
         !$OMP END PARALLEL
      end if

      if(on_root) then
         me%Nk = size(Hk0, dim=3)
         me%nbnd = size(Hk0, dim=1)
      end if

      call MPI_Bcast(me%Nk,1,MPI_INT,master,MPI_COMM_WORLD,ierr)
      call MPI_Bcast(me%nbnd,1,MPI_INT,master,MPI_COMM_WORLD,ierr)

      call kdist%Init(ntasks,taskid,me%Nk)
      me%Nk_loc = kdist%N_loc(taskid)

      allocate(me%Hk0(me%nbnd,me%nbnd,me%Nk_loc))
      allocate(displ(0:ntasks-1),size_loc(0:ntasks-1))
      call GetDisplSize1D(kdist%N_loc,me%nbnd*me%nbnd,displ,nsize,size_loc)

      call MPI_Scatterv(Hk0,size_loc,displ,MPI_DOUBLE_COMPLEX,me%Hk0,nsize,&
         MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,ierr)

      allocate(me%vk0(me%nbnd,me%nbnd,3,me%Nk_loc))
      call GetDisplSize1D(kdist%N_loc,me%nbnd*me%nbnd*3,displ,nsize,size_loc)

      call MPI_Scatterv(vk0,size_loc,displ,MPI_DOUBLE_COMPLEX,me%vk0,nsize,&
         MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,ierr)

      deallocate(displ,size_loc)      

      if(present(T1)) me%T1 = T1
      if(present(T2)) me%T2 = T2
      if(present(evol)) me%evol = evol

      me%Beta = Beta
      me%MuChem = MuChem

      me%screen_par = screen_dyn(1)
      me%screen_perp = screen_dyn(2)
      me%screen_pe_par = screen_pe(1)
      me%screen_pe_perp = screen_pe(2)

      call me%tgrd%Init(Nt,0.0_dp,Tmax)

      large_size = me%nbnd >= 8

      call Init_1time_array(me%Ht,me%Nk_loc,me%Beta,me%nbnd,Nt)
      call Init_1time_array(me%Rho,me%Nk_loc,me%Beta,me%nbnd,Nt)

      do ik=1,me%Nk_loc
         ik_glob = kdist%Indx_Loc2Glob(taskid,ik)
         me%Ht(ik)%Mats = me%Hk0(:,:,ik)
      end do

      allocate(me%vectk(me%nbnd,me%nbnd,me%Nk_loc),epsk(me%nbnd))
      do ik=1,me%Nk_loc
         call EigHE(me%Ht(ik)%Mats, epsk, me%vectk(:,:,ik))
      end do
      deallocate(epsk)

      do ik=1,me%Nk_loc
         call EqFromHam(me%nbnd,me%Beta,me%MuChem,me%Ht(ik)%Mats,me%Rho(ik)%Mats)
         me%Rho(ik)%Ret(:,:,0) = me%Rho(ik)%Mats
      end do

   end subroutine Init
!--------------------------------------------------------------------------------------
   subroutine SetLaserpulse(me,Laserfield)
      class(hamk_tdarpes_t)  :: me
      procedure(vecpot_efield_func) :: Laserfield
      integer :: it
      real(dp) :: AF(3),EF(3)

      field => Laserfield

      allocate(me%Avec_pe(3,0:me%tgrd%Npts)); me%Avec_pe = 0.0_dp
      if(associated(field)) then
         do it=0,me%tgrd%Npts
            call field(me%tgrd%Xpts(it),AF,EF)
            me%Avec_pe(1:2,it) = me%screen_pe_par * AF(1:2)
            me%Avec_pe(3,it) = me%screen_pe_perp * AF(3)
         end do
      end if

   end subroutine SetLaserpulse
!--------------------------------------------------------------------------------------
   subroutine SetMatrixElements(me,mel)
      class(hamk_tdarpes_t)  :: me
      complex(dp),intent(in) :: mel(:,:)
      integer :: nsize
      integer,allocatable :: displ(:),size_loc(:)

      allocate(me%mel(me%nbnd,me%Nk_loc))
      call GetDisplSize1D(kdist%N_loc,me%nbnd,displ,nsize,size_loc)

      call MPI_Scatterv(mel,size_loc,displ,MPI_DOUBLE_COMPLEX,me%mel,nsize,&
         MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,ierr)

      deallocate(displ,size_loc)      

   end subroutine SetMatrixElements
!--------------------------------------------------------------------------------------
   function GetHam(me,ik,t) result(Ht)
      class(hamk_tdarpes_t) :: me
      integer,intent(in)    :: ik
      real(dp),intent(in)   :: t
      complex(dp),dimension(me%nbnd,me%nbnd) :: Ht
      integer :: i,idir
      real(dp) :: AF(3),EF(3),AFscr(3)

      Ht = me%Hk0(:,:,ik)

      if(.not.associated(field)) then
         return
      end if

      call field(t,AF,EF)

      AFscr(1:2) = me%screen_par * AF(1:2)
      AFscr(3) = me%screen_perp * AF(3)

      do idir=1,3
         Ht = Ht - AFscr(idir) * me%vk0(:,:,idir,ik)
      end do

      do i=1,me%nbnd
         Ht(i,i) = Ht(i,i) + 0.5_dp * sum(AF**2)
      end do


   end function GetHam
!--------------------------------------------------------------------------------------
   subroutine Propagate(me)
      use Mdef,only: nfermi
      real(dp),parameter :: c1=0.5_dp-sqrt(3.0_dp)/6.0_dp
      real(dp),parameter :: c2=0.5_dp+sqrt(3.0_dp)/6.0_dp
      class(hamk_tdarpes_t) :: me
      integer :: tstp,ik
      real(dp) :: t1,t2,EF(2),feq(me%nbnd)
      complex(dp),dimension(me%nbnd,me%nbnd) :: H1,H2,Udt

      do ik=1,me%Nk_loc
         do tstp=0,me%tgrd%Npts
            me%Ht(ik)%Ret(:,:,tstp) = me%GetHam(ik,me%tgrd%Xpts(tstp))
         end do
      end do

      select case(me%evol)
      case(evol_free)
         do ik=1,me%Nk_loc
            do tstp=0,me%tgrd%Npts-1
               t1 = (tstp + c1) * me%tgrd%dX
               t2 = (tstp + c2) * me%tgrd%dX
               H1 = me%GetHam(ik,t1)
               H2 = me%GetHam(ik,t2)
               call GenU_CF4(me%tgrd%dX,H1,H2,Udt)
               call UnitaryStepFBW(me%nbnd,Udt,me%Rho(ik)%ret(:,:,tstp),me%Rho(ik)%ret(:,:,tstp+1))
            end do
         end do
      case(evol_obe) 
         if(me%nbnd /= 2) then
            write(error_unit,*) "OBEs only implemented for two-band systems!"
            stop
         end if
         do ik=1,me%Nk_loc
            feq(1) = dble(me%Rho(ik)%Mats(1,1))
            feq(2) = dble(me%Rho(ik)%Mats(2,2))
            call Propagate_OBE(me%tgrd,feq,me%T1,me%T2,me%Ht(ik),me%Rho(ik),band_basis=.true.)
         end do
      case(evol_houst)
         if(me%nbnd /= 2) then
            write(output_unit,*) "Houston OBEs for more than two bands is experimental."
         end if
         do ik=1,me%Nk_loc
            call Propagate_Houston(me%tgrd,me%Beta,me%MuChem,me%T1,me%T2,me%Ht(ik),me%Rho(ik))
         end do
      case(evol_houst_multi)
         do ik=1,me%Nk_loc
            call Propagate_Houston_multiband(me%tgrd,me%Beta,me%MuChem,me%T1,me%T2,me%Ht(ik),me%Rho(ik))
         end do         
      case(evol_houst_ext)
         do ik=1,me%Nk_loc
            call Propagate_Houston_ext(me%tgrd,me%Beta,me%MuChem,me%Tmat,me%Ht(ik),me%Rho(ik))
         end do
      case default
         write(error_unit,*) "Invalid flag for time evolution!"
         call MPI_Finalize(ierr)
         stop
      end select

   end subroutine Propagate
!--------------------------------------------------------------------------------------
   function Occupation(me,tstp) result(nocc)
      class(hamk_tdarpes_t) :: me
      integer,intent(in)   :: tstp
      real(dp) :: nocc(me%nbnd)
      integer :: i,ik
      real(dp) :: nocc_loc(me%nbnd)
      complex(dp),dimension(me%nbnd,me%nbnd) :: rhob

      nocc_loc = 0.0_dp
      do ik=1,me%Nk_loc
         rhob = util_rotate(me%nbnd,me%vectk(:,:,ik),me%Rho(ik)%Ret(:,:,tstp),large_size=.false.)
         do i=1,me%nbnd
            nocc_loc(i) = nocc_loc(i) + dble(Rhob(i,i)) / me%Nk
         end do
      end do

      call MPI_ALLREDUCE(nocc_loc,nocc,me%nbnd,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD, ierr)

   end function Occupation
!--------------------------------------------------------------------------------------
   function Dipole(me,tstp) result(dip)
      class(hamk_tdarpes_t) :: me
      integer,intent(in)   :: tstp
      real(dp) :: dip(3)
      integer :: idir,ik
      real(dp) :: dip_loc(3)

      dip_loc = 0.0_dp
      do ik=1,me%Nk_loc
         do idir=1,3
            dip_loc(idir) = dip_loc(idir) + DTrAB(me%Rho(ik)%Ret(:,:,tstp),me%vk0(:,:,idir,ik))/me%Nk
         end do
      end do

      call MPI_ALLREDUCE(dip_loc,dip,3,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD, ierr)

   end function Dipole
!--------------------------------------------------------------------------------------
   subroutine GetObs(me,dip,occ)
      class(hamk_tdarpes_t) :: me
      real(dp),allocatable,intent(out) :: dip(:,:),occ(:,:)
      integer :: tstp

      allocate(dip(3,0:me%tgrd%Npts),occ(me%nbnd,0:me%tgrd%Npts))

      do tstp=0,me%tgrd%Npts
         dip(:,tstp) = me%Dipole(tstp)
         occ(:,tstp) = me%Occupation(tstp)
      end do

   end subroutine GetObs
!-------------------------------------------------------------------------------------- 
   subroutine CalcTARPES(me,tprobe,t0,wgrd,spect,kvecs,pulse_shape,diag_matel)
      use Mlinalg,only: Eye, util_rotate_cc
      class(hamk_tdarpes_t)      :: me
      real(dp),intent(in)       :: tprobe,t0
      type(grid1d_t),intent(in) :: wgrd
      real(dp),intent(inout)    :: spect(0:,:)
      real(dp),intent(in),optional :: kvecs(:,:)
      integer,intent(in),optional  :: pulse_shape
      logical,intent(in),optional  :: diag_matel
      logical :: lape,diag_matel_
      integer :: pulse_shape_
      integer :: ik,ik_glob,i
      real(dp),allocatable :: spect_loc(:,:)
      complex(dp),allocatable :: matel(:,:)
      integer :: nsize
      integer,allocatable :: displ(:),size_loc(:)

      lape = present(kvecs)
      pulse_shape_ = 0
      if(present(pulse_shape)) pulse_shape_ = pulse_shape

      diag_matel_ = .false.
      if(present(diag_matel)) diag_matel_ = diag_matel

      allocate(spect_loc(0:wgrd%Npts,me%Nk_loc))

      MATEL_SWITCH: if(allocated(me%mel)) then

         allocate(matel(me%nbnd,0:wgrd%Npts))

         if(lape) then
            do ik=1,me%Nk_loc
               ik_glob = kdist%Indx_loc2glob(taskid,ik)
               do i=1,me%nbnd
                  matel(i,0:) = me%mel(i,ik)
               end do
               call ProbeSpectrum_PumpPulse_MatrixElements(me%tgrd,wgrd,tprobe,t0,me%Ht(ik),me%Rho(ik),&
                  kvecs(:,ik_glob),me%Avec_pe,matel,spect_loc(0:,ik),pulse_shape=pulse_shape_,&
                  diag_matel=diag_matel_)
            end do
         else  
            do ik=1,me%Nk_loc
               do i=1,me%nbnd
                  matel(i,0:) = me%mel(i,ik)
               end do
               call ProbeSpectrum_MatrixElements(me%tgrd,wgrd,tprobe,t0,me%Ht(ik),me%Rho(ik),matel,&
                  spect_loc(0:,ik),pulse_shape=pulse_shape_,diag_matel=diag_matel_)
            end do
         end if

         deallocate(matel)

      else
         if(lape) then
            do ik=1,me%Nk_loc
               ik_glob = kdist%Indx_loc2glob(taskid,ik)
               call ProbeSpectrum_PumpPulse(me%tgrd,wgrd,tprobe,t0,me%Ht(ik),me%Rho(ik),&
                  kvecs(:,ik_glob),me%Avec_pe,spect_loc(0:,ik),pulse_shape=pulse_shape_)
            end do
         else  
            do ik=1,me%Nk_loc
               call ProbeSpectrum(me%tgrd,wgrd,tprobe,t0,me%Ht(ik),me%Rho(ik),spect_loc(0:,ik),&
                  pulse_shape=pulse_shape_)
            end do
         end if
      end if MATEL_SWITCH
     
      allocate(displ(0:ntasks-1),size_loc(0:ntasks-1))
      call GetDisplSize1D(kdist%N_loc,wgrd%Npts+1,displ,nsize,size_loc)

      call MPI_GATHERV(spect_loc,nsize,MPI_DOUBLE_PRECISION,spect,size_loc,displ,&
         MPI_DOUBLE_PRECISION,master,MPI_COMM_WORLD, ierr)

      deallocate(size_loc,displ,spect_loc)

   end subroutine CalcTARPES
!-------------------------------------------------------------------------------------- 

!======================================================================================
end module Mhamk_free_tdarpes
