program ham_free_tdarpes
!======================================================================================
   use,intrinsic :: iso_fortran_env,only: output_unit,error_unit
   use omp_lib
   use Mdebug
   use Mdef,only: dp,zero,iu
   use Mtime,only:&
      Timer_Act => TAct,&
      Timer_SetName => SetName,&
      Timer_Run => TRun,&
      Timer_stop => Tstop,&
      Timer_DTShow => DTShow
   use Mgrid1d,only: grid1d_t
   use Mlaserpulse,only: LaserPulse_spline_t
   use Mham_free_tdarpes,only: ham_tdarpes_t
   implicit none
   include 'formats.h'
!--------------------------------------------------------------------------------------
   ! -- constants --
   character(len=*),parameter :: fmt_time='(a,"/",a,"/",a," at ",a,":",a,":",a)'
   character(len=*),parameter :: fmt_info='(" Info: ",a)'
   character(len=*),parameter :: fmt_input='(" Input: [",a,"]")'
   integer,parameter :: evol_free=0, evol_obe=1, evol_houst=2, evol_houst_ext=3
   ! -- timing --
   character(len=8) :: date
   character(len=10) :: time
   ! -- for reading i/o --
   logical :: PrintToFile = .false.
   integer :: Narg,unit_inp
   character(len=255) :: FlIn,FlOutPref
   ! -- input variables --
   real(dp)           :: MuChem,Beta
   real(dp)           :: kvec(3),screen_surf(2),screen_pe(2)
   real(dp)           :: T1=1.0e8_dp, T2=1.0e8_dp
   character(len=255) :: file_ham,file_pmom
   character(len=255) :: file_field='',file_tmat='',file_matel=''
   namelist/SYSPARAMS/MuChem,Beta,kvec,screen_surf,screen_pe,T1,T2,&
      file_ham,file_pmom,file_field,file_tmat,file_matel
   !......................................
   logical  :: egkba_mode=.false.,store_obs=.false.,retarded_mode=.false.
   integer  :: spectrum_mode=0,evol=evol_free,output_every=10
   integer  :: Nt
   real(dp) :: Tmax
   namelist/TIMEPARAMS/Nt,Tmax,spectrum_mode,evol,egkba_mode,store_obs,retarded_mode,&
      output_every
   !......................................
   logical  :: lape=.false.,diag_matel=.false.
   integer  :: Nomega,num_probe_t0,pulse_shape=0
   real(dp) :: probe_dur,omega_min,omega_max
   namelist/PROBEPARAMS/lape,Nomega,num_probe_t0,omega_min,omega_max,probe_dur,&
      pulse_shape,diag_matel
   !......................................
   real(dp),allocatable :: probe_t0(:)
   namelist/PROBETIMES/probe_t0
   !......................................
   ! -- internal variables --
   logical  :: file_ok
   logical  :: ApplyField=.false.,prop_ext=.false.,matel_mode=.false.
   integer  :: nst,n
   type(grid1d_t) :: wgrd
   real(dp),allocatable,dimension(:,:)      :: spect,dip,occ
   real(dp),allocatable,dimension(:,:)      :: Tmat
   complex(dp),allocatable,dimension(:)     :: mel
   complex(dp),allocatable,dimension(:,:)   :: H0
   complex(dp),allocatable,dimension(:,:,:) :: Pmom
   type(LaserPulse_spline_t)                :: pulse_x,pulse_y,pulse_z
   type(ham_tdarpes_t)                      :: lattsys
!--------------------------------------------------------------------------------------
   write(output_unit,'(A)') '+------------------------------------------------------+'
   write(output_unit,'(A)') '|            Photoemission dynamics: Free              |'
   write(output_unit,'(A)') '+------------------------------------------------------+'
   write(output_unit,*)

   call date_and_time(date=date,time=time)
   write(output_unit,'(a)',advance='no') '  Calculation started on '
   write(output_unit,fmt_time) date(1:4),date(5:6),date(7:8),time(1:2),time(3:4),time(5:6)
   write(output_unit,*)

   call Timer_Act
   call Timer_SetName('total',1)
   call Timer_Run(N=1)
!--------------------------------------------------------------------------------------
!                               ++  Read input ++
!--------------------------------------------------------------------------------------
   call Timer_SetName('Reading input',2); call Timer_Run(N=2)

   Narg=command_argument_count()
   if(Narg>=1) then
      call get_command_argument(1,FlIn)
      open(newunit=unit_inp,file=trim(FlIn),status='OLD',action='READ')
      read(unit_inp,nml=SYSPARAMS) ; rewind(unit_inp)
      read(unit_inp,nml=TIMEPARAMS) ; rewind(unit_inp)
      read(unit_inp,nml=PROBEPARAMS) ; rewind(unit_inp)
      allocate(probe_t0(num_probe_t0))
      read(unit_inp,nml=PROBETIMES)
      close(unit_inp)
   else
      write(output_unit,fmt900) 'Please provide a namelist input file.'
      stop
   end if

   if(Narg>=2) then
      call get_command_argument(2,FlOutPref)
      PrintToFile=.true.
   end if
   if((.not.PrintToFile)) then
      write(output_unit,fmt_info) 'No output prefix given. No output will be produced.'
   end if

   inquire(file=trim(file_ham),exist=file_ok)
   if(.not.file_ok) then
      write(error_unit,fmt900) 'Input file does not exist: '//trim(file_ham); stop
   end if
   write(output_unit,fmt_input) 'Hamiltionian from file: '//trim(file_ham)
   call ReadHam(file_ham,nst,H0)

   inquire(file=trim(file_pmom),exist=file_ok)
   if(.not.file_ok) then
      write(error_unit,fmt900) 'Input file does not exist: '//trim(file_pmom); stop
   end if
   write(output_unit,fmt_input) 'Momentum matrix from file: '//trim(file_pmom)
   call ReadMomentum(file_pmom,Pmom)

   if(len_trim(file_matel) > 0) then
      inquire(file=trim(file_matel),exist=file_ok)
      if(.not.file_ok) then
         write(error_unit,fmt900) 'Input file does not exist: '//trim(file_matel); stop
      end if
      write(output_unit,fmt_input) 'Matrix elements from file: '//trim(file_matel)
      call ReadMatrixElements(file_matel,mel)      
      matel_mode = .true.
   end if

   if(evol == evol_houst_ext .and. len_trim(file_tmat) > 0) then
      inquire(file=trim(file_tmat),exist=file_ok)
      if(.not.file_ok) then
         write(error_unit,fmt900) 'Input file does not exist: '//trim(file_tmat); stop
      end if
      write(output_unit,fmt_input) 'Dissipation matrix from file: '//trim(file_tmat)
      call ReadTmat(file_tmat,Tmat)
      prop_ext = .true.
   end if

   ApplyField = len_trim(file_field) > 0
   if(ApplyField) then
      inquire(file=trim(file_field),exist=file_ok)
      if(.not.file_ok) then
         write(error_unit,fmt900) 'Input file does not exist: '//trim(file_field); stop
      end if
      write(output_unit,fmt_input) 'External field from file: '//trim(file_field)
      call pulse_x%Load_ElectricField(file_field,usecol=2,CalcAfield=.true.)
      call pulse_y%Load_ElectricField(file_field,usecol=3,CalcAfield=.true.)
      call pulse_z%Load_ElectricField(file_field,usecol=4,CalcAfield=.true.)
   end if
   
   write(output_unit,*)
   call Timer_Stop(N=2); call Timer_DTShow(N=2)
!--------------------------------------------------------------------------------------
!                               ++  Prepare ++
!--------------------------------------------------------------------------------------
   if(prop_ext) then
      call lattsys%Init(Beta,MuChem,H0,Pmom,screen_surf,screen_pe,Nt,Tmax,&
         Tmat=Tmat,evol=evol,retarded_mode=retarded_mode)
   else
      call lattsys%Init(Beta,MuChem,H0,Pmom,screen_surf,screen_pe,Nt,Tmax,&
         T1=T1,T2=T2,evol=evol,retarded_mode=retarded_mode)
   end if

   if(ApplyField) call lattsys%SetLaserPulse(external_field)   

   if(matel_mode) call lattsys%SetMatrixElements(mel)
!--------------------------------------------------------------------------------------
!                         ++  Time propagation ++
!--------------------------------------------------------------------------------------
   call Timer_SetName('propagation',2); call Timer_Run(N=2)
   call lattsys%Propagate()
   if(store_obs) call lattsys%GetObs(dip,occ)
   call Timer_Stop(N=2); call Timer_DTShow(N=2)

   call wgrd%Init(NOmega,Omega_min,Omega_max)

   call Timer_SetName('spectrum',2); call Timer_Run(N=2)
   allocate(spect(0:Nomega,num_probe_t0))
   do n=1,num_probe_t0
      if(lape) then
         call lattsys%CalcTARPES(probe_dur,probe_t0(n),wgrd,spect(0:,n),&
            kvec=kvec,pulse_shape=pulse_shape,diag_matel=diag_matel)
      else
         call lattsys%CalcTARPES(probe_dur,probe_t0(n),wgrd,spect(0:,n),&
            pulse_shape=pulse_shape,diag_matel=diag_matel)
      end if
   end do
   call Timer_Stop(N=2); call Timer_DTShow(N=2)      
!--------------------------------------------------------------------------------------
!                             ++  Output ++
!--------------------------------------------------------------------------------------
   if(PrintToFile) then
      if(spectrum_mode <= 2) then
         call SaveSpectrum(trim(FlOutPref)//'_spectrum.h5',num_probe_t0,probe_t0,wgrd,spect)
      end if
      if(store_obs) then
         call SaveObs(trim(FlOutPref)//'_observables.h5',Nt,Tmax,dip,occ,output_every)
      end if
   end if

   write(output_unit,*)
   call Timer_Stop(N=1)
   call Timer_DTShow(N=1)
   write(output_unit,'(A)') '+------------------------------------------------------+'
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
   subroutine external_field(t,AF,EF)
      real(dp),intent(in)   :: t
      real(dp),intent(out) :: AF(3),EF(3)

      AF = 0.0_dp; EF = 0.0_dp

      if(ApplyField) then
         AF(1) = pulse_x%Afield(t)
         AF(2) = pulse_y%Afield(t)
         AF(3) = pulse_z%Afield(t)
         EF(1) = pulse_x%Efield(t)
         EF(2) = pulse_y%Efield(t)
         EF(3) = pulse_z%Efield(t)
      end if

   end subroutine external_field
!--------------------------------------------------------------------------------------
   subroutine ReadTmat(fname,Tmat)
      use Mhdf5_utils
      character(len=*),intent(in)      :: fname
      real(dp),allocatable,intent(out) :: Tmat(:,:)
      integer(HID_t) :: file_id

      call hdf_open_file(file_id, trim(fname), STATUS='OLD', ACTION='READ')
      call hdf_read_attribute(file_id,'','nst', nst)
      allocate(Tmat(nst,nst))
      call hdf_read_dataset(file_id,'tmat',Tmat)
      call hdf_close_file(file_id)

   end subroutine ReadTmat
!--------------------------------------------------------------------------------------
   subroutine ReadHam(fname,nst,H0)
      use Mhdf5_utils
      character(len=*),intent(in)         :: fname
      integer,intent(out)                 :: nst
      complex(dp),allocatable,intent(out) :: H0(:,:)
      integer(HID_t) :: file_id
      real(dp),allocatable :: rdata(:,:)

      call hdf_open_file(file_id, trim(fname), STATUS='OLD', ACTION='READ')

      call hdf_read_attribute(file_id,'','nst', nst)
      allocate(H0(nst,nst))
      allocate(rdata(nst,nst))

      call hdf_read_dataset(file_id,'real',rdata); H0 = rdata
      call hdf_read_dataset(file_id,'imag',rdata); H0 = H0 + iu*rdata

      call hdf_close_file(file_id)

      deallocate(rdata)

   end subroutine ReadHam
!--------------------------------------------------------------------------------------
   subroutine ReadMomentum(fname,Pmom)
      use Mhdf5_utils
      character(len=*),intent(in) :: fname
      complex(dp),allocatable,intent(out) :: Pmom(:,:,:)
      integer(HID_t) :: file_id
      integer :: nst
      real(dp),allocatable :: rdata(:,:,:)

      call hdf_open_file(file_id, trim(fname), STATUS='OLD', ACTION='READ')

      call hdf_read_attribute(file_id,'','nst', nst)
      allocate(Pmom(nst,nst,3))
      allocate(rdata(nst,nst,3))

      call hdf_read_dataset(file_id,'real',rdata); Pmom = rdata
      call hdf_read_dataset(file_id,'imag',rdata); Pmom = Pmom + iu*rdata

      call hdf_close_file(file_id)

      deallocate(rdata)

   end subroutine ReadMomentum
!--------------------------------------------------------------------------------------
   subroutine ReadMatrixElements(fname,mel)
      use Mhdf5_utils
      character(len=*),intent(in) :: fname
      complex(dp),allocatable,intent(out) :: mel(:)
      integer(HID_t) :: file_id
      integer :: nst
      real(dp),allocatable :: rdata(:)

      call hdf_open_file(file_id, trim(fname), STATUS='OLD', ACTION='READ')
      call hdf_read_attribute(file_id,'','nst', nst)      
      allocate(mel(nst),rdata(nst))

      call hdf_read_dataset(file_id,'real',rdata); mel = rdata
      call hdf_read_dataset(file_id,'imag',rdata); mel = mel + iu*rdata

      call hdf_close_file(file_id)

      deallocate(rdata)

   end subroutine ReadMatrixElements
!--------------------------------------------------------------------------------------
   subroutine SaveSpectrum(fname,Nprobe,probe_ts,wgrd,spect)
      use Mhdf5_utils
      character(len=*),intent(in) :: fname
      integer,intent(in)  :: Nprobe
      real(dp),intent(in) :: probe_ts(:)
      type(grid1d_t),intent(in) :: wgrd
      real(dp),intent(in) :: spect(0:,:)
      integer(HID_t) :: file_id

      call hdf_open_file(file_id, trim(fname), STATUS='NEW')
      call hdf_write_attribute(file_id,'','nprobe', Nprobe)
      call hdf_write_attribute(file_id,'','nomega', wgrd%Npts+1)
      call hdf_write_attribute(file_id,'','omega_min', wgrd%Xmin)
      call hdf_write_attribute(file_id,'','omega_max', wgrd%Xmax)
      call hdf_write_dataset(file_id,'probe_ts',probe_ts)
      call hdf_write_dataset(file_id,'spectrum',spect)
      call hdf_close_file(file_id)

   end subroutine SaveSpectrum
!--------------------------------------------------------------------------------------
   subroutine SaveObs(fname,Nt,Tmax,dip,occ,output_every)
      use Mhdf5_utils
      character(len=*),intent(in) :: fname
      integer,intent(in)  :: Nt
      real(dp),intent(in) :: Tmax
      real(dp),intent(in) :: dip(:,0:), occ(:,0:)
      integer,intent(in)  :: output_every
      integer(HID_t) :: file_id
      integer :: it,Nsteps,istp
      real(dp) :: dt
      real(dp),allocatable :: dip_stp(:,:),occ_stp(:,:)

      Nsteps = 0
      do it=1,Nt+1
         if(mod(it,output_every) == 0) Nsteps = Nsteps+1
      end do

      allocate(dip_stp(3,Nsteps),occ_stp(nst,Nsteps))
      istp = 0
      do it=1,Nt+1
         if(mod(it,output_every) == 0) then
            istp = istp + 1
            dip_stp(:,istp) = dip(:,it)
            occ_stp(:,istp) = occ(:,it)
         end if
      end do

      dt = Tmax/dble(Nt)

      call hdf_open_file(file_id, trim(fname), STATUS='NEW')
      call hdf_write_attribute(file_id,'','nt', Nsteps+1)
      call hdf_write_attribute(file_id,'','tmax', Nsteps*output_every*dt)
      call hdf_write_dataset(file_id,'dip',dip_stp)
      call hdf_write_dataset(file_id,'occ',occ_stp)
      call hdf_close_file(file_id)

      deallocate(dip_stp,occ_stp)

   end subroutine SaveObs
!--------------------------------------------------------------------------------------


!======================================================================================
end program ham_free_tdarpes
