module Madiabatic
!======================================================================================
  use Mdef,only:&
       dp
  use kbkit_greenfun,only:&
       CntrFun_timeslice_t,&
       CntrFun_2time_t
  implicit none
!--------------------------------------------------------------------------------------
  private
  public &
       AdiabaticSwitch,&
       SelfEn_MultiplySwitch
!--------------------------------------------------------------------------------------
  interface SelfEn_MultiplySwitch
     module procedure SelfEn_MultiplySwitch_tstp,SelfEn_MultiplySwitch_start,&
        SelfEn_MultiplySwitch_tstp_j
  end interface SelfEn_MultiplySwitch
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
  pure real(dp) function save_exp(x)
    real(dp),parameter  :: expmax=50.0_dp,expmin=1.0e-10_dp
    real(dp),intent(in) :: x

    if(x < -expmax) then
       save_exp = 0.0_dp
       return
    end if

    if(abs(x) < expmin) then
       save_exp = 1.0_dp
       return
    end if

    save_exp = exp(x)
    
  end function save_exp
!--------------------------------------------------------------------------------------
  pure real(dp) function AdiabaticSwitch(t_startswitch,t_endswitch,tau,t)
    real(dp),parameter  :: eps5=1.0e-3_dp
    real(dp),parameter  :: expmax=30.0_dp,expmin=1.0e-10_dp
    real(dp),intent(in) :: t_startswitch,t_endswitch,tau,t
    real(dp)::A,tH,dlt_t,tl
    
    tH=0.5_dp*(t_endswitch-t_startswitch)
  
    if(t < t_startswitch + eps5*tH) then
       AdiabaticSwitch=0.0_dp
       return
    end if

    if(t > t_endswitch - eps5*tH) then
       AdiabaticSwitch=1.0_dp
       return
    end if

    dlt_t = t - t_startswitch
    tL = tau*log(2.0_dp)

   
    A = -(tH/dlt_t)*save_exp((dlt_t-tH)*(2*tH-tL)/(dlt_t-2*tH)/tL)
    AdiabaticSwitch = save_exp(log(2.0_dp)*A)
    
  end function AdiabaticSwitch
!--------------------------------------------------------------------------------------
  pure subroutine SelfEn_MultiplySwitch_tstp(it,dt,t_startswitch,t_endswitch,tau_switch,SGM)
    !.................................
    integer,intent(in)                      :: it
    real(dp),intent(in)                     :: dt,t_startswitch,t_endswitch,tau_switch
    type(CntrFun_timeslice_t),intent(inout) :: SGM
    !.................................
    integer  :: jt
    real(dp) :: t1,t2,f1,f2
    !.................................

    t1=it*dt
    f1=AdiabaticSwitch(t_startswitch,t_endswitch,tau_switch,t1)
    do jt=0,it
       t2=jt*dt
       f2=AdiabaticSwitch(t_startswitch,t_endswitch,tau_switch,t2)
       SGM%Lesser(:,:,jt)=f1*f2*SGM%Lesser(:,:,jt)
       SGM%Greater(:,:,jt)=f1*f2*SGM%Greater(:,:,jt)
    end do

  end subroutine SelfEn_MultiplySwitch_Tstp
!--------------------------------------------------------------------------------------
  pure subroutine SelfEn_MultiplySwitch_tstp_j(it,jt,dt,t_startswitch,t_endswitch,tau_switch,SGM)
    !.................................
    integer,intent(in)                      :: it,jt
    real(dp),intent(in)                     :: dt,t_startswitch,t_endswitch,tau_switch
    type(CntrFun_timeslice_t),intent(inout) :: SGM
    !.................................
    real(dp) :: t1,t2,f1,f2
    !.................................

    t1=it*dt
    f1=AdiabaticSwitch(t_startswitch,t_endswitch,tau_switch,t1)
    t2=jt*dt
    f2=AdiabaticSwitch(t_startswitch,t_endswitch,tau_switch,t2)
    SGM%Lesser(:,:,jt)=f1*f2*SGM%Lesser(:,:,jt)
    SGM%Greater(:,:,jt)=f1*f2*SGM%Greater(:,:,jt)

  end subroutine SelfEn_MultiplySwitch_Tstp_j
!--------------------------------------------------------------------------------------
  pure subroutine SelfEn_MultiplySwitch_start(dt,t_startswitch,t_endswitch,tau_switch,SGM)
    integer,parameter :: nstart=4
    !.................................
    real(dp),intent(in)                     :: dt,t_startswitch,t_endswitch,tau_switch
    type(CntrFun_2time_t),intent(inout)     :: SGM
    !.................................
    integer  :: it,jt
    real(dp) :: t1,t2,f1,f2
    !.................................

    do it=0,nstart
       t1=it*dt
       f1=AdiabaticSwitch(t_startswitch,t_endswitch,tau_switch,t1)
       do jt=0,nstart
          t2=jt*dt
          f2=AdiabaticSwitch(t_startswitch,t_endswitch,tau_switch,t2)
          SGM%Lesser(:,:,it,jt)=f1*f2*SGM%Lesser(:,:,it,jt)
          SGM%Greater(:,:,it,jt)=f1*f2*SGM%Greater(:,:,it,jt)
       end do
    end do
       
  end subroutine SelfEn_MultiplySwitch_Start
!--------------------------------------------------------------------------------------
  
!======================================================================================
end module Madiabatic
