module Membedding
!======================================================================================
   use Mdebug
   use Mdef,only:&
      dp
   use Mgrid1d,only:&
      grid1d_t
   !---------------------
   !    KBKIT modules
   !---------------------
   use kbkit_greenfun
   use kbkit_spectral
   !---------------------
   implicit none
!--------------------------------------------------------------------------------------
   private
   public :: semicircle_bath_t, gauss_bath_t
!--------------------------------------------------------------------------------------
   type semicircle_bath_t
      integer  :: FermBos,Nt
      real(dp) :: Beta,MuChem,eps0,DeltaW 
      type(CntrFun_2time_t)     :: Gemb
      type(CntrFun_timeslice_t) :: Gemb_slice
      complex(dp),allocatable   :: Gemb_gtr(:),Gemb_lsr(:)
   contains
      procedure,public :: Init => semicircle_Init
      procedure,public :: Clean => semicircle_Clean
      procedure,public :: GetDeltaWeight => semicircle_GetDeltaWeight
      procedure,public :: CalcGreen => semicircle_CalcGreen
      procedure,public :: CalcGreen_slice => semicircle_CalcGreen_slice
      procedure,public :: CalcGreen_diff => semicircle_CalcGreen_diff
   end type semicircle_bath_t

   type gauss_bath_t
      integer  :: FermBos,Nt
      real(dp) :: Beta,MuChem,eps0,DeltaW 
      type(CntrFun_2time_t)     :: Gemb
      type(CntrFun_timeslice_t) :: Gemb_slice
      complex(dp),allocatable   :: Gemb_gtr(:),Gemb_lsr(:)
   contains
      procedure,public :: Init => gauss_Init
      procedure,public :: CalcGreen => gauss_CalcGreen
      procedure,public :: CalcGreen_slice => gauss_CalcGreen_slice
      procedure,public :: CalcGreen_diff => gauss_CalcGreen_diff
   end type gauss_bath_t
!--------------------------------------------------------------------------------------
   real(dp),parameter :: pi_ = 3.1415926535897932384626433832795_dp
!--------------------------------------------------------------------------------------
contains
!--------------------------------------------------------------------------------------
   subroutine semicircle_Init(me,FermBos,Beta,MuChem,eps0,DeltaW,Nt,store_mode)
      class(semicircle_bath_t) :: me
      integer,intent(in)       :: FermBos
      real(dp),intent(in)      :: Beta
      real(dp),intent(in)      :: MuChem
      real(dp),intent(in)      :: eps0
      real(dp),intent(in)      :: DeltaW
      integer,intent(in)       :: Nt
      integer,intent(in),optional :: store_mode
      integer :: store_mode_

      me%FermBos = FermBos
      me%Beta = Beta
      me%MuChem = MuChem
      me%eps0 = eps0
      me%DeltaW = DeltaW
      me%Nt = Nt

      store_mode_ = 0
      if(present(store_mode)) store_mode_ = store_mode

      select case(store_mode_)
      case(0)
         call me%Gemb%Init_Keldysh(FermBos,Beta,1,Nt)
      case(1)
         call me%Gemb_slice%Init_Keldysh(FermBos,Beta,1,Nt)
      case(2)
         allocate(me%Gemb_gtr(0:2*Nt),me%Gemb_lsr(0:2*Nt))
      end select

   end subroutine semicircle_Init
!--------------------------------------------------------------------------------------
   subroutine semicircle_Clean(me)
      class(semicircle_bath_t) :: me

      if(allocated(me%Gemb%Lesser)) call me%Gemb%Clean()
      if(allocated(me%Gemb_slice%Lesser)) call me%Gemb_slice%Clean()
      if(allocated(me%Gemb_lsr)) deallocate(me%Gemb_lsr)
      if(allocated(me%Gemb_gtr)) deallocate(me%Gemb_gtr)

   end subroutine semicircle_Clean
!--------------------------------------------------------------------------------------
   function semicircle_GetDeltaWeight(me) result(gmm)
      class(semicircle_bath_t) :: me
      real(dp) :: gmm

      gmm = 4.0_dp/me%DeltaW

   end function semicircle_GetDeltaWeight
!--------------------------------------------------------------------------------------
   subroutine semicircle_CalcGreen(me,dt)
      class(semicircle_bath_t) :: me
      real(dp),intent(in)      :: dt
      type(grid1d_t) :: tgrd

      call tgrd%Init(me%Gemb%Nt1,0.0_dp,me%Gemb%Nt1 * dt)

      call Green_from_spectral(bath_dos,me%eps0-me%DeltaW, &
         me%eps0+me%DeltaW,me%MuChem,tgrd,me%Gemb)

      ! call me%Gemb%SaveToHDF5('out/gemb.h5')

      call tgrd%Clean()

   !................................
   contains
   !................................
   pure real(dp) function bath_dos(w)
      real(dp),intent(in) :: w

      if(w > me%Eps0 - me%DeltaW .and. w < me%Eps0 + me%DeltaW) then
         bath_dos = 2.0_dp/(PI_*me%DeltaW) &
              * sqrt(1.0_dp-((w-me%Eps0)/me%DeltaW)**2)
      else
         bath_dos = 0.0_dp
      end if

   end function bath_dos
   !................................   
   end subroutine semicircle_CalcGreen
!--------------------------------------------------------------------------------------
   subroutine semicircle_CalcGreen_slice(me,dt)
      class(semicircle_bath_t) :: me
      real(dp),intent(in)      :: dt
      type(grid1d_t) :: tgrd

      call tgrd%Init(me%Gemb_slice%Nt,0.0_dp,me%Gemb_slice%Nt * dt)

      call Green_from_spectral(bath_dos,me%eps0-me%DeltaW, &
         me%eps0+me%DeltaW,me%MuChem,tgrd,me%Gemb_slice)

      call tgrd%Clean()

   !................................
   contains
   !................................
   pure real(dp) function bath_dos(w)
      real(dp),intent(in) :: w

      if(w > me%Eps0 - me%DeltaW .and. w < me%Eps0 + me%DeltaW) then
         bath_dos = 2.0_dp/(PI_*me%DeltaW) &
              * sqrt(1.0_dp-((w-me%Eps0)/me%DeltaW)**2)
      else
         bath_dos = 0.0_dp
      end if

   end function bath_dos
   !................................   
   end subroutine semicircle_CalcGreen_slice
!--------------------------------------------------------------------------------------
   subroutine semicircle_CalcGreen_diff(me,dt)
      class(semicircle_bath_t) :: me
      real(dp),intent(in)      :: dt
      type(grid1d_t) :: tgrd

      call tgrd%Init(me%Nt,0.0_dp,me%Nt * dt)

      call Green_from_spectral(bath_dos,me%eps0-me%DeltaW, &
         me%eps0+me%DeltaW,me%MuChem,me%Beta,me%FermBos,tgrd,me%Gemb_gtr,me%Gemb_lsr)

      call tgrd%Clean()

   !................................
   contains
   !................................
   pure real(dp) function bath_dos(w)
      real(dp),intent(in) :: w

      if(w > me%Eps0 - me%DeltaW .and. w < me%Eps0 + me%DeltaW) then
         bath_dos = 2.0_dp/(PI_*me%DeltaW) &
              * sqrt(1.0_dp-((w-me%Eps0)/me%DeltaW)**2)
      else
         bath_dos = 0.0_dp
      end if

   end function bath_dos
   !................................   
   end subroutine semicircle_CalcGreen_diff
!--------------------------------------------------------------------------------------

!--------------------------------------------------------------------------------------
   subroutine gauss_Init(me,FermBos,Beta,MuChem,eps0,DeltaW,Nt,store_mode)
      class(gauss_bath_t)      :: me
      integer,intent(in)       :: FermBos
      real(dp),intent(in)      :: Beta
      real(dp),intent(in)      :: MuChem
      real(dp),intent(in)      :: eps0
      real(dp),intent(in)      :: DeltaW
      integer,intent(in)       :: Nt
      integer,intent(in),optional :: store_mode
      integer :: store_mode_

      me%FermBos = FermBos
      me%Beta = Beta
      me%MuChem = MuChem
      me%eps0 = eps0
      me%DeltaW = DeltaW
      me%Nt = Nt

      store_mode_ = 0
      if(present(store_mode)) store_mode_ = store_mode

      select case(store_mode_)
      case(0)
         call me%Gemb%Init_Keldysh(FermBos,Beta,1,Nt)
      case(1)
         call me%Gemb_slice%Init_Keldysh(FermBos,Beta,1,Nt)
      case(2)
         allocate(me%Gemb_gtr(0:2*Nt),me%Gemb_lsr(0:2*Nt))
      end select

   end subroutine gauss_Init
!--------------------------------------------------------------------------------------
   subroutine gauss_CalcGreen(me,dt)
      class(gauss_bath_t)  :: me
      real(dp),intent(in)  :: dt
      type(grid1d_t) :: tgrd

      call tgrd%Init(me%Gemb%Nt1,0.0_dp,me%Gemb%Nt1 * dt)

      call Green_from_spectral(bath_dos,me%eps0-5.0_dp*me%DeltaW, &
         me%eps0+5.0_dp*me%DeltaW,me%MuChem,tgrd,me%Gemb)

      call tgrd%Clean()

   !................................
   contains
   !................................
   pure real(dp) function bath_dos(w)
      real(dp),intent(in) :: w
      real(dp) :: x

      x = (w - me%eps0) / me%DeltaW
      bath_dos = exp(-0.5_dp * x**2) / sqrt(2.0_dp*PI_) / me%DeltaW

   end function bath_dos
   !................................   
   end subroutine gauss_CalcGreen
!--------------------------------------------------------------------------------------
   subroutine gauss_CalcGreen_slice(me,dt)
      class(gauss_bath_t) :: me
      real(dp),intent(in) :: dt
      type(grid1d_t) :: tgrd

      call tgrd%Init(me%Gemb_slice%Nt,0.0_dp,me%Gemb_slice%Nt * dt)

      call Green_from_spectral(bath_dos,me%eps0-5.0_dp*me%DeltaW, &
         me%eps0+5.0_dp*me%DeltaW,me%MuChem,tgrd,me%Gemb_slice)

      call tgrd%Clean()

   !................................
   contains
   !................................
   pure real(dp) function bath_dos(w)
      real(dp),intent(in) :: w
      real(dp) :: x

      x = (w - me%eps0) / me%DeltaW
      bath_dos = exp(-0.5_dp * x**2) / sqrt(2.0_dp*PI_) / me%DeltaW

   end function bath_dos
   !................................   
   end subroutine gauss_CalcGreen_slice
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
   subroutine gauss_CalcGreen_diff(me,dt)
      class(gauss_bath_t) :: me
      real(dp),intent(in)      :: dt
      type(grid1d_t) :: tgrd

      call tgrd%Init(me%Nt,0.0_dp,me%Nt * dt)

      call Green_from_spectral(bath_dos,me%eps0-5.0_dp*me%DeltaW, &
         me%eps0+5.0_dp*me%DeltaW,me%MuChem,me%Beta,me%FermBos,tgrd,me%Gemb_gtr,me%Gemb_lsr)

      call tgrd%Clean()

   !................................
   contains
   !................................
   pure real(dp) function bath_dos(w)
      real(dp),intent(in) :: w
      real(dp) :: x

      x = (w - me%eps0) / me%DeltaW
      bath_dos = exp(-0.5_dp * x**2) / sqrt(2.0_dp*PI_) / me%DeltaW

   end function bath_dos
   !................................   
   end subroutine gauss_CalcGreen_diff
!--------------------------------------------------------------------------------------

!======================================================================================
end module Membedding
