#!/opt/psi/Programming/Python/3.9.10/bin/python3 
#===================================================#
#SBATCH --job-name=snap
#SBATCH --cluster=merlin6
#SBATCH --partition=hourly
#SBATCH -V 
#SBATCH --ntasks=16
#SBATCH --cpus-per-task=1
#SBATCH --mem=1G 
#SBATCH --array=1-3
#SBATCH --time=00:30:00
#===================================================# 
import os
import sys
import numpy as np
sys.path.append("../utils/")
import pulse
import Hamiltonian as ham
sys.path.append("./")
from bite_kp_snapshots_prep import GenField
sys.path.append("../code/python_drivers/")
from driver_free import Ham_TDARPES
#-------------------------------------------------------------
def run_calc(mpicmd,tag,file_ham,file_pmom,file_field,T2,Nt,dt,\
             pump,probe,screen,lape=False,file_matel="",scr_dir="./",debug_mode=True):
    Ry = 27.21138602
    tfs = 0.02418884254 # conversion factor fs -> a.u.
    #f_fac = 1.0e-6 / 5.142
    f_fac = 1.945e-7 # conversion factor kV/cm -> a.u.
    nst = 2

    Evac = 5.6 / Ry # vacuum enery wrh to Dirac point
    wphot = 6.3 / Ry # photon energy 
    kperp = np.sqrt(2.0*(wphot - Evac))

    Beta = 1000.0 # ~ 80 K
    Mu = 0.175592 / Ry # chemical potential
    
    screen_surf = screen['surf']
    screen_pe = screen['pe']
    
    calc = Ham_TDARPES("../code/",PathInp=scr_dir+"/inp/",PathOut=scr_dir+"/out/",PathLog=scr_dir+"/log/")
    
    calc.SetSystem(Beta,Mu,screen_surf,screen_pe,file_ham,file_pmom,file_field,\
                   T2=T2/tfs,file_matel=file_matel)

    tmax = Nt * dt
    calc.SetTimestepping(tmax,Nt,evol=2,store_obs=False,store_spect=True)

    wmin, wmax = Mu - 0.35/Ry, Mu + 0.35/Ry
    lape = probe['lape']
    Nepe = probe['Nepe']
    fwhm = probe['fwhm']
    pulse_shape = probe['pulse_shape']
    probe_dur = pulse.GetProbeDur(fwhm,pulse_shape=pulse_shape) / tfs   
    probe_t0s = probe['delays']
    calc.SetProbe(Nepe,wmin,wmax,probe_dur,probe_t0s,\
                  lape=lape,kperp=kperp,pulse_shape=pulse_shape,diag_matel=True)

    F0, w0, Tdur = pump['F0'], pump['w0'], pump['Tdur']

    prefix = "{}_T2{}_F{}_w{}_dur{}_s{}_{}_nepe{}_fwhm{}_pr{}".\
             format(tag,T2,F0,w0,Tdur,screen_surf[0],screen_surf[1],Nepe,fwhm,pulse_shape)
    if lape:
        prefix += "_lape_s{}_{}".format(screen_pe[0],screen_pe[1])

    calc.Run(mpicmd,prefix,debug_mode=debug_mode)
#-------------------------------------------------------------
def main(mpicmd, taskid, nthreads):
    os.environ['OMP_NUM_THREADS'] = str(nthreads)

    # scratch directory for the many input and output files
    scratch = "./"
    
    Ry = 27.21138602 # conversion factor eV -> a.u.
    tfs = 0.02418884254 # conversion factor fs -> a.u.

    Nt = 16000 # number of time steps
    dt = 0.8 # time step 

    #kmax = 0.12
    #nk = 32
    kmax = 0.08
    nk = 64
    ks = np.linspace(-kmax, kmax, nk)
    
    Nepe = 127 # number of energy points
    Beta = 1000.0 # invere temperature ~ 80 K 

    wpump = 0.1061995 # pump energy
    Tdur = np.round(107 / 2.355, decimals=5) # pump pulse FWHM = 107 fs
    phi = 0.5 # x pi (phase shift)

    F0 = 0.5
    pump = {'F0': F0, 'w0': wpump, 'Tdur': Tdur, 'phi': phi, 's0': 4.0}
    lape = False

    # Determines the shape of probe pulse. Option are
    # 0: sin^2 envelop
    # 1: gaussian envelop
    # 2: Sech envelop
    # 3: Lorentzian envelop
    pulse_shape = 2

    #fwhm_vals = [28.0, 32.0, 36.0, 40.0, 44.0, 48.0]
    fwhm_vals = [28.0]

    probe = {
        'lape': False,
        'Nepe': 127,
        'fwhm': fwhm_vals[0],
        'pulse_shape': pulse_shape
    }    

    # mel_pars = {
    #     'use_mel': True,
    #     'b0': 1.0,
    #     'dk': 0.06
    # }
    mel_pars = {'use_mel': False}

    # amplitudes of pump pulse (kV/cm)
    F0_vals = [700, 800, 900]

    T2 = 10.0 #  dephasing time = 10 fs
    
    # LAPE on/off
    screen1 = {'surf': [0.0, 0.0], 'pe': [1.0, 1.0]}
    screen2 = {'surf': [1.0, 1.0], 'pe': [0.0, 0.0]}
    screen3 = {'surf': [1.0, 1.0], 'pe': [1.0, 1.0]}
    #scr_vals = [screen2, screen3]
    scr_vals = [screen2]

    kdir = "x"
    kps = np.zeros([len(ks),2])
    kps[:,0] = ks
    
    vm = 'kp' # use the k.p Hamiltonian to incorporate
    # light-matter coupling

    #jobs = ['buildup', 'extended', 'snapexp']
    jobs = ['trace']
    delay_vals = [
        np.array([-100, -40, -20]),
        np.array([-80, -40, -20, 0, 20, 80]),
        np.array([-60, -50, -40, -20])
    ]
    delay_vals = [
        np.array([-100, -60, -50, -40, -30, -20, -10, 0, 100])
    ]

    path = "./data/ham/"

    run_count = 0
    for ijob, job_tag in enumerate(jobs):
        delays = delay_vals[ijob]
        for fwhm in fwhm_vals:
            for F0 in F0_vals:
                for screen in scr_vals:
                    run_count += 1
                    if run_count == taskid:
                        pump['F0'] = F0
                        file_field = GenField(pump)

                        file_ham = path + "bite_ham_kp_nk{}.h5".format(nk)
                        file_pmom = path + "bite_pmom_kp_nk{}.h5".format(nk)

                        if mel_pars['use_mel']:
                            file_matel = path + "bite_kp_matel_b{}_dk{}_nk{}.h5".\
                                format(mel_pars['a0'],mel_pars['b0'],mel_pars['dk'],nk)
                            tag = "bite_kp_{}_mel_dk{}".format(job_tag,mel_pars['dk'])
                        else:
                            file_matel = ""
                            tag = "bite_kp_{}".format(job_tag)

                        probe['fwhm'] = fwhm
                        probe_dur = pulse.GetProbeDur(fwhm,pulse_shape=pulse_shape) / tfs
                 
                        tp = Tdur/tfs
                        t0 = pump['s0'] * tp
                        probe_t0s = t0 + delays / tfs - 0.5*probe_dur
                        probe['delays'] = probe_t0s             

                        run_calc(mpicmd,tag,file_ham,file_pmom,file_field,T2,Nt,dt,\
                                 pump,probe,screen,file_matel=file_matel,scr_dir=scratch,debug_mode=False)                        
#------------------------------------------------------------- 
if __name__ == "__main__":
    taskid = int(os.environ['SLURM_ARRAY_TASK_ID'])
    nthreads = int(os.environ['SLURM_CPUS_PER_TASK'])

    mpicmd = 'srun'
    #mpicmd = 'mpirun -np 2'
    main(mpicmd, taskid, nthreads)
    
