import os
import sys
import numpy as np
import pulse
import Hamiltonian as ham
import h5py
#--------------------------------------------------------------------
def GenHamiltonian_kp(ks,path="./data/ham/"):
    nk = len(ks)
    H0k = np.zeros([2,2,nk], dtype=np.complex_)
    v0k = np.zeros([2,2,3,nk], dtype=np.complex_)
    kp = np.zeros([2,nk])

    for ik, kx in enumerate(ks):
        kpt = [kx, 0.0]
        H, v2 = ham.GetHam_kp(kpt)
        H0k[:,:,ik] = H
        v0k[:,:,0:2,ik] = v2 

    kp[0,:] = ks

    file_ham = path + "bite_ham_kp_nk{}.h5".format(nk)
    file_pmom = path + "bite_pmom_kp_nk{}.h5".format(nk)

    if os.path.isfile(file_ham):
        return file_ham, file_pmom

    f = h5py.File(file_ham, "w")
    f.attrs['nst'] = 2
    f.attrs['nk'] = nk
    f.create_dataset('kp', data=kp.T)
    f.create_dataset('real', data=np.real(H0k.T))
    f.create_dataset('imag', data=np.imag(H0k.T))
    f.close()

    f = h5py.File(file_pmom, "w")
    f.attrs['nst'] = 2
    f.attrs['nk'] = nk
    f.create_dataset('real', data=np.real(v0k.T))
    f.create_dataset('imag', data=np.imag(v0k.T))
    f.close()

    return file_ham, file_pmom
#-------------------------------------------------------------
def GenMatrixElements(kpts,pars,path="./data/ham/"):
    nk = kpts.shape[0]
    mels = np.zeros([2,nk], dtype=np.complex_)

    for ik, kpt in enumerate(kpts):
        mels[:,ik] = ham.GetPESMatel_kp(kpt,pars)

    file_mel = path + "bite_ham_kp_matel_b{}_dk{}_nk{}.h5".\
        format(pars['b0'],pars['dk'],nk)

    f = h5py.File(file_mel, "w")
    f.attrs['nst'] = 2
    f.attrs['nk'] = nk
    f.create_dataset('real', data=np.real(mels.T))
    f.create_dataset('imag', data=np.imag(mels.T))
    f.close()

    return file_mel
#--------------------------------------------------------------------
def GenField(pump):
    Ry = 27.21138602
    tfs = 0.02418884254
    f_fac = 1.945e-7
    F0, w0, Tdur, phi, s0 = pump['F0'], pump['w0'], pump['Tdur'], pump['phi'], pump['s0']
    t0 = s0 * Tdur / tfs
    pol = [1.0, 0.0, 0.0]

    file_field = 'data/field/pulse_F{}_w{}_Tdur{}_phi{}_s{}_x.dat'.format(F0,w0,Tdur,phi,s0)
    pulse.WritePump(file_field,F0 * f_fac,w0/Ry,t0,Tdur/tfs,pol,phi=phi*np.pi,nt=1000,rewrite=False)

    return file_field
#-------------------------------------------------------------
def main():

    Ry = 27.21138602 # conversion factor eV -> a.u.
    tfs = 0.02418884254 # conversion factor fs -> a.u.

    wpump = 0.1061995
    Tdur = np.round(107 / 2.355, decimals=5)
    phi = 0.5
    s0 = 4.0
    pump = {'F0': 0.0, 'w0': wpump, 'Tdur': Tdur, 'phi': phi, 's0': s0}

    F0_vals = [800]

    for F0 in F0_vals:
        pump['F0'] = F0
        GenField(pump)

    kmax = 0.12 # k-path along kx from -kmax to kmax (a.u.)
    #nk = 32
    kmax = 0.08
    nk = 64
    ks = np.linspace(-kmax, kmax, nk)

    file_ham, file_pmom = GenHamiltonian_kp(ks,path="./data/ham/")

    # matrix elemetns
    mel_pars = {
        'b0': 0.3,
        'dk': 0.03
    }
    
    dkm_vals = [0.04, 0.06]
    mel_b0_vals = [0.3]    

    for dkm in dkm_vals:
        for b0 in mel_b0_vals:
            kps = np.zeros([len(ks),2])
            kps[:,0] = ks

            #mel_pars['a1'] = a1
            mel_pars['b0'] = b0
            mel_pars['dk0'] = dkm
            mel_pars['dk1'] = dkm
        
            file_mel = GenMatrixElements(kps,mel_pars,path="./data/ham/")

#-------------------------------------------------------------            
if __name__ == "__main__":
    main()
    
