import sys
import os
import numpy as np
import h5py
sys.path.append("../utils/")
from constants import Ry,tfs
from pulse import GetProbeDur
from surface_kp import GetBands_kp
from plotting_tools import PlotSpectAll
#--------------------------------------------------------------
def ReadSpectrum(tag,Us,Ub,vsb,g0,g0_facs,nk,F0,w0,Tdur,fwhm_probe,scr,Nt,tmax,\
                 Nepe,lape=False,T1=None,T2=None,pulse_shape=1):

    pref = "out/" + tag
    
    scr_surf = scr['surf']
    scr_pe = scr['pe']

    app = ""
    if not T2 == None:
        app += "_T{}".format(T2)
    app += "_Us{}_Ub{}_vsb{}_g{}_f{}_{}_nk{}".format(Us,Ub,vsb,g0,g0_facs[0],g0_facs[1],nk)

    app += "_F{}_w{}_dur{}_nepe{}_fwhm{}"\
        .format(F0,w0,Tdur,Nepe,fwhm_probe)
    if pulse_shape != None:
        app += "_pr{}".format(pulse_shape)
    if lape:
        app += "_lape_s{}_{}".format(scr_pe[0],scr_pe[1])

    app += "_Nt{}_tmax{}_spectrum.h5".format(Nt,tmax)
        
    fname = pref + app
        
    f = h5py.File(fname, "r")
    nw = f.attrs['nomega']
    wmin, wmax = f.attrs['omega_min'], f.attrs['omega_max']
    probe_ts = np.array(f['probe_ts'])
    spectk = np.array(f['spectrum'])
    f.close()

    # reorder indices
    spectk = np.einsum('nkw->kwn', spectk)
    
    ws = np.linspace(wmin, wmax, nw)

    return ws, probe_ts, spectk
#--------------------------------------------------------------
def GenPlots_snapshots(param,plot_param,job_tag="snapexp"):
    Nt = param['Nt']
    tmax = param['tmax']

    lape = param['lape']
    Nepe = param['Nepe']

    Us = param['Us']
    Ub = param['Ub']
    vsb = param['vsb']
    g0 = param['g0']
    g0_facs = param['g0_facs']

    Eshift = Ry * param['mu']
    nk = param['nk']
    kmax = param['kmax']
    ks = np.linspace(-kmax, kmax, nk)
    epsk = GetBands_kp(ks)

    w0 = param['w0']
    Tdur = np.round(param['fwhm_pump'] / 2.355, decimals=5)

    fwhm_probe = param['fwhm_probe']
    pulse_shape = param['pulse_shape']
    probe_dur = GetProbeDur(fwhm_probe,pulse_shape=pulse_shape)

    F0 = param['F0']

    Emin, Emax = plot_param['Emin'], plot_param['Emax']
    ky_max = plot_param['ky_max']

    T2 = param['T2']
    scr = {'surf': [1.0, 1.0], 'pe': [0.0, 0.0]}
        
    path = "./out/"
    
    tag = "3bnd_gkba_" + job_tag

    if param['matel']:
        mel_b0 = param['mel_b0']
        mel_kc = param['mel_kc']
        tag += "_b0{}_kc{}".format(mel_b0,mel_kc)
        tag_out += "_b0{}_kc{}".format(mel_b0,mel_kc)

    ws, probe_t0, spectk = ReadSpectrum(tag,Us,Ub,vsb,g0,g0_facs,nk,F0,w0,Tdur,fwhm_probe,scr,Nt,tmax,\
                                        Nepe,lape=lape,T2=T2,pulse_shape=pulse_shape)

    t0 = param['s0_pump'] * Tdur
    delays = np.round(tfs * probe_t0 + 0.5*probe_dur - t0, decimals=4)
    
    PlotSpectAll(ws,ks,delays,spectk,ky_max,Emin,Emax,epsk=epsk,\
	         w0=w0,fac=1.0,Eshift=Eshift,fout="")
#--------------------------------------------------------------
def main():
    param = {
        'lape': False,
        'Nt': 18000,
        'tmax': 14400.0,
        'Us': 0.0,
        'Ub': 0.05,
        'vsb': 0.04,
        'g0': 0.0005,
        'g0_facs': [2.0, 1.0],
        'nk': 32,
        'kmax': 0.08,
        'T2': 10.0,
        'Nepe': 127,
        'mu': 0.175592 / Ry,
        'w0': 0.1061995,
        'fwhm_pump': 107,
        's0_pump': 4.5,
        'fwhm_probe': 28.0,
        'pulse_shape': 2,
        'F0': 800,
        'mel_dk': 0.06
    }

    plot_param = {
        'Emin': -0.3,
        'Emax': 0.3,
        'ky_max': 0.15
    }

    job_tag = "snapexp"
    param['matel'] = False
    GenPlots_snapshots(param,plot_param,job_tag=job_tag)
#--------------------------------------------------------------
if __name__ == "__main__":
    main()
