#!/opt/psi/Programming/Python/3.9.10/bin/python3
#===================================================#
#SBATCH --job-name=3bnd_gkba
#SBATCH --cluster=merlin6
#SBATCH --partition=hourly
#SBATCH -V
#SBATCH --ntasks=16
#SBATCH --cpus-per-task=1
#SBATCH --mem-per-cpu=2G 
#SBATCH --array=1-1
#SBATCH --time=01:00:00
#===================================================# 
import os
import sys
import numpy as np
import h5py
sys.path.append('../utils/')
import pulse
sys.path.append('./')
from threeband_gkba_2bloc_prep import GenField
sys.path.append('../code/python_drivers/')
from driver_2bloc_bath import Hamk_2Bloc_Bath
#--------------------------------------------------------------------
def run_calc(mpicmd,tag,file_ham,file_pmom,file_field,file_bath,file_tmat,Tmax,Nt,tswitch,\
             pump,probe,file_matel="",scr_dir="./",debug_mode=True):
    Ry = 27.21138602
    tfs = 0.02418884254 # conversion factor fs -> a.u.                                                                           
    #f_fac = 1.0e-6 / 5.142                                                                                                      
    f_fac = 1.945e-7 # conversion factor kV/cm -> a.u.                                                                           
    nst = 3

    Evac = 5.6 / Ry # vacuum enery wrh to Dirac point                                                                            
    wphot = 6.3 / Ry # photon energy                                                                                             
    kperp = np.sqrt(2.0*(wphot - Evac))

    calc = Hamk_2Bloc_Bath('../code/',PathInp=scr_dir+'/inp/',PathOut=scr_dir+'/out/')

    Beta = 1000.0 # ~ 80 K                                                                                                       
    Mu = 0.175592 / Ry # chemical potential     

    calc.SetSystem(Beta,Mu,file_ham,file_pmom,file_field=file_field,\
                   file_bath=file_bath,file_matel=file_matel,file_tmat=file_tmat)

    calc.SetTimestepping(Tmax,Nt,0.0,tswitch,0.6*tswitch,\
                         dephasing_on=True)

    calc.SetRunParams(AdiabaticSwitch=True,CalcSpectrum=True)

    wmin, wmax = Mu - 0.35/Ry, Mu + 0.35/Ry
    lape = probe['lape']
    Nepe = probe['Nepe']
    fwhm = probe['fwhm']
    pulse_shape = probe['pulse_shape']
    probe_dur = pulse.GetProbeDur(fwhm,pulse_shape=pulse_shape) / tfs
    probe_t0s = probe['delays']
    calc.SetProbe(Nepe,wmin,wmax,probe_dur,probe_t0s,lape=lape,kperp=kperp,\
                  pulse_shape=pulse_shape,diag_matel=True)

    F0, w0, Tdur = pump['F0'], pump['w0'], pump['Tdur']

    prefix = "{}_F{}_w{}_dur{}_nepe{}_fwhm{}_pr{}".format(tag,F0,w0,Tdur,Nepe,fwhm,pulse_shape)
    
    calc.Run(mpicmd,prefix,debug_mode=debug_mode)
#--------------------------------------------------------------------
def main(mpicmd,taskid):
    Ry = 27.21138602 # conversion factor eV -> a.u.
    tfs = 0.02418884254 # conversion factor fs -> a.u.    

    Nt = 18000 # number of time steps                                                                                            
    dt = 0.8 # time step                                                                                                         
    tmax = Nt * dt # propagation time 
    tswitch = 1000.0 # adiabatic switch-on time

    Us = 0.0
    Ub_vals = [0.05]
    vsb_vals = [0.04, 0.06, 0.08, 0.1]
    
    # nk = 128
    nk = 64

    # bath parameters
    bath_dos = "semi"
    nbaths = 2
    g0_vals = np.round(1.0e-3 * np.array([0.5]), decimals=6)
    gfacs = np.array([2.0, 1.0])

    # dephasing
    T2_ss = 10.0
    T2_sb_vals = [10.0]
    
    path = "./data/ham/"
    scratch = "./"

    wpump = 0.1061995
    Tdur = np.round(107 / 2.355, decimals=5)
    phi = 0.5
    s0 = 4.5
    asym = {'fac': 0.0}
    pump = {'F0': 0.0, 'w0': wpump, 'Tdur': Tdur, 'phi': phi, 's0': s0, 'asym': asym}
    F0_vals = [800]

    pulse_shape = 2
    fwhm_vals = [28.0]

    probe = {
    	'lape': False,
        'Nepe': 127,
        'fwhm': fwhm_vals[0],
        'pulse_shape': pulse_shape
    }

    mel_pars = {
        'use_mel': False
    }

    # mel_pars = {
    #     'use_mel': True
    #     'a0': 0.0,
    #     'b0': 0.3,
    #     'dk': 0.06
    # }

    jobs = ['snapexp', 'extended']
    delays_list = [
        np.array([-12, 22, 46]),
        np.array([-80, -40, -20, 0, 20, 80])
    ]

    run_count = 0
    for ijob, job_tag in enumerate(jobs):
        delays = delays_list[ijob]
        for F0 in F0_vals:
            for g0 in g0_vals:
                for Ub in Ub_vals:
                    for vsb in vsb_vals:
                        for T2_sb in T2_sb_vals:
                            for fwhm in fwhm_vals:
                                run_count += 1
                                if run_count == taskid:
                                    pump['F0'] = F0
                                    file_field = GenField(pump)

                                    file_ham = path + "bite_ham_3bnd_Us{}_Ub{}_nk{}.h5".format(Us,Ub,nk)
                                    file_pmom = path + "bite_pmom_3bnd_vb{}_nk{}.h5".format(vsb,nk)
                                    bath_tag = "{}_g{}_f{}_{}".format(bath_dos,g0,gfacs[0],gfacs[1])
                                    file_bath = path + "bite_bath_{}_nb{}.h5".format(bath_tag,nbaths)
                                    file_tmat = path + "bite_tmat_3bnd_T2{}_{}.h5".format(T2_ss,T2_sb)

                                    if mel_pars['use_mel']:
                                        file_matel = path + "bite_3bnd_matel_a{}_b{}_dk{}_nk{}.h5".\
                                            format(mel_pars['a0'],mel_pars['b0'],mel_pars['dk'],nk)
                                        tag = "3bnd_gkba_mel_dk{}_{}_T{}_Us{}_Ub{}_vsb{}_g{}_f{}_{}_nk{}".\
                                            format(mel_pars['dk'],job_tag,T2_sb,Us,Ub,vsb,g0,gfacs[0],gfacs[1],nk)
                                    else:
                                        file_matel = ""
                                        tag = "3bnd_gkba_{}_T{}_Us{}_Ub{}_vsb{}_g{}_f{}_{}_nk{}".\
                                            format(job_tag,T2_sb,Us,Ub,vsb,g0,gfacs[0],gfacs[1],nk)

                                    probe['fwhm'] = fwhm
                                    probe_dur = pulse.GetProbeDur(fwhm,pulse_shape=pulse_shape) / tfs
                             
                                    tp = Tdur/tfs
                                    t0 = pump['s0'] * tp
                                    probe_t0s = t0 + delays / tfs - 0.5*probe_dur
                                    probe['delays'] = probe_t0s
                                
                                    run_calc(mpicmd,tag,file_ham,file_pmom,file_field,file_bath,file_tmat,tmax,Nt,tswitch,\
                                             pump,probe,file_matel=file_matel,scr_dir=scratch,debug_mode=False)
#--------------------------------------------------------------------
if __name__ == '__main__':
    nthreads = int(os.environ['SLURM_CPUS_PER_TASK'])
    taskid = int(os.environ['SLURM_ARRAY_TASK_ID'])
    os.environ['OMP_NUM_THREADS'] = str(nthreads)
    
    mpicmd = 'srun'
    #mpicmd = 'mpirun -np 2'
    main(mpicmd,taskid)





