import os
import sys
import numpy as np
import h5py
import Hamiltonian as ham
import pulse
import surface_3bnd as surf
home = os.environ['HOME']
myroot = home + '/Programs/corrdyn2d/utils/'
sys.path.append(myroot)
from hamk_2bloc_bath import Hamk_2Bloc_Bath
#--------------------------------------------------------------------
def GenHamiltonian_3bnd(ks,vsb,Us=0.0,Ub=0.0,path="./data/ham/"):
    Ry = 27.21138602
    nbnd = 3
    nk = len(ks)
    H0k = np.zeros([nbnd,nbnd,nk], dtype=np.complex_)
    v0k = np.zeros([nbnd,nbnd,3,nk], dtype=np.complex_)
    kp = np.zeros([2,nk])

    bite = surf.BiSe_Surf_3bnd(vsb=vsb)

    for ik, kx in enumerate(ks):
        kpt = [kx, 0.0]
        H2 = bite.get_ham(kpt)
        H0k[:,:,ik] = H2[:,:]
        v2 = bite.get_velocity(kpt)
        v0k[0:3,0:3,0:2,ik] = v2[0:3,0:3,0:2]

    kp[0,:] = ks

    Uhubb = np.array([Us, Us, Ub]) / Ry

    file_ham = path + "bite_ham_3bnd_Us{}_Ub{}_nk{}.h5".format(Us,Ub,nk)
    file_pmom = path + "bite_pmom_3bnd_vb{}_nk{}.h5".format(vsb,nk)

    f = h5py.File(file_ham, "w")
    f.attrs['nst'] = nbnd
    f.attrs['nk'] = nk
    f.create_dataset('uhubb', data=Uhubb)
    f.create_dataset('kp', data=kp.T)
    f.create_dataset('real', data=np.real(H0k.T))
    f.create_dataset('imag', data=np.imag(H0k.T))
    f.close()

    f = h5py.File(file_pmom, "w")
    f.attrs['nst'] = nbnd
    f.attrs['nk'] = nk
    f.create_dataset('real', data=np.real(v0k.T))
    f.create_dataset('imag', data=np.imag(v0k.T))
    f.close()

    return file_ham, file_pmom
#--------------------------------------------------------------------
def GenBath_3bnd(tag,mus,eps0s,dws,coup,dos=0,path="./data/ham/"):
    nbaths = len(mus)

    file_bath = path + "bite_bath_{}_nb{}.h5".format(tag,nbaths)
    f = h5py.File(file_bath, "w")

    f.attrs['nst'] = 3
    f.attrs['nbaths'] = nbaths
    f.attrs['dos'] = dos
    f.create_dataset('mu', data=mus)
    f.create_dataset('eps0', data=eps0s)
    f.create_dataset('deltaw', data=dws)
    f.create_dataset('cp', data=coup.T)
    f.close()

    return file_bath
#--------------------------------------------------------------------
def GenField(pump):
    Ry = 27.21138602
    tfs = 0.02418884254
    f_fac = 1.945e-7
    F0, w0, Tdur, phi, s0 = pump['F0'], pump['w0'], pump['Tdur'], pump['phi'], pump['s0']
    t0 = s0 * Tdur / tfs
    pol = [1.0, 0.0, 0.0]

    file_field = 'data/field/pulse_F{}_w{}_Tdur{}_phi{}_s{}_x.dat'.format(F0,w0,Tdur,phi,s0)
    pulse.WritePump(file_field,F0 * f_fac,w0/Ry,t0,Tdur/tfs,pol,phi=phi*np.pi,nt=1000,rewrite=False)

    return file_field
#--------------------------------------------------------------------
def PrepMatel_3bnd(ks,mel_pars,path="./data/ham/"):
    if mel_pars['use_mel']:
        ham_pref = path + "bite"
        kpts = np.zeros([len(ks),2])
        kpts[:,0] = ks
        file_matel = ham.GenPESMatel_3bnd(ham_pref,kpts,mel_pars,band_basis=True)
    else:
        file_matel = ""

    return file_matel
#--------------------------------------------------------------------
def PrepTmat_3bnd(T2_ss,T2_sb,path="./data/ham/"):
    tfs = 0.02418884254
    nbnd = 3
    Tmat = 1.0e8 * np.ones([nbnd,nbnd])

    Tmat[0,1] = T2_ss / tfs
    Tmat[0,2] = T2_sb / tfs
    Tmat[1,2] = T2_sb / tfs
    Tmat[1,0] = T2_ss / tfs
    Tmat[2,0] = T2_sb / tfs
    Tmat[2,1] = T2_sb / tfs
    
    file_tmat = path + "bite_tmat_3bnd_T2{}_{}.h5".format(T2_ss,T2_sb)
    
    f = h5py.File(file_tmat, "w")
    f.attrs['nst'] = nbnd
    f.create_dataset('tmat', data=Tmat)
    f.close()

    return file_tmat
#--------------------------------------------------------------------
def main():
    Ry = 27.21138602

    wpump = 0.1061995
    Tdur = np.round(107 / 2.355, decimals=5)
    phi = 0.5
    s0 = 4.5
    asym = {'fac': 0.0}
    pump = {'F0': 0.0, 'w0': wpump, 'Tdur': Tdur, 'phi': phi, 's0': s0, 'asym': asym}

    F0_vals = [800]

    for F0 in F0_vals:
        pump['F0'] = F0
        GenField(pump)

    # kmax = 0.12
    # nk = 128
    kmax = 0.08
    nk = 64
    ks = np.linspace(-kmax, kmax, nk)

    Us_vals = [0.0]
    Ub_vals = [0.0, 0.05, 0.1, 0.15, 0.2]
    vsb_vals = [0.02, 0.04, 0.06, 0.08, 0.1]

    for Us in Us_vals:
        for Ub in Ub_vals:
            for vsb in vsb_vals:
                file_ham, file_pmom = GenHamiltonian_3bnd(ks,vsb,Us=Us,Ub=Ub)

    mu = 0.175592 / Ry


    #================ bath ================
    dos = 1 # semi-circular DOS
    dos_tag = "semi"
    eps0s = np.array([0.38, 0.42]) / Ry
    dws = np.array([0.1, 0.1]) / Ry
    g0_vals = np.round(1.0e-3 * np.array([0.2, 0.5, 1.0, 1.5]), decimals=6)
    gfacs = np.array([2.0, 1.0])
    nb = 2
    mus = np.array([mu, mu])
    for g0 in g0_vals:
        coup = np.zeros([3,nb])
        coup[0,0] = g0 * gfacs[0]
        coup[1,0] = g0 * gfacs[0]
        coup[2,1] = g0 * gfacs[1]

        bath_tag = "{}_g{}_f{}_{}".format(dos_tag,g0,gfacs[0],gfacs[1])
        file_bath = GenBath_3bnd(bath_tag,mus,eps0s,dws,coup,dos=dos)


    # matrix elemetns
    mel_pars = {
        'use_mel': False,
        'a0': 0.0,
        'b0': 0.3,
        'dk': 0.06
    }

    file_matel = PrepMatel_3bnd(ks,mel_pars)

    T2_ss = 10.0
    T2_sb_vals = [10.0, 40.0, 200.0]

    for T2_sb in T2_sb_vals:
        file_tmat = PrepTmat_3bnd(T2_ss,T2_sb)    
#--------------------------------------------------------------------
if __name__ == '__main__':
    main()


