import os
import numpy as np
from scipy.interpolate import interp1d
from scipy.integrate import simps
import matplotlib.pyplot as plt
import h5py
#-------------------------------------------------------------
def get_mel(kpar,Nepe,Emin,Emax):
    fname = "data/surf_integral.h5"
    f = h5py.File(fname, "r")

    ks = np.array(f['k'])
    fre = np.array(f['real'])
    fim = np.array(f['imag'])
    nst = fre.shape[1]
    
    f.close()

    func_re = interp1d(ks, fre, axis=0)
    func_im = interp1d(ks, fim, axis=0)

    epe = np.linspace(Emin, Emax, Nepe)

    mel = np.zeros([nst, Nepe, 3], dtype=np.complex_)

    Ekz = epe - 0.5 * (kpar[0]**2 + kpar[1]**2)
    Iz, = np.where(Ekz > 0.0)
    
    for i in Iz:
        kperp = np.sqrt(2.0 * epe[i] - kpar[0]**2 - kpar[1]**2)
        x = func_re(kperp) + 1j * func_im(kperp)
        mel[:,i,0] = kpar[0] * x
        mel[:,i,1] = kpar[1] * x
        mel[:,i,2] = kperp * x

    return epe, mel
#-------------------------------------------------------------
def store_mel(fname, epe, mel, rewrite=True):
    if (not os.path.isfile(fname)) or rewrite:
        f = h5py.File(fname, "w")
        
        f.attrs['nst'] = mel.shape[0]
        f.attrs['nepe'] = len(epe)
        f.attrs['epe_min'] = epe[0]
        f.attrs['epe_max'] = epe[-1]
        
        f.create_dataset('real', data=np.real(mel).T)
        f.create_dataset('imag', data=np.imag(mel).T)
    
        f.close()
#-------------------------------------------------------------
def store_tmat(fname, Tmat, rewrite=True):
    if (not os.path.isfile(fname)) or rewrite:
        f = h5py.File(fname, "w")
        f.attrs['nst'] = Tmat.shape[0]
        f.create_dataset('tmat', data=Tmat.T)
        f.close()
#-------------------------------------------------------------
def store_ham(fname, h, rewrite=True):
    if (not os.path.isfile(fname)) or rewrite:
        f = h5py.File(fname, "w")
        f.attrs['nst'] = h.shape[0]
        f.create_dataset('real', data=np.real(h).T)
        f.create_dataset('imag', data=np.imag(h).T)
        f.close()
#-------------------------------------------------------------
def store_pmom(fname, pmom, rewrite=True):
    if (not os.path.isfile(fname)) or rewrite:
        f = h5py.File(fname, "w")
        f.attrs['nst'] = pmom.shape[0]
        f.create_dataset('real', data=np.real(pmom).T)
        f.create_dataset('imag', data=np.imag(pmom).T)
        f.close()    
#-------------------------------------------------------------
def test():
    sq2 = np.sqrt(2.0)
    pol = [1.0/sq2, 0.0, 1.0/sq2]
    kpar = [0.0, 0.0]
    Nepe = 40
    Emin, Emax = 0.0, 0.5

    epe, mel = get_mel(pol, kpar, Nepe, Emin, Emax)

    fname = "data/mel_kx{}_ky{}.h5".format(kpar[0], kpar[1])
    store_mel(fname, epe, mel)
#-------------------------------------------------------------
if __name__ == "__main__":
    test()
