import sys
import os
import numpy as np
import h5py
#--------------------------------------------------------------
def GetParam(fname):
    f = h5py.File(fname, "r")

    nw = f.attrs['nomega']
    wmin = f.attrs['omega_min']
    wmax = f.attrs['omega_max']
    probe_t0 = np.array(f['probe_ts'])
    ws = np.linspace(wmin, wmax, nw)
    
    f.close()

    return ws,probe_t0
#--------------------------------------------------------------
def GetTimes(fname):
    f = h5py.File(fname, "r")

    nt = f.attrs['nt']
    tmax = f.attrs['tmax']
    f.close()

    ts = np.linspace(0.0, tmax, nt)
    
    return ts
#--------------------------------------------------------------
def ReadSpectrum(fname):
    
    f = h5py.File(fname, "r")
    spect = np.array(f['spectrum'])
    f.close()

    return spect.T
#--------------------------------------------------------------
def ReadOcc(fname):
    
    f = h5py.File(fname, "r")
    occ = np.array(f['occ'])
    f.close()

    return occ
#-------------------------------------------------------------- 
def GetFname(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,kp,Nepe=100,\
             lape=True,T1=None,T2=None,pulse_shape=None,bt=None,gmm=None,dlt=None,\
             retarded=False,ftype="spectrum"):

    scr_surf = scr['surf']
    scr_pe = scr['pe']
    
    app = ""
    if not bt == None:
        app += "_bt{}".format(bt)
    if not T1 == None:
        app += "_T1{}".format(T1)
    if not T2 == None:
        app += "_T2{}".format(T2)
    if not gmm == None:
        app += "_gmm{}".format(gmm)
    if not dlt == None:
        app += "_dlt{}".format(dlt)

    app += "_F{}_w{}_dur{}_s{}_{}_nepe{}_fwhm{}"\
        .format(F0,w0,Tdur,scr_surf[0],scr_surf[1],Nepe,fwhm)
    if pulse_shape != None:
        app += "_pr{}".format(pulse_shape)
    if lape:
        app += "_lape_s{}_{}".format(scr_pe[0],scr_pe[1])
    app += "_Nt{}_tmax{}_{}.h5".format(Nt,tmax,ftype)

    pref = tag
    if retarded:
        pref += "_ret"

    kx = np.round(kp[0], decimals=5)
    ky = np.round(kp[1], decimals=5)
    pref += "_kx{}_ky{}" .format(kx, ky)
        
    return pref + app
#--------------------------------------------------------------
def GetFname_sb(tag,nbulk,vsb,F0,w0,Tdur,fwhm,scr,Nt,tmax,k,Nepe=100,kdir="x",\
                lape=True,T1_ss=None,T2_ss=None,T2_sb=None,T1_bb=None):

    scr_surf = scr['surf']
    scr_pe = scr['pe']

    app = ""
    if not T1_ss == None:
        app += "_T1ss{}".format(T1_ss)
    if not T2_ss == None:
        app += "_T2ss{}".format(T2_ss)
    if not T2_sb == None:
        app += "_Tsb{}".format(T2_sb)
    if not T1_bb == None:
        app += "_Tbb{}".format(T1_bb)
    app += "_F{}_w{}_dur{}_nepe{}_fwhm{}".format(F0,w0,Tdur,Nepe,fwhm)
    if lape:
        app += "_lape_s{}_{}".format(scr_pe[0],scr_pe[1])
        
    app += "_Nt{}_tmax{}_spectrum.h5".format(Nt,tmax)

    pref = tag
    pref += "_nb{}_vsb{}".format(nbulk,vsb)
    if kdir == "x":
        pref += "_kx{}_ky{}" .format(k, 0.0)
    elif kdir == "y":
        pref += "_kx{}_ky{}" .format(0.0, k)
        

    return pref + app
#--------------------------------------------------------------
def ReadPath(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,kps,Nepe=100,kdir="x",\
             lape=True,bt=None,T1=None,T2=None,pulse_shape=None,\
             gmm=None,dlt=None,retarded=False,path="./out/"):
    
    scr_surf = scr['surf']
    scr_pe = scr['pe']
    nk = len(kps)

    fname = GetFname(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,kps[0],Nepe=Nepe,\
                     lape=lape,bt=bt,T1=T1,T2=T2,pulse_shape=pulse_shape,\
                     gmm=gmm,dlt=dlt,retarded=retarded)
    
    ws, probe_t0 = GetParam(path + fname)
    
    spectk = np.zeros([nk,len(ws),len(probe_t0)])

    for ik,kp in enumerate(kps):
        fname = GetFname(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,kp,Nepe=Nepe,\
                         lape=lape,bt=bt,T1=T1,T2=T2,pulse_shape=pulse_shape,\
                         gmm=gmm,dlt=dlt,retarded=retarded)
        s = ReadSpectrum(path + fname)
        spectk[ik,:,:] = ReadSpectrum(path + fname)        

    return ws, probe_t0, spectk
#--------------------------------------------------------------
def ReadPath_sb(tag,nbulk,vsb,F0,w0,Tdur,fwhm,scr,Nt,tmax,ks,Nepe=100,kdir="x",\
                lape=True,T1_ss=None,T2_ss=None,T2_sb=None,T1_bb=None,path="./out/"):
    
    scr_surf = scr['surf']
    scr_pe = scr['pe']
    nk = len(ks)

    fname = GetFname_sb(tag,nbulk,vsb,F0,w0,Tdur,fwhm,scr,Nt,tmax,ks[0],Nepe=Nepe,kdir=kdir,\
                        lape=lape,T1_ss=T1_ss,T2_ss=T2_ss,T2_sb=T2_sb,T1_bb=T1_bb)
    
    ws, probe_t0 = GetParam(path + fname)

    spectk = np.zeros([nk,len(ws),len(probe_t0)])

    for ik,kx in enumerate(ks):
        fname = GetFname_sb(tag,nbulk,vsb,F0,w0,Tdur,fwhm,scr,Nt,tmax,kx,Nepe=Nepe,\
                            kdir=kdir,lape=lape,T1_ss=T1_ss,T2_ss=T2_ss,T2_sb=T2_sb,T1_bb=T1_bb)

        spectk[ik,:,:] = ReadSpectrum(path + fname)

    return ws, probe_t0, spectk
#--------------------------------------------------------------  
def ReadOccPath(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,ks,nbnd=2,Nepe=100,kdir="x",\
                T1=None,T2=None,gmm=None,dlt=None,path="./out/"):
    
    scr_surf = scr['surf']
    scr_pe = scr['pe']
    nk = len(ks)

    fname = GetFname(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,ks[0],Nepe=Nepe,\
                         kdir=kdir,lape=False,T1=T1,T2=T2,gmm=gmm,dlt=dlt,\
                         ftype="observables")
    
    ts = GetTimes(path + fname)

    occk = np.zeros([nk,len(ts),nbnd])

    for ik,kx in enumerate(ks):
        fname = GetFname(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,ks[0],Nepe=Nepe,\
                         kdir=kdir,lape=False,T1=T1,T2=T2,gmm=gmm,dlt=dlt,\
                         ftype="observables")

        occk[ik,:,:] = ReadOcc(path + fname)

    return ts, occk
#-------------------------------------------------------------- 
def GetSpectrumFile(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,Nepe,kdir,\
                    lape,bt=None,T1=None,T2=None,pulse_shape=None,\
                    gmm=None,dlt=None,retarded=False):

    scr_surf = scr['surf']
    scr_pe = scr['pe']

    app = ""
    if not bt == None:
        app += "_bt{}".format(bt)
    if not T1 == None:
        app += "_T1{}".format(T1)
    if not T2 == None:
        app += "_T2{}".format(T2)
    if not gmm == None:
        app += "_gmm{}".format(gmm)
    if not dlt == None:
        app += "_dlt{}".format(dlt)

    app += "_F{}_w{}_dur{}_s{}_{}_nepe{}_fwhm{}"\
        .format(F0,w0,Tdur,scr_surf[0],scr_surf[1],Nepe,fwhm)
    if pulse_shape != None:
        app += "_pr{}".format(pulse_shape)
    if lape:
        app += "_lape_s{}_{}".format(scr_pe[0],scr_pe[1])
    app += "_Nt{}_tmax{}.h5".format(Nt,tmax)

    pref = tag
    if retarded:
        pref += "_ret"
    pref += "_dir{}".format(kdir)

    fname = "data/spectrum/" + pref + app
    return fname
#--------------------------------------------------------------
def GetSpectrumFile_sb(tag,nbulk,vsb,F0,w0,Tdur,fwhm,scr,Nt,tmax,Nepe,kdir,\
                       lape,T1_ss=None,T2_ss=None,T2_sb=None,T1_bb=None):

    scr_surf = scr['surf']
    scr_pe = scr['pe']

    app = ""
    if not T1_ss == None:
        app += "_T1ss{}".format(T1_ss)
    if not T2_ss == None:
        app += "_T2ss{}".format(T2_ss)
    if not T2_sb == None:
        app += "_Tsb{}".format(T2_sb)
    if not T1_bb == None:
        app += "_Tbb{}".format(T1_bb)
    app += "_F{}_w{}_dur{}_nepe{}_fwhm{}".format(F0,w0,Tdur,Nepe,fwhm)
    if lape:
        app += "_lape_s{}_{}".format(scr_pe[0],scr_pe[1])
    app += "_Nt{}_tmax{}.h5".format(Nt,tmax)
    
    pref = tag
    pref += "_nb{}_vsb{}_{}".format(nbulk,vsb,kdir)

    fname = "data/spectrum/" + pref + app
    return fname
#--------------------------------------------------------------
def GetOccFile(tag,F0,w0,Tdur,Nt,tmax,kdir,\
               T1=None,T2=None,gmm=None,dlt=None):
    
    app = ""
    if not T1 == None:
        app += "_T1{}".format(T1)
    if not T2 == None:
        app += "_T2{}".format(T2)
    if not gmm == None:
        app += "_gmm{}".format(gmm)
    if not dlt == None:
        app += "_dlt{}".format(dlt)
    app += "_F{}_w{}_dur{}".format(F0,w0,Tdur)
    app += "_Nt{}_tmax{}.h5".format(Nt,tmax)
    
    pref = tag
    pref += "_{}".format(kdir)

    fname = "data/occ/" + pref + app
    return fname
#--------------------------------------------------------------
def GetOccFile_sb(tag,nbulk,vsb,F0,w0,Tdur,Nt,tmax,kdir,\
                  T1_ss=None,T2_ss=None,T2_sb=None,T1_bb=None):

    app = ""
    if not T1_ss == None:
        app += "_T1ss{}".format(T1_ss)
    if not T2_ss == None:
        app += "_T2ss{}".format(T2_ss)
    if not T2_sb == None:
        app += "_Tsb{}".format(T2_sb)
    if not T1_bb == None:
        app += "_Tbb{}".format(T1_bb)
    app += "_F{}_w{}_dur{}".format(F0,w0,Tdur)
    app += "_Nt{}_tmax{}.h5".format(Nt,tmax)
    
    pref = tag
    pref += "_nb{}_vsb{}_{}".format(nbulk,vsb,kdir)

    fname = "data/occ/" + pref + app
    return fname
#-------------------------------------------------------------- 
def StoreSpectrum(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,Nepe,kdir,\
                  lape,ws, probe_t0, spectk,bt=None,T1=None,T2=None,\
                  pulse_shape=None,gmm=None,dlt=None,retarded=False):

    fname = GetSpectrumFile(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,Nepe,kdir,\
                            lape,bt=bt,T1=T1,T2=T2,pulse_shape=pulse_shape,\
                            gmm=gmm,dlt=dlt,retarded=retarded)

    f = h5py.File(fname, "w")
    
    f.create_dataset("ws", data=ws)
    f.create_dataset("probe_t0", data=probe_t0)
    f.create_dataset("spectk", data=spectk)

    f.close()

    return fname
#--------------------------------------------------------------
def StoreSpectrum_sb(tag,nbulk,vsb,F0,w0,Tdur,fwhm,scr,Nt,tmax,Nepe,kdir,\
                     lape,ws, probe_t0, spectk,T1_ss=None,T2_ss=None,T2_sb=None,T1_bb=None):    
    fname = GetSpectrumFile_sb(tag,nbulk,vsb,F0,w0,Tdur,fwhm,scr,Nt,tmax,Nepe,kdir,\
                               lape,T1_ss=T1_ss,T2_ss=T2_ss,T2_sb=T2_sb,T1_bb=T1_bb)

    f = h5py.File(fname, "w")

    f.create_dataset("ws", data=ws)
    f.create_dataset("probe_t0", data=probe_t0)
    f.create_dataset("spectk", data=spectk)

    f.close()

    return fname
#--------------------------------------------------------------
def StoreOcc(tag,F0,w0,Tdur,Nt,tmax,kdir,\
             ts,occk,T1=None,T2=None,gmm=None,dlt=None,kresolv=False):
    
    fname = GetOccFile(tag,F0,w0,Tdur,Nt,tmax,kdir,\
                          T1=T1,T2=T2,gmm=gmm,dlt=dlt)

    f = h5py.File(fname, "w")

    f.create_dataset("ts", data=ts)

    if kresolv:
        f.create_dataset("occk", data=occk)
    else:
        occ = np.sum(occk, axis=0) / occk.shape[0]
        f.create_dataset("occ", data=occ)

    f.close()

    return fname
#--------------------------------------------------------------
def StoreOcc_sb(tag,nbulk,vsb,F0,w0,Tdur,Nt,tmax,kdir,\
                ts,occk,T1_ss=None,T2_ss=None,T2_sb=None,T1_bb=None,kresolv=False):
    
    fname = GetOccFile_sb(tag,nbulk,vsb,F0,w0,Tdur,Nt,tmax,kdir,\
                          T1_ss=T1_ss,T2_ss=T2_ss,T2_sb=T2_sb,T1_bb=T1_bb)

    f = h5py.File(fname, "w")

    f.create_dataset("ts", data=ts)

    if kresolv:
        f.create_dataset("occk", data=occk)
    else:
        occ = np.sum(occk, axis=0) / occk.shape[0]
        f.create_dataset("occ", data=occ)

    f.close()

    return fname
#-------------------------------------------------------------- 
def main():

    kmax = 0.125
    #nk = 50
    nk = 100
    ks = np.round(np.linspace(-kmax, kmax, nk), decimals=4)

    Nepe = 200
    Nt = 16000
    #Nt= 18000
    dt = 0.8
    tmax = Nt * dt
    pulse_shape = 1
    lape = True

    w0 = 0.10753
    Tdur = np.round(90 / 2.355, decimals=5)   

    #fwhm_vals = [10.0, 15.0, 20.0, 30.0, 40.0]
    #fwhm_vals = [10.0, 15.0, 20.0]
    #F0_vals = [250, 300, 350, 500]
    #F0_vals = [250, 350, 500, 750, 1000]
    #T2_vals = [None, 4.0, 10.0, 20.0]
    fwhm_vals = np.arange(10.0, 34.0, step=4.0)
    #fwhm_vals = [40.0, 60.0]
    F0_vals = [400, 450, 500, 550]
    T2_vals = [10.0] 
    #T2_vals = [None]

    bt = 1000.0
    retarded =  False
    #bt = None
    #retarded = True

    screen1 = {'surf': [0.0, 0.0], 'pe': [1.0, 1.0]}
    screen2 = {'surf': [1.0, 1.0], 'pe': [0.0, 0.0]}
    screen3 = {'surf': [1.0, 1.0], 'pe': [1.0, 1.0]}
    #scr_vals = [screen1, screen2, screen3]
    scr_vals = [screen2, screen3]

    tag = "bite_kp"
    #tag = "bite_dir"
    kdir = "x"

    for F0 in F0_vals:
        for fwhm in fwhm_vals:
            for scr in scr_vals:
                for T2 in T2_vals:
                    ws, probe_t0, spectk = ReadPath(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,\
                                                    ks,Nepe=Nepe,kdir="x",lape=lape,\
                                                    bt=bt,T2=T2,retarded=retarded)
    
                    fout = StoreSpectrum(tag,F0,w0,Tdur,fwhm,scr,Nt,tmax,Nepe,kdir,\
                                         lape,ws,probe_t0,spectk,bt=bt,T2=T2,retarded=retarded)
                    
                    print(fout)
#-------------------------------------------------------------- 
if __name__ == "__main__":
    main()
                  
