import os
import re
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import PIL.Image
from matplotlib.colors import ListedColormap
from scipy import interpolate
import subprocess

# load homemade python moduels
import absplot_v03 as absplot

# matplotlib backend selection
matplotlib.use('TkAgg')

# file version check
this_file_name = os.path.basename(__file__)
regular_expression = re.compile(r'(edf_setup_)(\w+)(.py)')
regular_group = regular_expression.match(this_file_name).groups()
script_name = str(regular_group[0])
version = str(regular_group[1])
# print('file version:', version)

# global figure parameters
fontsize_main = 8
fontsize_colorbar = 7
fontsize_symbol = 9

ticks_xpad = 4
ticks_ypad = 3

outdir = '.'  # output directory


matplotlib.rcParams['font.family'] = 'Arial'
mainframe = absplot.MainFrame(width=205, height=200)

#######################################################################################################################
# Plot structure 1 (setup schematics)
#######################################################################################################################
# create empty frame for this structure
eframe1 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=121, width=205, height=80)
# eframe1.visualize_frame()

# load schematics image
img_schematics = np.array(PIL.Image.open('./Setup_v06.png'))
schematics_offset = 10
height, width = img_schematics[:, :, 0].shape
schematics_width = mainframe.width - schematics_offset * 2
schematics_height = schematics_width * (height / width)
pframe11 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=schematics_offset, bottom=0, width=schematics_width, height=schematics_height)
pframe11.hide_axis()
pframe11.hide_frame()
pframe11.assign_general_image(img_schematics)
def rel_xpos_schematics(x_abs):
    return (x_abs - schematics_offset) / schematics_width
def rel_ypos_schematics(y_abs):
    return (y_abs - 121) / schematics_height
color_schematics_text = 'black'
color_schematics_text2 = (0 / 255, 144 / 255, 53 / 255)
color_schematics_text3 = (240 / 255, 124 / 255, 0 / 255)
pframe11.ax.text(rel_xpos_schematics(102.9), rel_ypos_schematics(189.3), 'Ti:Sa', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(102.9), rel_ypos_schematics(184.8), 'amplifier', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(75.2), rel_ypos_schematics(186.8), 'Dual OPA', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(126.8), rel_ypos_schematics(193), r'$\lambda$/2', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(132.9), rel_ypos_schematics(193), 'P', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(139.9), rel_ypos_schematics(193), 'L', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(147.1), rel_ypos_schematics(193), 'A1', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(163.1), rel_ypos_schematics(193), 'MPC', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(178.8), rel_ypos_schematics(193), 'A2', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(165.1), rel_ypos_schematics(175.1), 'P1', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(147.6), rel_ypos_schematics(170.5), 'P2', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(127.2), rel_ypos_schematics(160.1), 'P4', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(116.8), rel_ypos_schematics(151.7), 'P3', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(169.7), rel_ypos_schematics(127.3), r'$\lambda$/2', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(145.3), rel_ypos_schematics(139.8), 'BBO', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(105.2), rel_ypos_schematics(134.4), 'ITO', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(72.1), rel_ypos_schematics(141), 'WG', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(62), rel_ypos_schematics(138.5), 'Ge', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(53.3), rel_ypos_schematics(128.1), 'GaSe', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(49), rel_ypos_schematics(147.8), r'$t$', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(51.2), rel_ypos_schematics(167), r'$t_{\mathrm{CEP}}$', fontsize=fontsize_main,
                 transform=pframe11.ax.transAxes, horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text)
pframe11.ax.text(rel_xpos_schematics(52.3), rel_ypos_schematics(184.4), r'$\lambda_{\mathrm{C}}$ = 1.3 $\mu$m',
                 fontsize=fontsize_main, transform=pframe11.ax.transAxes,
                 horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text2)
pframe11.ax.text(rel_xpos_schematics(52.3), rel_ypos_schematics(195), r'$\lambda_{\mathrm{C}}$ = 1.2 $\mu$m',
                 fontsize=fontsize_main, transform=pframe11.ax.transAxes,
                 horizontalalignment='center', verticalalignment='center',
                 color=color_schematics_text3)

#######################################################################################################################
# End of Plot structure 1
#######################################################################################################################


#######################################################################################################################
# Plot structure 2 (EOS waveform and spectrum)
#######################################################################################################################
# create empty frame for this structure
eframe2 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=65, width=95, height=55)
# eframe2.visualize_frame()

# create single plotting frame
pframe21 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2, left=14, bottom=10, width=75, height=40)
pframe21.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
inset_width = 25
inset_height = 12
pframe21_inset = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                                   left=pframe21.left + pframe21.width - inset_width,
                                   bottom=pframe21.bottom + pframe21.height - inset_height,
                                   width=inset_width, height=inset_height)
pframe21_inset.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black', ticks_label_xpad=3, ticks_label_ypad=3)

tmp_filepath = './25THzTimeDomain.txt'
data_eos = np.loadtxt(tmp_filepath)
delay_time = data_eos[:, 0]
eos = data_eos[:, 1]
tmp_filepath = './25THzFreqDomain.txt'
data_eos = np.loadtxt(tmp_filepath)
freq = data_eos[:, 0]
eos_fft = data_eos[:, 1]

# set plot range
tmin_plot = -150
tmax_plot = 200
pframe21.ax.set_xlim(tmin_plot, tmax_plot)
scale_eos = np.max(np.abs(eos))
pframe21.ax.set_ylim(-scale_eos * 1.2, scale_eos * 1.5)
pframe21_inset.ax.set_xlim(10, 60)
pframe21_inset.ax.set_ylim(0, 1.2)

# plot data
pframe21.ax.plot(delay_time, eos, color='orangered', lw=1.5)
pframe21_inset.ax.plot(freq, np.abs(eos_fft) / np.max(np.abs(eos_fft)), color='orangered', lw=1)
pframe21_inset.ax.fill_between(freq, np.abs(eos_fft) / np.max(np.abs(eos_fft)), 0, color='orangered', alpha=0.5)

# set other attributes
pframe21.ax.set_xlabel(r'Delay time, $t$ (fs)', fontsize=fontsize_main, labelpad=3)
pframe21.ax.set_ylabel(r'$E_{\mathrm{MIR}}$ (a.u.)', fontsize=fontsize_main, labelpad=2)
pframe21.ax.hlines(0.0, delay_time[0], delay_time[-1], color='black', lw=0.25)
pframe21_inset.ax.set_xlabel(r'$\nu$ (THz)', fontsize=fontsize_main, labelpad=0)
pframe21_inset.ax.set_ylabel(r'$E_{\mathrm{MIR}}(\nu)$', fontsize=fontsize_main, labelpad=1)
pframe21.fix_yaxis_interval(1)

#######################################################################################################################
# End of Plot structure 2
#######################################################################################################################


#######################################################################################################################
# Plot structure 3 (MPC and UV spectra)
#######################################################################################################################
# create empty frame for this structure
eframe3 = absplot.EmptyFrame(mainframe=mainframe, left=95, bottom=65, width=110, height=55)
# eframe3.visualize_frame()

# create single plotting frame
pframe31 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe3, left=10, bottom=10, width=40, height=40)
pframe31.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe32 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe3, left=10 + 55, bottom=10, width=40, height=40)
pframe32.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# load waveform files
tmp_filepath = './strong800.txt'
data_spec1 = np.loadtxt(tmp_filepath)
wavelength1 = data_spec1[:, 0]
intensity_spec1 = data_spec1[:, 1]
intensity_spec1 /= np.max(intensity_spec1)
tmp_filepath = './tisa.txt'
data_spec2 = np.loadtxt(tmp_filepath)
wavelength2 = data_spec2[:, 0]
intensity_spec2 = data_spec2[:, 1]
intensity_spec2 /= np.max(intensity_spec2)
intensity_spec2 -= 0.02
tmp_filepath = './UV.txt'
data_spec3 = np.loadtxt(tmp_filepath)
wavelength3 = data_spec3[:, 0]
intensity_spec3 = data_spec3[:, 1]
intensity_spec3 /= np.max(intensity_spec3)

# calculate band width
idx_peak = np.argmax(intensity_spec3)
idx_w1 = np.argmin(np.abs(intensity_spec3[:idx_peak] - 0.5) ** 2)
idx_w2 = np.argmin(np.abs(intensity_spec3[idx_peak:] - 0.5) ** 2) + idx_peak
w1 = wavelength3[idx_w1]
w2 = wavelength3[idx_w2]

# set plot range
intensity_spec_min_plot = 0
intensity_spec_max_plot = 1.05
wmin_plot_uv = 370
wmax_plot_uv = 460
pframe31.ax.set_xlim(680, 920)
pframe31.ax.set_ylim(intensity_spec_min_plot, intensity_spec_max_plot)
pframe32.ax.set_xlim(wmin_plot_uv, wmax_plot_uv)
pframe32.ax.set_ylim(intensity_spec_min_plot, intensity_spec_max_plot)

# plot data
pframe31.ax.plot(wavelength1, intensity_spec1, color='red', lw=1.5, label='MPC')
pframe31.ax.fill_between(wavelength1, intensity_spec1, 0, color='red', alpha=0.3)
pframe31.ax.plot(wavelength2, intensity_spec2, color='black', lw=1, label='Ti:Sa')
pframe31.ax.legend(fontsize=fontsize_main, loc='upper right', frameon=False, handlelength=1, handletextpad=0.2)
pframe32.ax.plot(wavelength3, intensity_spec3, color='blue', lw=1.5)
pframe32.ax.fill_between(wavelength3, intensity_spec3, 0, color='blue', alpha=0.3)
pframe32.ax.hlines(0.5, wmin_plot_uv, wmax_plot_uv, color='blue', lw=0.5, ls=(0, (4, 4)))
arrow_start_rel = (w1 - wmin_plot_uv) / (wmax_plot_uv - wmin_plot_uv)
arrow_end_rel = (w2 - wmin_plot_uv) / (wmax_plot_uv - wmin_plot_uv)
arrow_vpos_rel = (0.5 - intensity_spec_min_plot) / (intensity_spec_max_plot - intensity_spec_min_plot)
pframe32.ax.arrow(0.5 * (arrow_end_rel + arrow_start_rel), arrow_vpos_rel, 0.5 * (arrow_end_rel - arrow_start_rel), 0,
                  facecolor='black', edgecolor='black', lw=1,
                  head_width=0.03, head_length=0.06, overhang=0.0,
                  length_includes_head=True, clip_on=False, transform=pframe32.ax.transAxes)
pframe32.ax.arrow(0.5 * (arrow_end_rel + arrow_start_rel), arrow_vpos_rel, -0.5 * (arrow_end_rel - arrow_start_rel), 0,
                  facecolor='black', edgecolor='black', lw=1,
                  head_width=0.03, head_length=0.06, overhang=0.0,
                  length_includes_head=True, clip_on=False, transform=pframe32.ax.transAxes)
pframe32.ax.text(0.49, 0.35, '{:.0f} nm'.format(w2 - w1), fontsize=fontsize_main,
                 transform=pframe32.ax.transAxes, horizontalalignment='center')

# set other attributes
pframe31.fix_yaxis_interval(0.5)
pframe32.fix_yaxis_interval(0.5)
pframe31.ax.set_xlabel(r'Wavelength (nm)', fontsize=fontsize_main, labelpad=3)
pframe31.ax.set_ylabel(r'Intensity (a.u.)', fontsize=fontsize_main, labelpad=3)
pframe32.ax.set_xlabel(r'Wavelength (nm)', fontsize=fontsize_main, labelpad=3)
pframe32.ax.set_ylabel(r'Intensity (a.u.)', fontsize=fontsize_main, labelpad=3)
pframe32.fix_xaxis_interval(25)

#######################################################################################################################
# End of Plot structure 3
#######################################################################################################################


#######################################################################################################################
# Plot structure 4 (XFROG)
#######################################################################################################################
def cmap_original(cmap_name):
    if cmap_name == 'whitejet':
        list_colormap = np.loadtxt('./colormap_whitejet.dat', delimiter=',')
        return ListedColormap(list_colormap, name='whitejet')


# create empty frame for this structure
eframe4 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=0, width=125, height=65)
# eframe4.visualize_frame()

# create single plotting frame
pframe41 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe4, left=14, bottom=10 + 26, width=35, height=24)
pframe41.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe42 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe4, left=14, bottom=10, width=35, height=24)
pframe42.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe43 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe4, left=14 + 50, bottom=10, width=50, height=50)
pframe43.add_right_axis()
pframe43.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# set plot range
pframe41.ax.tick_params(labelbottom=False)
xfrog_freq_min = 1000
xfrog_freq_max = 1250
xfrog_time_min = -300
xfrog_time_max = 250
figs_clim_xfrog_measured = [-3.3, 0]
figs_clim_xfrog_reconst = [-3.5, 0]
const_avoid_zero = 1e-15  # constant to be added for avoiding zero in log10
tmin_plot_pulse = -50
tmax_plot_pulse = 50
intensity_min_plot_pulse = 0
intensity_max_plot_pulse = 1.05
phase_min_plot_pulse = -12
phase_max_plot_pulse = 12
pframe43.ax.set_xlim(tmin_plot_pulse, tmax_plot_pulse)
pframe43.ax.set_ylim(intensity_min_plot_pulse, intensity_max_plot_pulse)
pframe43.ax_right.set_ylim(phase_min_plot_pulse, phase_max_plot_pulse)

# assign image data
coord_filepath = './PCGPA_XFROG_reconstruction_FROG_coordinates_XFROG_carrier_svd_v1.dat'
data_xfrog_coord = np.loadtxt(coord_filepath, comments=';')
delay_time_xfrog = data_xfrog_coord[:, 0]
freq_xfrog = data_xfrog_coord[:, 1]
extent_frog = [delay_time_xfrog[0], delay_time_xfrog[-1], freq_xfrog[0], freq_xfrog[-1]]
img_filepath1 = './PCGPA_XFROG_reconstruction_FROG_measured_XFROG_carrier_svd_v1.dat'
img_xfrog_measured = np.loadtxt(img_filepath1)
pframe41.assign_image(np.log10(np.abs(img_xfrog_measured + const_avoid_zero)
                               / np.max(np.abs(img_xfrog_measured + const_avoid_zero))),
                      extent=extent_frog,
                      xmin=xfrog_time_min, xmax=xfrog_time_max, ymin=xfrog_freq_min, ymax=xfrog_freq_max,
                      colormap=cmap_original('whitejet'), clim=figs_clim_xfrog_measured,
                      transpose=False, inversex=False)
img_filepath2 = './PCGPA_XFROG_reconstruction_FROG_reconstructed_XFROG_carrier_svd_v1.dat'
img_xfrog_reconstructed = np.loadtxt(img_filepath2)
pframe42.assign_image(np.log10(np.abs(img_xfrog_reconstructed + const_avoid_zero)
                               / np.max(np.abs(img_xfrog_reconstructed + const_avoid_zero))),
                      extent=extent_frog,
                      xmin=xfrog_time_min, xmax=xfrog_time_max, ymin=xfrog_freq_min, ymax=xfrog_freq_max,
                      colormap=cmap_original('whitejet'), clim=figs_clim_xfrog_reconst,
                      transpose=False, inversex=False)

# plot data
filepath1 = './PCGPA_XFROG_reconstruction_pulse_analyzed_XFROG_carrier_svd_v1.dat'
data_xfrog_pulse = np.loadtxt(filepath1, comments=';')
delay_time_xfrog_pulse = data_xfrog_pulse[:, 0]
intensity_xfrog_pulse = data_xfrog_pulse[:, 1]
xfrog_pulse_phase = data_xfrog_pulse[:, 2]

# interpolation for fine offset
t_offset = 2.5
delay_time_xfrog_pulse_interp = np.linspace(delay_time_xfrog_pulse[0], delay_time_xfrog_pulse[-1], 4000)
interp_func = interpolate.interp1d(delay_time_xfrog_pulse+t_offset, intensity_xfrog_pulse, fill_value='extrapolate')
intensity_xfrog_pulse_interp = interp_func(delay_time_xfrog_pulse_interp)
interp_func = interpolate.interp1d(delay_time_xfrog_pulse+t_offset, xfrog_pulse_phase, fill_value='extrapolate')
xfrog_pulse_phase_interp = interp_func(delay_time_xfrog_pulse_interp)

idx_peak = np.argmax(intensity_xfrog_pulse_interp)
idx_w1 = np.argmin(np.abs(intensity_xfrog_pulse_interp[:idx_peak] - 0.5) ** 2)
idx_w2 = np.argmin(np.abs(intensity_xfrog_pulse_interp[idx_peak:] - 0.5) ** 2) + idx_peak
t1 = delay_time_xfrog_pulse_interp[idx_w1]
t2 = delay_time_xfrog_pulse_interp[idx_w2]
# print(t2 - t1)

pframe43.ax.plot(delay_time_xfrog_pulse_interp, intensity_xfrog_pulse_interp, color='blue', lw=1.5)
pframe43.ax.fill_between(delay_time_xfrog_pulse_interp, intensity_xfrog_pulse_interp, 0, color='blue', alpha=0.3)
pframe43.ax_right.plot(delay_time_xfrog_pulse_interp, xfrog_pulse_phase_interp, color='orange', lw=1)
pframe43.ax.hlines(0.5, tmin_plot_pulse, tmax_plot_pulse, color='blue', lw=0.5, ls=(0, (4, 4)))
arrow_start_rel = (t1 - tmin_plot_pulse) / (tmax_plot_pulse - tmin_plot_pulse)
arrow_end_rel = (t2 - tmin_plot_pulse) / (tmax_plot_pulse - tmin_plot_pulse)
arrow_vpos_rel = (0.5 - intensity_min_plot_pulse) / (intensity_max_plot_pulse - intensity_min_plot_pulse)
pframe43.ax.arrow(0.5 * (arrow_end_rel + arrow_start_rel), arrow_vpos_rel, 0.5 * (arrow_end_rel - arrow_start_rel), 0,
                  facecolor='black', edgecolor='black', lw=1,
                  head_width=0.025, head_length=0.04, overhang=0.0,
                  length_includes_head=True, clip_on=False, transform=pframe43.ax.transAxes)
pframe43.ax.arrow(0.5 * (arrow_end_rel + arrow_start_rel), arrow_vpos_rel, -0.5 * (arrow_end_rel - arrow_start_rel), 0,
                  facecolor='black', edgecolor='black', lw=1,
                  head_width=0.025, head_length=0.04, overhang=0.0,
                  length_includes_head=True, clip_on=False, transform=pframe43.ax.transAxes)
pframe43.ax.text(0.49, 0.35, '{:.0f} fs'.format(t2 - t1), fontsize=fontsize_main,
                 transform=pframe43.ax.transAxes, horizontalalignment='center')

# set other attributes
pframe42.ax.set_xlabel(r'Delay time (fs)', fontsize=fontsize_main, labelpad=3)
pframe42.ax.set_ylabel(y=1.1, ylabel=r'Frequency (THz)', fontsize=fontsize_main, labelpad=3)
pframe43.ax.set_xlabel(r'Delay time (fs)', fontsize=fontsize_main, labelpad=3)
pframe43.ax.set_ylabel(r'Intensity (a.u.)', fontsize=fontsize_main, labelpad=3)
pframe43.ax_right.set_ylabel(r'Phase (rad)', fontsize=fontsize_main, labelpad=0)
pframe43.ax_right.tick_params(axis='y', colors='orange')
pframe43.ax_right.yaxis.label.set_color('orange')
pframe43.fix_xaxis_interval(20)

#######################################################################################################################
# End of Plot structure 4
#######################################################################################################################


#######################################################################################################################
# Plot structure 5 (2PPE intensity)
#######################################################################################################################
# create empty frame for this structure
eframe5 = absplot.EmptyFrame(mainframe=mainframe, left=128, bottom=0, width=75, height=65)
# eframe5.visualize_frame()

# create single plotting frame
pframe51 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe5, left=12, bottom=10, width=60, height=50)
pframe51.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# set plot range
pframe51.ax.set_xlim(1.1, 2.4)
pframe51.ax.set_ylim(0, 1.6)

# plot data
filepath1 = './extracted_EDC_v04.dat'
data_EDC = np.loadtxt(filepath1, comments=';')
EDC_axis = data_EDC[:, 0]
EDC = data_EDC[:, 1:9]
EDC_norm = data_EDC[:, 9:17]
EDC -= np.min(EDC)
list_color = ['orangered', 'orange', 'lawngreen', 'dodgerblue', 'blue']
list_label = ['-0.6 mm', '-0.4 mm', '-0.2 mm', 'optimal', '+0.2 mm']
list_idx_data = [3, 4, 5, 6, 7]
for idx, idx_data in enumerate(list_idx_data):
    pframe51.ax.plot(EDC_axis, EDC[:, idx_data] / 10, color=list_color[idx], lw=1,
                     label='{}'.format(list_label[idx]))
pframe51.ax.legend(fontsize=fontsize_main, loc='upper right', frameon=False)
pframe51.fix_xaxis_interval(0.2)
pframe51.fix_yaxis_interval(0.5)

# set other attributes
pframe51.ax.set_xlabel(r'$ϵ_{\mathrm{kin}}$ (eV)', fontsize=fontsize_main, labelpad=3)
pframe51.ax.set_ylabel(r'$I_{\mathrm{2PPE}}$ (a.u.)', fontsize=fontsize_main, labelpad=3)

#######################################################################################################################
# End of Plot structure 5
#######################################################################################################################


#######################################################################################################################
# Figure symbols
#######################################################################################################################

mainframe.ax.text(0.5, 195, 'a', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(0.5, 114, 'b', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(93, 114, 'c', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(148, 114, 'd', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(0.5, 58, 'e', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(54, 58, 'f', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(128, 58, 'g', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.set_zorder(1)

#######################################################################################################################
# export image files
#######################################################################################################################

plt.savefig(outdir + '/' + script_name + version + '.png', dpi=160)
# plt.savefig(outdir + '/' + script_name + version + '.png', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.jpg', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.tiff', dpi=600)  # this creates a huge TIFF file!
# cmd = "convert {}.png {}.tiff".format(outdir + '/' + script_name + version, outdir + '/' + script_name + version)
# subprocess.call(cmd.split())  # execute the command from this script
# plt.savefig(outdir + '/' + script_name + version + '.pdf', dpi=600)

plt.show()
