import os
import re
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import subprocess

# load homemade python moduels
import absplot_v03 as absplot
import zapes_utilities_v03 as zutil

# matplotlib backend selection
matplotlib.use('TkAgg')

# file version check
this_file_name = os.path.basename(__file__)
regular_expression = re.compile(r'(edf_freq_dep_)(\w+)(.py)')
regular_group = regular_expression.match(this_file_name).groups()
script_name = str(regular_group[0])
version = str(regular_group[1])
# print('file version:', version)

# physical constant
hb = 0.65821  # Dirac constant, eV•fs unit
me = 5.68563e-2  # electron mass, eV•fs^2/A^2 unit
kb = 8.61733e-5  # Boltzmann constant, eV/K unit
elem_charge = 1.602176634e-19  # elementary charge, C unit

# figure parameters
kmin_plot = -0.15
kmax_plot = 0.15
emin_plot = -0.3
emax_plot = 0.35
tmin_plot = -115
tmax_plot = 115
colormap = 'gnu_v2_r'
ticks_color = 'black'
figs_clim_second_derivative_29THz = [0.003, 0.03]
figs_clim_second_derivative_31THz = [0.003, 0.03]
figs_clim_second_derivative_40THz = [0.001, 0.025]

lw_dft_thick = 1.5
lw_dft_thin = 0.5
color_dft = 'gray'
color_dft_projection = 'gray'
alpha_dft_projection = 0.2

fontsize_main = 8
fontsize_colorbar = 7
fontsize_symbol = 9

ticks_xpad = 4
ticks_ypad = 3

ticks_width = 0.75
ticks_length = 3.5

# directory paths
outdir = '.'  # output directory


matplotlib.rcParams['font.family'] = 'Arial'
mainframe = absplot.MainFrame(width=205, height=152)

#######################################################################################################################
# Plot structure 11 (ARPES images, 29 THz)
#######################################################################################################################
# create empty frame for this structure
eframe1 = absplot.EmptyFrame(mainframe=mainframe, left=42.5, bottom=100, width=162, height=48)
# eframe1.visualize_frame()

# create plot frames
img_left = 10
img_bottom = 9
img_width = 28
img_hoffset = 29.5
img_height = 28 * (48/35)
pframe11 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left, bottom=img_bottom, width=img_width, height=img_height)
pframe11.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe12 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset, bottom=img_bottom, width=img_width, height=img_height)
pframe12.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe13 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 2, bottom=img_bottom, width=img_width, height=img_height)
pframe13.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe14 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 3, bottom=img_bottom, width=img_width, height=img_height)
pframe14.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe15 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 4, bottom=img_bottom, width=img_width, height=img_height)
pframe15.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)

# assign image data
img_filepath1 = './image_processed_2ppe7629_0_sc_2deriv_narrow_shift2d_own_n10_v06.png'
image_processed1 = zutil.ImageProcessed(img_filepath1, ef_shift=0.02)
pframe11.assign_image(image_processed1.image, extent=image_processed1.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_29THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath2 = './image_processed_2ppe7629_5_sc_2deriv_narrow_shift2d_own_n10_v06.png'
image_processed2 = zutil.ImageProcessed(img_filepath2, ef_shift=0.02)
pframe12.assign_image(image_processed2.image, extent=image_processed2.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_29THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath3 = './image_processed_2ppe7629_75_sc_2deriv_narrow_shift2d_n40_v10.png'
image_processed3 = zutil.ImageProcessed(img_filepath3, ef_shift=0.02)
pframe13.assign_image(image_processed3.image, extent=image_processed3.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_29THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath4 = './image_processed_2ppe7629_10_sc_2deriv_narrow_shift2d_own_n10_v06.png'
image_processed4 = zutil.ImageProcessed(img_filepath4, ef_shift=0.02)
pframe14.assign_image(image_processed4.image, extent=image_processed4.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_29THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath5 = './image_processed_2ppe7629_13_sc_2deriv_narrow_shift2d_own_n10_v06.png'
image_processed5 = zutil.ImageProcessed(img_filepath5, ef_shift=0.02)
pframe15.assign_image(image_processed5.image, extent=image_processed5.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_29THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)

# load DFT calculation data
data_dft_plot = np.loadtxt('./dft_plot_data_v1.dat', comments=';')
k_plot_dft = data_dft_plot[:, 0]
band_dft_SS1 = data_dft_plot[:, 1]
band_dft_SS2 = data_dft_plot[:, 2]
projection_dft_VB1 = data_dft_plot[:, 3]
projection_dft_VB2 = data_dft_plot[:, 4]
projection_dft_CB1 = data_dft_plot[:, 5]
projection_dft_CB2 = data_dft_plot[:, 6]
band_dft_VB1 = data_dft_plot[:, 7]
band_dft_VB2 = data_dft_plot[:, 8]
band_dft_VB3 = data_dft_plot[:, 9]
band_dft_VB4 = data_dft_plot[:, 10]
band_dft_CB1 = data_dft_plot[:, 11]
band_dft_CB2 = data_dft_plot[:, 12]
band_dft_CB3 = data_dft_plot[:, 13]
band_dft_CB4 = data_dft_plot[:, 14]

# load Floquet center frequency
tmp_filepath = './waveform_params_2ppe7629_shift2d_n40_v19.dat'
data_waveform_params = np.loadtxt(tmp_filepath, comments=';')
freq_fit_parallel = data_waveform_params[5]

# plot DFT band structures
for pframe in [pframe11, pframe12, pframe13, pframe14, pframe15]:
    pframe.ax.plot(k_plot_dft, band_dft_SS1, color=color_dft, lw=lw_dft_thick)
    pframe.ax.plot(k_plot_dft, band_dft_SS2, color=color_dft, lw=lw_dft_thick)
    pframe.ax.fill_between(k_plot_dft, projection_dft_VB1, projection_dft_VB2, 
                           facecolor=color_dft_projection, alpha=alpha_dft_projection)
    pframe.ax.fill_between(k_plot_dft, projection_dft_CB1, projection_dft_CB2, 
                           facecolor=color_dft_projection, alpha=alpha_dft_projection)
    if pframe in [pframe11, pframe15]:
        pframe.ax.plot(k_plot_dft, band_dft_VB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB4, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB4, color=color_dft, lw=lw_dft_thin)
    else:
        e_Floquet = hb * 2.0 * np.pi * freq_fit_parallel * 1e-3
        # print(e_Floquet)
        for n_Floquet in [-3, -2, -1, 1, 2, 3]:
            pframe.ax.plot(k_plot_dft, band_dft_SS1 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)
            pframe.ax.plot(k_plot_dft, band_dft_SS2 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)

# index in the original streaking analysis
# can be checked based on the process_image file names and descriptions in corresponding source files
list_delay_time = [-120, -48, -22, 4, 120]

# set other attributes
pframe11.ax.set_ylabel(r'$ϵ-ϵ_{\mathrm{F}}$ (eV)', fontsize=fontsize_main, labelpad=1)
for idx, pframe in enumerate([pframe11, pframe12, pframe13, pframe14, pframe15]):
    pframe.ax.set_xlabel(r'$k_y$ (Å$^{-1}$)', fontsize=fontsize_main, labelpad=3)
    pframe.ax.hlines(0, image_processed1.kmin, image_processed1.kmax,
                     color=ticks_color, lw=0.5, ls=(0, (8, 5)))
    str_sign = ''
    pframe.ax.text(0.5, 0.90, '{}{:.0f} fs'.format(str_sign, list_delay_time[idx]),
                   transform=pframe.ax.transAxes, horizontalalignment='center',
                   fontsize=fontsize_main, color=ticks_color)
    pframe.fix_yaxis_interval(0.1)
for pframe in [pframe12, pframe13, pframe14, pframe15]:
    pframe.ax.tick_params(labelleft=False)

# add colorbar
pframe15.add_colorbar(cleft=30.5, cbottom=6, cwidth=1.5, cheight=20)
pframe15.modify_colorbar(ticks_fontsize=fontsize_main, ticks_color=ticks_color, show_ticks=False)
pframe15.cax.text(0.5, 1.05, 'max', transform=pframe15.cax.transAxes, fontsize=fontsize_colorbar, 
                  horizontalalignment='center')
pframe15.cax.text(0.5, -0.13, 'min', transform=pframe15.cax.transAxes, fontsize=fontsize_colorbar, 
                  horizontalalignment='center')

#######################################################################################################################
# End of Plot structure 11
#######################################################################################################################


#######################################################################################################################
# Plot structure 21 (Waveform panel, 29 THz)
#######################################################################################################################
# create empty frame for this structure
eframe2 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=100, width=42, height=45)
# eframe2.visualize_frame()

# create single plotting frame
pframe21 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2, left=11.5, bottom=14, width=27, height=25)
pframe21.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# load waveform files
# can be checked in 2ppe_evaluation/2ppe7629/image_analysis_selected_v06.py
list_idx_data = [0, 62, 75, 87, 158]
tmp_filepath = './waveform_extraction_2ppe7629_shift2d_n40_v19.dat'
data_waveform = np.loadtxt(tmp_filepath,
                           dtype=complex,
                           converters={1: lambda s: complex(s.decode().replace('i', 'j'))},
                           comments=';')
factor_uv_deconvolve_parallel = 0.8
delay_time = np.real(data_waveform[:, 0])
E_parallel_deconvolved = np.real(data_waveform[:, 4]) / factor_uv_deconvolve_parallel * 1e-3
E_parallel_smooth_deconvolved = np.real(data_waveform[:, 6]) / factor_uv_deconvolve_parallel * 1e-3
E_parallel_fit_rescaled_deconvolved = np.real(data_waveform[:, 10]) / factor_uv_deconvolve_parallel * 1e-3

# set plot range
pframe21.ax.set_xlim(tmin_plot, tmax_plot)
scale_efield = np.max(np.abs(E_parallel_fit_rescaled_deconvolved)) * 1.4
pframe21.ax.set_ylim(-scale_efield, scale_efield)

# plot data
pframe21.ax.plot(delay_time, E_parallel_deconvolved,
                 color='black', lw=0, marker='o', ms=2, markerfacecolor='none')
pframe21.ax.plot(delay_time, E_parallel_fit_rescaled_deconvolved, color='black', lw=1)
for idx_data in list_idx_data:
    pframe21.ax.vlines(delay_time[idx_data], -scale_efield, scale_efield, color='blue', lw=1.5, alpha=0.3)

# set other attributes
pframe21.ax.text(0.5, 1.08, '29 THz', transform=pframe21.ax.transAxes, horizontalalignment='center',
                 fontsize=fontsize_main, color='black')
pframe21.ax.set_xlabel(r'Delay time, $t$ (fs)', fontsize=fontsize_main, labelpad=3)
pframe21.ax.set_ylabel(r'$E_y$ (MV cm$^{-1}$)', fontsize=fontsize_main, labelpad=2)
pframe21.ax.hlines(0.0, delay_time[0], delay_time[-1], color='black', lw=0.25)

#######################################################################################################################
# End of Plot structure 21
#######################################################################################################################


#######################################################################################################################
# Plot structure 12 (ARPES images, 31 THz)
#######################################################################################################################
# create empty frame for this structure
eframe1 = absplot.EmptyFrame(mainframe=mainframe, left=42.5, bottom=50, width=162, height=48)
# eframe1.visualize_frame()

# create plot frames
img_left = 10
img_bottom = 9
img_width = 28
img_hoffset = 29.5
img_height = 28 * (48/35)
pframe11 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left, bottom=img_bottom, width=img_width, height=img_height)
pframe11.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe12 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset, bottom=img_bottom, width=img_width, height=img_height)
pframe12.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe13 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 2, bottom=img_bottom, width=img_width, height=img_height)
pframe13.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe14 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 3, bottom=img_bottom, width=img_width, height=img_height)
pframe14.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe15 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 4, bottom=img_bottom, width=img_width, height=img_height)
pframe15.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)

# assign image data
img_filepath1 = './image_processed_2ppe7586_0_sc_2deriv_narrow2_shift2d_n40_v10.png'
image_processed1 = zutil.ImageProcessed(img_filepath1, ef_shift=0.02)
pframe11.assign_image(image_processed1.image, extent=image_processed1.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_31THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath2 = './image_processed_2ppe7586_2_sc_2deriv_narrow2_shift2d_own_n10_v06.png'
image_processed2 = zutil.ImageProcessed(img_filepath2, ef_shift=0.02)
pframe12.assign_image(image_processed2.image, extent=image_processed2.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_31THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath3 = './image_processed_2ppe7586_6_sc_2deriv_narrow2_shift2d_own_n10_v06.png'
image_processed3 = zutil.ImageProcessed(img_filepath3, ef_shift=0.02)
pframe13.assign_image(image_processed3.image, extent=image_processed3.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_31THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath4 = './image_processed_2ppe7586_11_sc_2deriv_narrow2_shift2d_own_n10_v06.png'
image_processed4 = zutil.ImageProcessed(img_filepath4, ef_shift=0.02)
pframe14.assign_image(image_processed4.image, extent=image_processed4.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_31THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath5 = './image_processed_2ppe7586_15_sc_2deriv_narrow2_shift2d_own_n10_v06.png'
image_processed5 = zutil.ImageProcessed(img_filepath5, ef_shift=0.02)
pframe15.assign_image(image_processed5.image, extent=image_processed5.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_31THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)

# load DFT calculation data
data_dft_plot = np.loadtxt('./dft_plot_data_v1.dat', comments=';')
k_plot_dft = data_dft_plot[:, 0]
band_dft_SS1 = data_dft_plot[:, 1]
band_dft_SS2 = data_dft_plot[:, 2]
projection_dft_VB1 = data_dft_plot[:, 3]
projection_dft_VB2 = data_dft_plot[:, 4]
projection_dft_CB1 = data_dft_plot[:, 5]
projection_dft_CB2 = data_dft_plot[:, 6]
band_dft_VB1 = data_dft_plot[:, 7]
band_dft_VB2 = data_dft_plot[:, 8]
band_dft_VB3 = data_dft_plot[:, 9]
band_dft_VB4 = data_dft_plot[:, 10]
band_dft_CB1 = data_dft_plot[:, 11]
band_dft_CB2 = data_dft_plot[:, 12]
band_dft_CB3 = data_dft_plot[:, 13]
band_dft_CB4 = data_dft_plot[:, 14]

# load Floquet center frequency
tmp_filepath = './waveform_params_2ppe7586_shift2d_n40_v19.dat'
data_waveform_params = np.loadtxt(tmp_filepath, comments=';')
freq_fit_parallel = data_waveform_params[5]
# print(freq_fit_parallel)

# plot DFT band structures
for pframe in [pframe11, pframe12, pframe13, pframe14, pframe15]:
    pframe.ax.plot(k_plot_dft, band_dft_SS1, color=color_dft, lw=lw_dft_thick)
    pframe.ax.plot(k_plot_dft, band_dft_SS2, color=color_dft, lw=lw_dft_thick)
    pframe.ax.fill_between(k_plot_dft, projection_dft_VB1, projection_dft_VB2, 
                           facecolor=color_dft_projection, alpha=alpha_dft_projection)
    pframe.ax.fill_between(k_plot_dft, projection_dft_CB1, projection_dft_CB2, 
                           facecolor=color_dft_projection, alpha=alpha_dft_projection)
    if pframe in [pframe11, pframe15]:
        pframe.ax.plot(k_plot_dft, band_dft_VB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB4, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB4, color=color_dft, lw=lw_dft_thin)
    else:
        e_Floquet = hb * 2.0 * np.pi * freq_fit_parallel * 1e-3
        # print(e_Floquet)
        for n_Floquet in [-3, -2, -1, 1, 2, 3]:
            pframe.ax.plot(k_plot_dft, band_dft_SS1 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)
            pframe.ax.plot(k_plot_dft, band_dft_SS2 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)

# index in the original streaking analysis
# can be checked based on the process_image file names and descriptions in corresponding source files
list_delay_time = [-120, -54, -22, 18, 120]

# set other attributes
pframe11.ax.set_ylabel(r'$ϵ-ϵ_{\mathrm{F}}$ (eV)', fontsize=fontsize_main, labelpad=1)
for idx, pframe in enumerate([pframe11, pframe12, pframe13, pframe14, pframe15]):
    pframe.ax.set_xlabel(r'$k_y$ (Å$^{-1}$)', fontsize=fontsize_main, labelpad=3)
    pframe.ax.hlines(0, image_processed1.kmin, image_processed1.kmax,
                     color=ticks_color, lw=0.5, ls=(0, (8, 5)))
    str_sign = ''
    pframe.ax.text(0.5, 0.90, '{}{:.0f} fs'.format(str_sign, list_delay_time[idx]),
                   transform=pframe.ax.transAxes, horizontalalignment='center',
                   fontsize=fontsize_main, color=ticks_color)
    pframe.fix_yaxis_interval(0.1)
for pframe in [pframe12, pframe13, pframe14, pframe15]:
    pframe.ax.tick_params(labelleft=False)
#######################################################################################################################
# End of Plot structure 12
#######################################################################################################################


#######################################################################################################################
# Plot structure 22 (Waveform panel, 31 THz)
#######################################################################################################################
# create empty frame for this structure
eframe2 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=50, width=42, height=45)
# eframe2.visualize_frame()

# create single plotting frame
pframe21 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2, left=11.5, bottom=14, width=27, height=25)
pframe21.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# load waveform files
# can be checked in 2ppe_evaluation/2ppe7586/image_analysis_selected_v06.py
list_idx_data = [0, 54, 70, 90, 140]
tmp_filepath = './waveform_extraction_2ppe7586_shift2d_n40_v19.dat'
data_waveform = np.loadtxt(tmp_filepath,
                           dtype=complex,
                           converters={1: lambda s: complex(s.decode().replace('i', 'j'))},
                           comments=';')
factor_uv_deconvolve_parallel = 0.8
delay_time = np.real(data_waveform[:, 0])
E_parallel_deconvolved = np.real(data_waveform[:, 4]) / factor_uv_deconvolve_parallel * 1e-3
E_parallel_smooth_deconvolved = np.real(data_waveform[:, 6]) / factor_uv_deconvolve_parallel * 1e-3
E_parallel_fit_rescaled_deconvolved = np.real(data_waveform[:, 10]) / factor_uv_deconvolve_parallel * 1e-3

# set plot range
pframe21.ax.set_xlim(tmin_plot, tmax_plot)
scale_efield = np.max(np.abs(E_parallel_fit_rescaled_deconvolved)) * 1.5
pframe21.ax.set_ylim(-scale_efield, scale_efield)

# plot data
pframe21.ax.plot(delay_time, E_parallel_deconvolved,
                 color='black', lw=0, marker='o', ms=2, markerfacecolor='none')
pframe21.ax.plot(delay_time, E_parallel_fit_rescaled_deconvolved, color='black', lw=1)
for idx_data in list_idx_data:
    pframe21.ax.vlines(delay_time[idx_data], -scale_efield, scale_efield, color='blue', lw=1.5, alpha=0.3)

# set other attributes
pframe21.ax.text(0.5, 1.08, '31 THz', transform=pframe21.ax.transAxes, horizontalalignment='center',
                 fontsize=fontsize_main, color='black')
pframe21.ax.set_xlabel(r'Delay time, $t$ (fs)', fontsize=fontsize_main, labelpad=3)
pframe21.ax.set_ylabel(r'$E_y$ (MV cm$^{-1}$)', fontsize=fontsize_main, labelpad=2)
pframe21.ax.hlines(0.0, delay_time[0], delay_time[-1], color='black', lw=0.25)

#######################################################################################################################
# End of Plot structure 22
#######################################################################################################################


#######################################################################################################################
# Plot structure 13 (ARPES images, 40 THz)
#######################################################################################################################
# create empty frame for this structure
eframe1 = absplot.EmptyFrame(mainframe=mainframe, left=42.5, bottom=0, width=162, height=48)
# eframe1.visualize_frame()

# create plot frames
img_left = 10
img_bottom = 9
img_width = 28
img_hoffset = 29.5
img_height = 28 * (48/35)
pframe11 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left, bottom=img_bottom, width=img_width, height=img_height)
pframe11.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe12 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset, bottom=img_bottom, width=img_width, height=img_height)
pframe12.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe13 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 2, bottom=img_bottom, width=img_width, height=img_height)
pframe13.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe14 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 3, bottom=img_bottom, width=img_width, height=img_height)
pframe14.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)
pframe15 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 4, bottom=img_bottom, width=img_width, height=img_height)
pframe15.modify_ticks(ticks_fontsize=fontsize_main, ticks_color=ticks_color,
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad,
                      ticks_width=ticks_width, ticks_length=ticks_length)

# assign image data
img_filepath1 = './image_processed_2ppe7405_0_sc_2deriv_narrow_shift2d_n20_v11.png'
image_processed1 = zutil.ImageProcessed(img_filepath1, ef_shift=0.02)
pframe11.assign_image(image_processed1.image, extent=image_processed1.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_40THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath2 = './image_processed_2ppe7405_76_sc_2deriv_narrow_shift2d_n20_v11.png'
image_processed2 = zutil.ImageProcessed(img_filepath2, ef_shift=0.02)
pframe12.assign_image(image_processed2.image, extent=image_processed2.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_40THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath3 = './image_processed_2ppe7405_83_sc_2deriv_narrow_shift2d_n20_v11.png'
image_processed3 = zutil.ImageProcessed(img_filepath3, ef_shift=0.02)
pframe13.assign_image(image_processed3.image, extent=image_processed3.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_40THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath4 = './image_processed_2ppe7405_89_sc_2deriv_narrow_shift2d_n20_v11.png'
image_processed4 = zutil.ImageProcessed(img_filepath4, ef_shift=0.02)
pframe14.assign_image(image_processed4.image, extent=image_processed4.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_40THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath5 = './image_processed_2ppe7405_155_sc_2deriv_narrow_shift2d_n20_v11.png'
image_processed5 = zutil.ImageProcessed(img_filepath5, ef_shift=0.02)
pframe15.assign_image(image_processed5.image, extent=image_processed5.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap=colormap, clim=figs_clim_second_derivative_40THz, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)

# load DFT calculation data
data_dft_plot = np.loadtxt('./dft_plot_data_v1.dat', comments=';')
k_plot_dft = data_dft_plot[:, 0]
band_dft_SS1 = data_dft_plot[:, 1]
band_dft_SS2 = data_dft_plot[:, 2]
projection_dft_VB1 = data_dft_plot[:, 3]
projection_dft_VB2 = data_dft_plot[:, 4]
projection_dft_CB1 = data_dft_plot[:, 5]
projection_dft_CB2 = data_dft_plot[:, 6]
band_dft_VB1 = data_dft_plot[:, 7]
band_dft_VB2 = data_dft_plot[:, 8]
band_dft_VB3 = data_dft_plot[:, 9]
band_dft_VB4 = data_dft_plot[:, 10]
band_dft_CB1 = data_dft_plot[:, 11]
band_dft_CB2 = data_dft_plot[:, 12]
band_dft_CB3 = data_dft_plot[:, 13]
band_dft_CB4 = data_dft_plot[:, 14]

# load Floquet center frequency
tmp_filepath = './waveform_params_2ppe7405_shift2d_n40_v19.dat'
data_waveform_params = np.loadtxt(tmp_filepath, comments=';')
freq_fit_parallel = data_waveform_params[5]

# plot DFT band structures
for pframe in [pframe11, pframe12, pframe13, pframe14, pframe15]:
    pframe.ax.plot(k_plot_dft, band_dft_SS1, color=color_dft, lw=lw_dft_thick)
    pframe.ax.plot(k_plot_dft, band_dft_SS2, color=color_dft, lw=lw_dft_thick)
    pframe.ax.fill_between(k_plot_dft, projection_dft_VB1, projection_dft_VB2, 
                           facecolor=color_dft_projection, alpha=alpha_dft_projection)
    pframe.ax.fill_between(k_plot_dft, projection_dft_CB1, projection_dft_CB2, 
                           facecolor=color_dft_projection, alpha=alpha_dft_projection)
    if pframe in [pframe11, pframe15]:
        pframe.ax.plot(k_plot_dft, band_dft_VB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB4, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB4, color=color_dft, lw=lw_dft_thin)
    else:
        e_Floquet = hb * 2.0 * np.pi * freq_fit_parallel * 1e-3
        # print(e_Floquet)
        for n_Floquet in [-3, -2, -1, 1, 2, 3]:
            pframe.ax.plot(k_plot_dft, band_dft_SS1 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)
            pframe.ax.plot(k_plot_dft, band_dft_SS2 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)

# index in the original streaking analysis
# can be checked based on the process_image file names and descriptions in corresponding source files
list_delay_time = [-120, -16, -2, 10, 120]

# set other attributes
pframe11.ax.set_ylabel(r'$ϵ-ϵ_{\mathrm{F}}$ (eV)', fontsize=fontsize_main, labelpad=1)
for idx, pframe in enumerate([pframe11, pframe12, pframe13, pframe14, pframe15]):
    pframe.ax.set_xlabel(r'$k_y$ (Å$^{-1}$)', fontsize=fontsize_main, labelpad=3)
    pframe.ax.hlines(0, image_processed1.kmin, image_processed1.kmax,
                     color=ticks_color, lw=0.5, ls=(0, (8, 5)))
    str_sign = ''
    pframe.ax.text(0.5, 0.90, '{}{:.0f} fs'.format(str_sign, list_delay_time[idx]),
                   transform=pframe.ax.transAxes, horizontalalignment='center',
                   fontsize=fontsize_main, color=ticks_color)
    pframe.fix_yaxis_interval(0.1)
for pframe in [pframe12, pframe13, pframe14, pframe15]:
    pframe.ax.tick_params(labelleft=False)

#######################################################################################################################
# End of Plot structure 13
#######################################################################################################################


#######################################################################################################################
# Plot structure 23 (Waveform panel, 40 THz)
#######################################################################################################################
# create empty frame for this structure
eframe2 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=0, width=42, height=45)
# eframe2.visualize_frame()

# create single plotting frame
pframe21 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2, left=11.5, bottom=14, width=27, height=25)
pframe21.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# load waveform files
# can be checked in 2ppe_evaluation/2ppe7586/image_analysis_selected_v06.py
list_idx_data = [0, 76, 83, 89, 155]
tmp_filepath = './waveform_extraction_2ppe7405_shift2d_n40_v19.dat'
data_waveform = np.loadtxt(tmp_filepath,
                           dtype=complex,
                           converters={1: lambda s: complex(s.decode().replace('i', 'j'))},
                           comments=';')
factor_uv_deconvolve_parallel = 0.4
delay_time = np.real(data_waveform[:, 0])
E_parallel_deconvolved = np.real(data_waveform[:, 4]) / factor_uv_deconvolve_parallel * 1e-3
E_parallel_smooth_deconvolved = np.real(data_waveform[:, 6]) / factor_uv_deconvolve_parallel * 1e-3
E_parallel_fit_rescaled_deconvolved = np.real(data_waveform[:, 10]) / factor_uv_deconvolve_parallel * 1e-3

# set plot range
pframe21.ax.set_xlim(tmin_plot, tmax_plot)
scale_efield = np.max(np.abs(E_parallel_fit_rescaled_deconvolved)) * 1.4
pframe21.ax.set_ylim(-scale_efield, scale_efield)

# plot data
pframe21.ax.plot(delay_time, E_parallel_deconvolved,
                 color='black', lw=0, marker='o', ms=2, markerfacecolor='none')
pframe21.ax.plot(delay_time, E_parallel_fit_rescaled_deconvolved, color='black', lw=1)
for idx_data in list_idx_data:
    pframe21.ax.vlines(delay_time[idx_data], -scale_efield, scale_efield, color='blue', lw=1.5, alpha=0.3)

# set other attributes
pframe21.ax.text(0.5, 1.08, '41 THz', transform=pframe21.ax.transAxes, horizontalalignment='center',
                 fontsize=fontsize_main, color='black')
pframe21.ax.set_xlabel(r'Delay time, $t$ (fs)', fontsize=fontsize_main, labelpad=3)
pframe21.ax.set_ylabel(r'$E_y$ (MV cm$^{-1}$)', fontsize=fontsize_main, labelpad=2)
pframe21.ax.hlines(0.0, delay_time[0], delay_time[-1], color='black', lw=0.25)
pframe21.fix_yaxis_interval(0.4)

#######################################################################################################################
# End of Plot structure 23
#######################################################################################################################


#######################################################################################################################
# Figure symbols
#######################################################################################################################

mainframe.ax.text(0.5, 147, 'a', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(42, 147, 'b', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(0.5, 97, 'c', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(42, 97, 'd', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(0.5, 47, 'e', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(42, 47, 'f', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.set_zorder(1)

#######################################################################################################################
# export image files
#######################################################################################################################

plt.savefig(outdir + '/' + script_name + version + '.png', dpi=160)
# plt.savefig(outdir + '/' + script_name + version + '.png', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.jpg', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.tiff', dpi=600)  # this creates a huge TIFF file!
# cmd = "convert {}.png {}.tiff".format(outdir + '/' + script_name + version, outdir + '/' + script_name + version)
# subprocess.call(cmd.split())  # execute the command from this script
# plt.savefig(outdir + '/' + script_name + version + '.pdf', dpi=600)

plt.show()
