import os
import re
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import subprocess

# load homemade python moduels
import zapes_utilities_v03 as zutil
import absplot_v03 as absplot

# matplotlib backend selection
matplotlib.use('TkAgg')

# file version check
this_file_name = os.path.basename(__file__)
regular_expression = re.compile(r'(edf_raw_data_sequence_)(\w+)(.py)')
regular_group = regular_expression.match(this_file_name).groups()
script_name = str(regular_group[0])
version = str(regular_group[1])
# print('file version:', version)

# physical constant
hb = 0.65821  # Dirac constant, eV•fs unit
me = 5.68563e-2  # electron mass, eV•fs^2/A^2 unit
kb = 8.61733e-5  # Boltzmann constant, eV/K unit
elem_charge = 1.602176634e-19  # elementary charge, C unit

# global figure parameters
figs_clim_second_derivative = [0.0055, 0.035]
figs_clim_raw = [350, 1250]

lw_dft_thick = 1.2
lw_dft_thin = 0.3
color_dft = 'gray'
color_dft_projection = 'gray'
alpha_dft_projection = 0.2

fontsize_main = 8
fontsize_colorbar = 7
fontsize_symbol = 9

ticks_xpad = 4
ticks_ypad = 3

# directory paths
outdir = '.'  # output directory


matplotlib.rcParams['font.family'] = 'Arial'
mainframe = absplot.MainFrame(width=205, height=110)

#######################################################################################################################
# Plot structure 1 (experimental filetered ARPES images)
#######################################################################################################################
# create empty frame for this structure
eframe1 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=55, width=203, height=52)
# eframe1.visualize_frame()

# create plot frames
img_left = 11
img_bottom = 10
factor = 0.85
img_width = 35 * factor  # 35
img_hoffset = 36.75 * factor  # 36.75
img_height = 48 * factor
pframe11 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left, bottom=img_bottom, width=img_width, height=img_height)
pframe11.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe12 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset, bottom=img_bottom, width=img_width, height=img_height)
pframe12.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe13 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 2, bottom=img_bottom, width=img_width, height=img_height)
pframe13.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe14 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 3, bottom=img_bottom, width=img_width, height=img_height)
pframe14.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe15 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 4, bottom=img_bottom, width=img_width, height=img_height)
pframe15.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe16 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=img_left + img_hoffset * 5, bottom=img_bottom, width=img_width, height=img_height)
pframe16.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# assign image data
kmin_plot = -0.15
kmax_plot = 0.15
emin_plot = -0.3
emax_plot = 0.35
img_filepath1 = './image_processed_2ppe8621_1_sc_2deriv_narrow6_shift2d_own_n20_v07.png'
image_processed1 = zutil.ImageProcessed(img_filepath1, ef_shift=0.02)
pframe11.assign_image(image_processed1.image, extent=image_processed1.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='gnu_v2_r', clim=figs_clim_second_derivative, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath2 = './image_processed_2ppe8621_8_sc_2deriv_narrow6_shift2d_own_n20_v07.png'
image_processed2 = zutil.ImageProcessed(img_filepath2, ef_shift=0.02)
pframe12.assign_image(image_processed2.image, extent=image_processed2.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='gnu_v2_r', clim=figs_clim_second_derivative, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath3 = './image_processed_2ppe8621_13_sc_2deriv_narrow6_shift2d_own_n20_v07.png'
image_processed3 = zutil.ImageProcessed(img_filepath3, ef_shift=0.02)
pframe13.assign_image(image_processed3.image, extent=image_processed3.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='gnu_v2_r', clim=figs_clim_second_derivative, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath4 = './image_processed_2ppe8621_4_sc_2deriv_narrow8_shift2d_own_n20_v07_3.png'
image_processed4 = zutil.ImageProcessed(img_filepath4, ef_shift=0.02)
pframe14.assign_image(image_processed4.image, extent=image_processed4.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='gnu_v2_r', clim=figs_clim_second_derivative, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath5 = './image_processed_2ppe8621_27_sc_2deriv_narrow8_shift2d_own_n20_v07.png'
image_processed5 = zutil.ImageProcessed(img_filepath5, ef_shift=0.02)
pframe15.assign_image(image_processed5.image, extent=image_processed5.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='gnu_v2_r', clim=figs_clim_second_derivative, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath6 = './image_processed_2ppe8621_11_sc_2deriv_narrow8_shift2d_own_n20_v07_2.png'
image_processed6 = zutil.ImageProcessed(img_filepath6, ef_shift=0.02)
pframe16.assign_image(image_processed6.image, extent=image_processed6.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='gnu_v2_r', clim=figs_clim_second_derivative, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)

# load DFT calculation data
data_dft_plot = np.loadtxt('./dft_plot_data_v1.dat', comments=';')
k_plot_dft = data_dft_plot[:, 0]
band_dft_SS1 = data_dft_plot[:, 1]
band_dft_SS2 = data_dft_plot[:, 2]
projection_dft_VB1 = data_dft_plot[:, 3]
projection_dft_VB2 = data_dft_plot[:, 4]
projection_dft_CB1 = data_dft_plot[:, 5]
projection_dft_CB2 = data_dft_plot[:, 6]
band_dft_VB1 = data_dft_plot[:, 7]
band_dft_VB2 = data_dft_plot[:, 8]
band_dft_VB3 = data_dft_plot[:, 9]
band_dft_VB4 = data_dft_plot[:, 10]
band_dft_CB1 = data_dft_plot[:, 11]
band_dft_CB2 = data_dft_plot[:, 12]
band_dft_CB3 = data_dft_plot[:, 13]
band_dft_CB4 = data_dft_plot[:, 14]

# load Floquet center frequency
tmp_filepath = './waveform_params_2ppe8621_shift2d_n60_v19.dat'
data_waveform_params = np.loadtxt(tmp_filepath, comments=';')
freq_fit_parallel = data_waveform_params[5]

# plot DFT band structures
for pframe in [pframe11, pframe12, pframe13, pframe14, pframe15, pframe16]:
    pframe.ax.plot(k_plot_dft, band_dft_SS1, color=color_dft, lw=lw_dft_thick)
    pframe.ax.plot(k_plot_dft, band_dft_SS2, color=color_dft, lw=lw_dft_thick)
    if pframe != pframe16:
        e_Floquet = hb * 2.0 * np.pi * freq_fit_parallel * 1e-3
        # print(e_Floquet)
        for n_Floquet in [-3, -2, -1, 1, 2, 3]:
            pframe.ax.plot(k_plot_dft, band_dft_SS1 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)
            pframe.ax.plot(k_plot_dft, band_dft_SS2 + n_Floquet * e_Floquet, color=color_dft, lw=lw_dft_thin)
    else:
        pframe.ax.fill_between(k_plot_dft, projection_dft_VB1, projection_dft_VB2,
                               facecolor=color_dft_projection, alpha=alpha_dft_projection)
        pframe.ax.fill_between(k_plot_dft, projection_dft_CB1, projection_dft_CB2,
                               facecolor=color_dft_projection, alpha=alpha_dft_projection)
        pframe.ax.plot(k_plot_dft, band_dft_VB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_VB4, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB1, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB2, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB3, color=color_dft, lw=lw_dft_thin)
        pframe.ax.plot(k_plot_dft, band_dft_CB4, color=color_dft, lw=lw_dft_thin)

# set attributes
pframe11.ax.set_ylabel(r'$ϵ-ϵ_{\mathrm{F}}$ (eV)', fontsize=fontsize_main, labelpad=3)
for pframe in [pframe12, pframe13, pframe14, pframe15, pframe16]:
    pframe.ax.tick_params(labelleft=False)
for pframe in [pframe11, pframe12, pframe13, pframe14, pframe15, pframe16]:
    pframe.ax.set_xlabel(r'$k_y$ (Å$^{-1}$)', fontsize=fontsize_main, labelpad=3)
    pframe.ax.hlines(0, image_processed1.kmin, image_processed1.kmax,
                     color='black', lw=0.5, ls=(0, (8, 5)))

list_delay_time = [-60, -50, -40, -18, -12, 120]
for idx, pframe in enumerate([pframe11, pframe12, pframe13, pframe14, pframe15, pframe16]):
    pframe.ax.text(0.5, 0.9, r'$t = $' + '{:.0f} fs'.format(list_delay_time[idx]),
                   transform=pframe.ax.transAxes, horizontalalignment='center',
                   fontsize=fontsize_main, color='black')
    pframe.fix_yaxis_interval(0.1)

# add colorbar
pframe16.add_colorbar(cleft=38.5 * factor, cbottom=8 * factor, cwidth=2 * factor, cheight=25 * factor)
pframe16.modify_colorbar(ticks_fontsize=fontsize_main, ticks_color='black', show_ticks=True)
pframe16.cax.text(0.5, 1.05, 'max', transform=pframe16.cax.transAxes,
                  fontsize=fontsize_colorbar, horizontalalignment='center')
pframe16.cax.text(0.5, -0.12, 'min', transform=pframe16.cax.transAxes,
                  fontsize=fontsize_colorbar, horizontalalignment='center')
pframe16.cax.tick_params(axis='y', right=False, labelright=False)

#######################################################################################################################
# End of Plot structure 1
#######################################################################################################################


#######################################################################################################################
# Plot structure 2 (experimental raw ARPES images)
#######################################################################################################################
# create empty frame for this structure
eframe2 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=1, width=203, height=52)
# eframe2.visualize_frame()

# create plot frames
img_left = 11
img_bottom = 10
factor = 0.85
img_width = 35 * factor  # 35
img_hoffset = 36.75 * factor  # 36.75
img_height = 48 * factor
pframe21 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                             left=img_left, bottom=img_bottom, width=img_width, height=img_height)
pframe21.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='white',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe22 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                             left=img_left + img_hoffset, bottom=img_bottom, width=img_width, height=img_height)
pframe22.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='white',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe23 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                             left=img_left + img_hoffset * 2, bottom=img_bottom, width=img_width, height=img_height)
pframe23.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='white',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe24 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                             left=img_left + img_hoffset * 3, bottom=img_bottom, width=img_width, height=img_height)
pframe24.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='white',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe25 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                             left=img_left + img_hoffset * 4, bottom=img_bottom, width=img_width, height=img_height)
pframe25.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='white',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)
pframe26 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                             left=img_left + img_hoffset * 5, bottom=img_bottom, width=img_width, height=img_height)
pframe26.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='white',
                      ticks_label_xpad=ticks_xpad, ticks_label_ypad=ticks_ypad)

# assign image data
kmin_plot = -0.15
kmax_plot = 0.15
emin_plot = -0.3
emax_plot = 0.35
img_filepath1 = './image_processed_2ppe8621_1_sc_raw_shift2d_own_n20_v07.png'
image_processed1 = zutil.ImageProcessed(img_filepath1, ef_shift=0.02)
pframe21.assign_image(image_processed1.image, extent=image_processed1.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='inferno', clim=figs_clim_raw, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath2 = './image_processed_2ppe8621_8_sc_raw_shift2d_own_n20_v07.png'
image_processed2 = zutil.ImageProcessed(img_filepath2, ef_shift=0.02)
pframe22.assign_image(image_processed2.image, extent=image_processed2.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='inferno', clim=figs_clim_raw, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath3 = './image_processed_2ppe8621_13_sc_raw_shift2d_own_n20_v07.png'
image_processed3 = zutil.ImageProcessed(img_filepath3, ef_shift=0.02)
pframe23.assign_image(image_processed3.image, extent=image_processed3.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='inferno', clim=figs_clim_raw, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath4 = './image_processed_2ppe8621_4_sc_raw_shift2d_own_n20_v07_3.png'
image_processed4 = zutil.ImageProcessed(img_filepath4, ef_shift=0.02)
pframe24.assign_image(image_processed4.image, extent=image_processed4.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='inferno', clim=figs_clim_raw, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath5 = './image_processed_2ppe8621_27_sc_raw_shift2d_own_n20_v07.png'
image_processed5 = zutil.ImageProcessed(img_filepath5, ef_shift=0.02)
pframe25.assign_image(image_processed5.image, extent=image_processed5.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='inferno', clim=figs_clim_raw, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)
img_filepath6 = './image_processed_2ppe8621_11_sc_raw_shift2d_own_n20_v07_2.png'
image_processed6 = zutil.ImageProcessed(img_filepath6, ef_shift=0.02)
pframe26.assign_image(image_processed6.image, extent=image_processed6.get_extent(),
                      xmin=kmin_plot, xmax=kmax_plot, ymin=emin_plot, ymax=emax_plot,
                      colormap='inferno', clim=figs_clim_raw, transpose=True, inversex=True,
                      func_colormap=zutil.util_cmap)

# overlay contour plots
list_idx_contour = [2, 3, 5, 7, 8, 9]
for idx_plot, pframe in enumerate([pframe21, pframe22, pframe23, pframe24, pframe25, pframe26]):
    tmp_filepath = './image_processed_filtered_contour_idx{}_v12.png'.format(list_idx_contour[idx_plot])
    image_processed_tmp = zutil.ImageProcessed(tmp_filepath, ef_shift=0.0)

    img_raw_filtered_tmp = image_processed_tmp.image
    extent_tmp = image_processed_tmp.get_extent()
    img_min_tmp = np.min(img_raw_filtered_tmp)
    img_max_tmp = np.max(img_raw_filtered_tmp)
    factor_contour_lower_limit = 0.0
    n_contour_level = 60
    pframe.ax.set_xlim(kmin_plot, kmax_plot)
    pframe.ax.set_ylim(emin_plot, emax_plot)
    pframe.ax.contour(img_raw_filtered_tmp[:, :].T,
                      levels=np.linspace(img_min_tmp + img_max_tmp * factor_contour_lower_limit,
                                         img_max_tmp,
                                         num=n_contour_level),
                      colors='white', origin='lower',
                      extent=extent_tmp, linewidths=0.25)

# set attributes
pframe21.ax.set_ylabel(r'$ϵ-ϵ_{\mathrm{F}}$ (eV)', fontsize=fontsize_main, labelpad=3)
for pframe in [pframe22, pframe23, pframe24, pframe25, pframe26]:
    pframe.ax.tick_params(labelleft=False)
for pframe in [pframe21, pframe22, pframe23, pframe24, pframe25, pframe26]:
    pframe.ax.set_xlabel(r'$k_y$ (Å$^{-1}$)', fontsize=fontsize_main, labelpad=3)
    pframe.fix_yaxis_interval(0.1)

# add colorbar
pframe26.add_colorbar(cleft=38.5 * factor, cbottom=8 * factor, cwidth=2 * factor, cheight=25 * factor)
pframe26.modify_colorbar(ticks_fontsize=fontsize_main, ticks_color='black', show_ticks=True)
pframe26.cax.text(0.5, 1.05, 'max', transform=pframe26.cax.transAxes,
                  fontsize=fontsize_colorbar, horizontalalignment='center')
pframe26.cax.text(0.5, -0.12, 'min', transform=pframe26.cax.transAxes,
                  fontsize=fontsize_colorbar, horizontalalignment='center')
pframe26.cax.tick_params(axis='y', right=False, labelright=False)

#######################################################################################################################
# End of Plot structure 2
#######################################################################################################################


#######################################################################################################################
# Figure symbols
#######################################################################################################################

mainframe.ax.text(0.5, 105, 'a', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(0.5, 51, 'b', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.set_zorder(1)

#######################################################################################################################
# export image files
#######################################################################################################################

plt.savefig(outdir + '/' + script_name + version + '.png', dpi=160)
# plt.savefig(outdir + '/' + script_name + version + '.png', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.jpg', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.tiff', dpi=600)  # this creates a huge TIFF file!
# cmd = "convert {}.png {}.tiff".format(outdir + '/' + script_name + version, outdir + '/' + script_name + version)
# subprocess.call(cmd.split())  # execute the command from this script
# plt.savefig(outdir + '/' + script_name + version + '.pdf', dpi=600)

plt.show()
