import os
import re
import PIL
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import subprocess

# load homemade python moduels
import absplot_v03 as absplot

# matplotlib backend selection
matplotlib.use('TkAgg')

# file version check
this_file_name = os.path.basename(__file__)
regular_expression = re.compile(r'(fig1_)(\w+)(.py)')
regular_group = regular_expression.match(this_file_name).groups()
script_name = str(regular_group[0])
version = str(regular_group[1])
# print('file version:', version)

# global figure parameters
fontsize_main = 8
fontsize_colorbar = 7
fontsize_symbol = 9

# directory paths
outdir = '.'  # output directory


matplotlib.rcParams['font.family'] = 'Arial'
mainframe = absplot.MainFrame(width=205, height=80)

#######################################################################################################################
# Plot structure 1 (main schematics)
#######################################################################################################################
#  load schematics image and create empty frame for this structure
schematics_width = 180
img_schematics = np.array(PIL.Image.open('./fig1_povray_v5.png'))
height, width = img_schematics[:, :, 0].shape
schematics_height = schematics_width * (height / width)
eframe1 = absplot.EmptyFrame(mainframe=mainframe, left=0, bottom=0,
                             width=schematics_width + 1, height=schematics_height + 1)
# eframe1.visualize_frame()
pframe11 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe1,
                             left=0, bottom=0, width=schematics_width, height=schematics_height)
pframe11.hide_axis()
pframe11.hide_frame()
pframe11.assign_general_image(img_schematics)

# left schematics labels
pframe11.ax.text(0.45, 0.57, r'$t$', fontsize=fontsize_main+1, transform=pframe11.ax.transAxes)
pframe11.ax.text(0.065, 0.45, 'Interband\nexcitations', fontsize=fontsize_main+1, 
                 transform=pframe11.ax.transAxes, ha='center')
pframe11.ax.text(0.405, 0.06, 'Intraband\ncurrents', fontsize=fontsize_main+1, 
                 transform=pframe11.ax.transAxes, ha='center')
pframe11.ax.text(0.225, 0.78, 'Floquet-Bloch\n states', fontsize=fontsize_main+1, 
                 transform=pframe11.ax.transAxes, ha='center')

# right schematics labels
pframe11.ax.text(0.88, 0.79, r'$x: [0\overline{1}10]$', fontsize=fontsize_main, transform=pframe11.ax.transAxes)
pframe11.ax.text(0.792, 0.965, r'$z: [0001]$', fontsize=fontsize_main, transform=pframe11.ax.transAxes)
pframe11.ax.text(0.672, 0.66, r'$y: [2\overline{1}\overline{1}0]$', fontsize=fontsize_main, 
                 transform=pframe11.ax.transAxes)
pframe11.ax.text(0.785, 0.815, r'$\theta$', fontsize=fontsize_main, transform=pframe11.ax.transAxes)
pframe11.ax.text(0.75, 0.865, r'$e^{-}$', fontsize=fontsize_main, transform=pframe11.ax.transAxes, color='blue')
pframe11.ax.text(0.625, 0.55, r'UV', fontsize=fontsize_main, transform=pframe11.ax.transAxes, color='blue')
pframe11.ax.text(0.685, 0.245, r'$E_{\mathrm{MIR}}$', fontsize=fontsize_main+1, 
                 transform=pframe11.ax.transAxes, color='red')
pframe11.ax.text(0.935, 0.66, r'Bi$_\mathrm{2}$Te$_\mathrm{3}$', fontsize=fontsize_main, 
                 transform=pframe11.ax.transAxes)

#######################################################################################################################
# End of Plot structure 1
#######################################################################################################################


#######################################################################################################################
# Plot structure 2 (band structure plot)
#######################################################################################################################
eframe2 = absplot.EmptyFrame(mainframe=mainframe, left=174, bottom=11, width=25, height=30.5)

# load DFT calculation data
data_dft_plot = np.loadtxt('./dft_plot_data_v1.dat', comments=';')
k_plot_dft = data_dft_plot[:, 0]
band_dft_SS1 = data_dft_plot[:, 1]
band_dft_SS2 = data_dft_plot[:, 2]
projection_dft_VB1 = data_dft_plot[:, 3]
projection_dft_VB2 = data_dft_plot[:, 4]
projection_dft_CB1 = data_dft_plot[:, 5]
projection_dft_CB2 = data_dft_plot[:, 6]
band_dft_VB1 = data_dft_plot[:, 7]
band_dft_VB2 = data_dft_plot[:, 8]
band_dft_VB3 = data_dft_plot[:, 9]
band_dft_VB4 = data_dft_plot[:, 10]
band_dft_CB1 = data_dft_plot[:, 11]
band_dft_CB2 = data_dft_plot[:, 12]
band_dft_CB3 = data_dft_plot[:, 13]
band_dft_CB4 = data_dft_plot[:, 14]

pframe21 = absplot.PlotFrame(mainframe=mainframe, parentframe=eframe2,
                             left=0.0001, bottom=0.0001, width=25,
                             height=30.5)
pframe21.modify_ticks(ticks_fontsize=fontsize_main, ticks_color='black', ticks_length=3)

# plot DFT band structures
pframe21.ax.plot(k_plot_dft, band_dft_SS1, color='gray', lw=1)
pframe21.ax.plot(k_plot_dft, band_dft_SS2, color='gray', lw=1)
pframe21.ax.fill_between(k_plot_dft, projection_dft_VB1, projection_dft_VB2, color='gray', alpha=0.3)
pframe21.ax.fill_between(k_plot_dft, projection_dft_CB1, projection_dft_CB2, color='gray', alpha=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_VB1, color='gray', lw=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_VB2, color='gray', lw=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_VB3, color='gray', lw=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_VB4, color='gray', lw=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_CB1, color='gray', lw=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_CB2, color='gray', lw=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_CB3, color='gray', lw=0.3)
pframe21.ax.plot(k_plot_dft, band_dft_CB4, color='gray', lw=0.3)

kmin = -0.2
kmax = 0.2
emin = -0.33
emax = 0.32
pframe21.ax.set_ylim(emin, emax)
pframe21.ax.set_xlim(kmin, kmax)
pframe21.ax.hlines(0, kmin, kmax, color='black', lw=0.4, ls=(0, (8, 5)))
pframe21.ax.set_ylabel(r'Energy, $ϵ$', fontsize=fontsize_main, labelpad=1)
pframe21.ax.set_xlabel(r'Momentum, $k_y$', fontsize=fontsize_main, labelpad=3)
pframe21.ax.text(0.5, 1.06, r'$\overline{\Gamma}$', fontsize=fontsize_main,
                 horizontalalignment='center',
                 verticalalignment='center',
                 transform=pframe21.ax.transAxes)
pframe21.ax.text(0.1, 1.06, r'$\overline{\mathrm{K}}\leftarrow$', fontsize=fontsize_main,
                 horizontalalignment='center',
                 verticalalignment='center',
                 transform=pframe21.ax.transAxes)
pframe21.ax.text(0.9, 1.06, r'$\rightarrow\overline{\mathrm{K}}$', fontsize=fontsize_main,
                 horizontalalignment='center',
                 verticalalignment='center',
                 transform=pframe21.ax.transAxes)

pframe21.ax.text(1.05, 0.475, r'$ϵ_{\mathrm{F}}$', fontsize=fontsize_main, transform=pframe21.ax.transAxes)
pframe21.ax.text(0.5, 0.41, r'TSS', color='black', fontsize=fontsize_main, transform=pframe21.ax.transAxes, 
                 horizontalalignment='center')
pframe21.ax.text(0.5, 0.74, r'BCB', color='black', fontsize=fontsize_main, transform=pframe21.ax.transAxes, 
                 horizontalalignment='center')
pframe21.ax.text(0.5, 0.07, r'BVB', color='black', fontsize=fontsize_main, transform=pframe21.ax.transAxes, 
                 horizontalalignment='center')
eframe2.ax.set_zorder(1)

#######################################################################################################################
# End of Plot structure 2
######################################################################################################################


#######################################################################################################################
# Figure symbols
#######################################################################################################################

mainframe.ax.text(2, 75, 'a', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.text(98, 75, 'b', weight='bold', fontsize=fontsize_symbol)
mainframe.ax.set_zorder(1)

#######################################################################################################################
# export image files
#######################################################################################################################

plt.savefig(outdir + '/' + script_name + version + '.png', dpi=160)
# plt.savefig(outdir + '/' + script_name + version + '.png', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.jpg', dpi=600)
# # plt.savefig(outdir + '/' + script_name + version + '.tiff', dpi=600)  # this creates a huge TIFF file!
# cmd = "convert {}.png {}.tiff".format(outdir + '/' + script_name + version, outdir + '/' + script_name + version)
# subprocess.call(cmd.split())  # execute the command from this script
# plt.savefig(outdir + '/' + script_name + version + '.pdf', dpi=600)

plt.show()
